/**
 * @file    development.h
 * @brief   Documentation: How to contribute and join WOSH
 ****************************************************************************
 * @author  Alessandro Polo
 * @version 0.8.499 $Id: development.h 2597 2010-05-29 00:54:19Z alex $
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

/*! \page page_development Development
 *
 * This information is intended primarily for WOSH developers (framework and extensions both).
 *
 * \section page_development_toc Table of Contents:
 *
 *  - \ref page_development_overview
 *
 *  - \ref page_development_join
 *  - \ref page_development_report
 *
 *  - \ref page_releasenotes
 *
 * Also comments and tips are very welcome.
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_development_overview Overview
 *
 * WOSH is architected to scale from tiny applications all the way up
 * to massive applications consisting of hundreds of components.
 *
 * That means, the framework helps developers to build software systems consisting of modules which are communicating,
 * via clearly defined service interfaces, with other modules of the system.
 *
 *  - \ref page_specifications "Specifications of WOSH"
 *
 *  - \ref page_core "WOSH Kernel and Core Modules"
 *  - \ref page_framework "WOSH Framework"
 *  - \ref page_bundles "Bundle (generic service)"
 *
 *  - \ref page_build "Compiling and Building WOSH"
 *
 * You may take a look to \ref woshsrv.cpp "woshsrv's source code".
 *
 * New developers are welcome to:
 *  - Contribute to the core library
 *  - Write bundles for WOSH
 *  - Write applications using WOSH
 *  - Contribute to the documentation
 *  - Build test cases
 *
 * Before coding, discuss your plans and design with other people by posting detailed messages.
 * By all means, plan everything before you do it, and share information.
 * Everyone working on this project has some level of expertise and previous coding experiences
 * which can help us all from falling on our faces more than necessary.
 * When you come across a problem don't bang your head against the wall for too long.
 *
 * Ask for help: perhaps someone has faced a similar problem in the past and can offer an immediate solution.
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_development_join Join WOSH
 *
 * Join online project (<a href="https://sourceforge.net/projects/wosh/">SourceForge</a> or
 * <a href="https://launchpad.net/wosh">LaunchPad</a>) or mail me at contact _AT_ alessandropolo.name
 * If you are interested in taking on a development task, first make sure that
 * everyone involved in the development effort knows your interest and intention.
 * This is appropriately done with a request to the developers, either by asking specifically 
 * to work on one of the sub-projects or tasks, or by describing your new, proposed task.
 * Once agreement is reached about the requirements, goals, the project administrators should then assign this task to you.
 *
 * \see <a href="https://sourceforge.net/people/?group_id=274637" title="list of positions available for WOSH Framework">Help Wanted on SourceForge</a>
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_development_report Report Bugs
 *
 * Sometimes the problem you encounter may look like a bug rather than a problem
 * with the documentation or a generic request for help. If this is the case,
 * please use the bug reporting service offered by SourceForge.
 * First check the documentation to make sure you are using WOSH as it is expected to be used.
 * Then check to see if anyone else has submitted the same bug (or possibly a tech support request)
 * that may affect your problem. If not, please submit the bug and give as much detail as possible
 * so that the person reading this bug will be able to recreate the situation.
 * Please specify the command line options you are using, as well as provide any source code which may be useful to reproduce the bug.
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_development_guide_lines Guide Lines for developers
 *
 * Enables developers to understand what they are doing relative to the whole project
 *  - Handles the projects current needs
 *  - Is stable as more needs are added to it
 *  - Isolates parts of the system from changes in other parts
 *  - Allows increased benefits (reuse) as the system grows
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_development_metrics Software Metrics
 *
Blank Lines 25.235
Classes 617
Code Lines 95.660
Comment Lines 44.277
Comment to Code Ratio 0,46
Declarative Statements 23.924
Executable Statements 43.287
Files 705
Functions 7.926
Inactive Lines 1.540
Lines 169.195

 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 *
 */ 
