/**
 * @file    distribution.h
 * @brief   Documentation: How WOSH Framework is distributed (released).
 ****************************************************************************
 * @author  Alessandro Polo
 * @version 0.8.499 $Id: distribution.h 2251 2010-03-29 08:32:31Z alex $
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

/*! \page page_distribution Distribution
 *
 * This page explains how to get and install WOSH on your system.
 *
 * Once downloaded you may check \ref page_run_req "requirements" for \ref page_run_run "running"
 * (binaries) or \ref page_build_req "requirements" for \ref page_build "building" (compiling source).
 *
 * Note that before running WOSH for first time you need to \ref page_config "configure" the software
 * and included bundles.
 *
 * \section page_distribution_toc Table of Contents:
 *
 *  - \ref page_distribution_overview
 *
 *  - \ref page_distribution_source
 *  - \ref page_distribution_filesystem
 *
 *  - \ref  page_distribution_full
 *
 *  - \ref page_distribution_posix
 *  - \ref page_distribution_windows
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_distribution_overview Overview
 *
 * \warning
 *  WOSH is still in development stage, binaries are provided as demonstration.
 *  Target audience are developers and expert (or curious) users.
 *
 * WOSH (official) website address is <a href="http://wosh.sourceforge.net">http://wosh.sourceforge.net</a>.
 *
 * Main file repository (the first updated) is hosted on SourceForge as
 * \b <a href="https://sourceforge.net/projects/wosh/">WOSH project</a> at 
 * <a href="https://sourceforge.net/projects/wosh/files/">https://sourceforge.net/projects/wosh/files/</a>.
 *
 * Other mirrors and websites are:
 *  - <a href="https://launchpad.net/wosh">https://launchpad.net/wosh</a>
 *  - <a href="http://freshmeat.net/projects/wosh-framework">http://freshmeat.net/projects/wosh-framework</a>
 *  - <a href="http://code.google.com/p/wosh/">http://code.google.com/p/wosh</a>
 *  - <a href="http://www.opensmarthome.com">http://www.opensmarthome.com</a>
 *  - <a href="http://www.alessandropolo.name/projects/WOSH">http://www.alessandropolo.name/projects/WOSH</a>
 *
 * WOSH may be installed anywhere, file-system structure is similar to POSIX system
 * (because it's planned to have WOSH fully integrated into Linux).
 *
 * The installation path should not contain any spaces.
 *
 * Binaries are located in <a href="../../bin">/bin</a> folder, by default
 * configuration files are loaded from <a href="../../etc/wosh">/etc/wosh</a>.
 * \see \ref page_config for more information.
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_distribution_source Source Code
 *
 * Developers should access to latest source from the SourceForge repository (SubVersion):
 * \verbatim
alex@linbox:/home/alex/MyWoshFolder$ svn co https://wosh.svn.sourceforge.net/svnroot/wosh wosh 
\endverbatim
 *
 * The repository is mirrored (synchronized) on <a href="https://launchpad.net/wosh/trunk" title="automatic synchronization against main repository">LaunchPad Bazaar</a>.
 *
 * You may also browse the repository online at
 * <a href="http://wosh.svn.sourceforge.net/viewvc/wosh/">http://wosh.svn.sourceforge.net/viewvc/wosh/</a>
 *
 * On Windows, I suggest to use TortoiseSVN (<a href="http://tortoisesvn.tigris.org">http://tortoisesvn.tigris.org</a>).
 *
 * Just create a new folder, right-click and \c CheckOut the address \b https://wosh.svn.sourceforge.net/svnroot/wosh
 *
 * To contribute the project you need to be registered on SourceForge (or LaunchPad) and join the WOSH project.
 *
 * You are welcome to contact project administrators.
 *
 * See page \ref page_build for information about compiling WOSH on your system.
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_distribution_filesystem WOSH File-System
 *
 * WOSH may be installed anywhere, also in a network-shared folder.
 *
 * You may notice similarities with Linux FS, in fact file system is designed
 * to be (further) fully integrated in a generic POSIX system (as package).
 *
 * Let's see sub-folders:
 *
 *    - <em>WOSH root folder</em>
 *      - <a href="../../bin">bin</a>
 *      - <a href="../../build">build</a> [<a href="../../build/README">readme</a>]
 *        - <a href="../../build/deb-package">deb-package</a>
 *        - <a href="../../build/msi-package">msi-package</a>
 *      - <a href="../../doc">doc</a> [<a href="../../doc/README">readme</a>]
 *        - <a href="../../doc/diagrams">diagrams</a> [custom diagrams of the documentation]
 *        - <a href="../../doc/html">html</a> [output folder of Doxygen]
 *      - <a href="../../etc">etc</a> [ contains configuration files ]
 *        - <a href="../../etc/wosh">wosh</a> [<a href="../../etc/wosh/README">readme</a>]
 *          - <a href="../../etc/wosh/bundles-available">bundles-available</a>
 *          - <a href="../../etc/wosh/bundles-enabled">bundles-enabled</a>
 *          - <a href="../../etc/wosh/scripts">scripts</a>
 *      - <a href="../../lib">lib</a> [<a href="../../lib/README">readme</a>]
 *        - <a href="../../lib/3rdparty">3rdparty</a> [<a href="../../src/3rdparty/README">readme</a>]
 *      - <a href="../../src">src</a>
 *        - <a href="../../src/apps">apps</a> [<a href="../../src/apps/README">readme</a>]
 *          - <a href="../../src/apps/woshcesrv">woshcesrv</a> [ \ref page_applications_woshcesrv ]
 *          - <a href="../../src/apps/woshkiosk">woshkiosk</a> [ \ref page_applications_woshkiosk ]
 *          - <a href="../../src/apps/woshremote">woshremote</a> [ \ref page_applications_woshremote ]
 *          - <a href="../../src/apps/woshshop">woshshop</a> [ \ref page_applications_woshshop ]
 *          - <a href="../../src/apps/woshsrv">woshsrv</a> [ \ref page_applications_woshsrv ]
 *        - <a href="../../src/bundles">bundles</a> [<a href="../../src/bundles/README">readme</a>]
 *          - <a href="../../src/bundles/devices">devices</a> [contains various Bundles]
 *          - <a href="../../src/bundles/services">services</a> [contains various Bundles]
 *        - <a href="../../src/core">core</a>
 *          - <a href="../../src/core/posix">posix</a> [POSIX scoped implementations]
 *          - <a href="../../src/core/utils">utils</a> [multiplatform 3rd party source]
 *            - <a href="../../src/core/utils/md5-cc">md5-cc</a>
 *            - <a href="../../src/core/utils/mtrand">mtrand</a>
 *          - <a href="../../src/core/win32">win32</a> [Windows32 scoped implementations]
 *          - <a href="../../src/core/wince">wince</a> [WindowsCE scoped implementations]
 *        - <a href="../../src/framework">framework</a>
 *          - <a href="../../src/framework/ai">ai</a> [ namespace wosh::ai ]
 *          - <a href="../../src/framework/automations">automations</a> [ namespace wosh::automations ]
 *          - <a href="../../src/framework/building">building</a> [ namespace wosh::building ]
 *          - <a href="../../src/framework/communication">communication</a> [ namespace wosh::communication ]
 *          - <a href="../../src/framework/database">database</a> [ namespace wosh::database ]
 *          - <a href="../../src/framework/entertainment">entertainment</a> [ namespace wosh::entertainment ]
 *          - <a href="../../src/framework/gui">gui</a> [ namespace wosh::gui ]
 *          - <a href="../../src/framework/network">network</a> [ namespace wosh::network ]
 *          - <a href="../../src/framework/plugins">plugins</a>
 *          - <a href="../../src/framework/security">security</a> [ namespace wosh::security ]
 *          - <a href="../../src/framework/xtended">xtended</a> [ namespace wosh::xtended ]
 *        - <a href="../../src/interfaces">interfaces</a> [<a href="../../src/interfaces/README">readme</a>]
 *          - <a href="../../src/interfaces/devices">devices</a>
 *          - <a href="../../src/interfaces/services">services</a>
 *        - <a href="../../src/testsuite">testsuite</a> [<a href="../../src/testsuite/README">readme</a>]
 *        - <a href="../../src/tools">tools</a> [<a href="../../src/tools/README">readme</a>]
 *      - <a href="../../tmp">tmp</a> <em>* may point to a different location</em>
 *      - <a href="../../var">var</a>
 *        - <a href="../../var/database">database</a> [ permanent storage, XML format by default ]
 *          - <a href="../../var/database/alarm">alarm</a>
 *          - <a href="../../var/database/automations">automations</a>
 *          - <a href="../../var/database/building">building</a>
 *          - <a href="../../var/database/cronjobs">cronjobs</a>
 *          - <a href="../../var/database/entertainment">entertainment</a>
 *          - <a href="../../var/database/users">users</a>
 *        - <a href="../../var/extras">extras</a>
 *          - <a href="../../var/extras/graph">graph</a>
 *          - <a href="../../var/extras/icons">icons</a>
 *          - <a href="../../var/extras/screenshots">screenshots</a>
 *          - <a href="../../var/extras/sounds">sounds</a>
 *        - <a href="../../var/log">log</a> [ output folder of applications' log ]
 *        - <a href="../../var/man">man</a>
 *        - <a href="../../var/stylesheets">stylesheets</a>
 *
 * WOSH won't really need all of these folders, a minimal installation requires
 * only binary and its configuration files.
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_distribution_full Full distribution [multi-platform]
 *
 * Full distribution of WOSH comes with:
 *  - Source code (core, framework, bundles, documentation)
 *  - HTML documentation
 *  - Binaries
 *  - Extras (located in /var/extras)
 *
 * It is provided as \c tar.gz of \c ZIP archive.
 *
 *
 * Open a shell and change your preferred folder, run \c tar to uncompress data
 * (will also create a subfolder):
 * \verbatim
alex@linbox:/home/alex/MyWoshFolder$ tar -xfv WOSH-full-<UTC-DATE>.tar.gz
\endverbatim
 * or \c unzip the archive executing:
 * \verbatim
alex@linbox:/home/alex/MyWoshFolder$ unzip WOSH-full-<UTC-DATE>.zip
\endverbatim
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_distribution_posix POSIX (Linux) distribution
 *
 * \warning
 *  WOSH POSIX distribution is not standardized yet, refer to \ref page_distribution_full
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_distribution_windows Windows (32) distribution (+ Windows CE/Mobile)
 * 
 * WOSH is distributed as Microsoft Installer Package (msi) and source code is not included.
 * Usually, Windows(32) distribution comes with Windows Mobile/CE (binary) archive
 * (in <a href="../../bin">/bin</a> folder).
 *
 * \warning Windows CE/Mobile deploy is still not standardized.
 * 
 * Windows Mobile release is distributed as ZIP archive, containing only binaries and required DLLs.
 * It usually ships with Windows installer and Full distribution.
 *
 * Installation on Windows Mobile/CE device from Windows 2000/XP:
 *  - Open Microsoft ActiveSynch
 *  - Have the device connected (PAN network enabled)
 *  - Explore file system, browsing to target directory (Programs/wosh)
 *  - Copy all files from the WinCE archive into target folder on device
 *  - Run the executable file from device
 *
 * On Linux platform you need to setup RNDIS driver and have device connected
 * as a (virtual) host (TCP-IP enabled).
 * You may find some information on \ref page_applications_woshcesrv "wincesrv page".
 *
 *
 ****************************************************************************
 *
 */

