/**
 * @class   wosh::application::WoshKioskWindow
 * @brief   
 *
 *
 ****************************************************************************
 * @version $Id: WoshKioskWindow.h 2283 2010-04-07 06:59:53Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Application_WoshKioskWindow_H__
 #define __WOSH_Application_WoshKioskWindow_H__


 #include <core/Object.h>
 #include <core/Bus.h>
 #include <core/BusConnector.h>
 #include <core/LogContext.h>
 #include <core/PropertiesProvider.h>

 #include <QtGui/QGraphicsView>

 #include <framework/gui/kiosk/KioskManager.h>


using namespace wosh::gui;
namespace wosh {
 namespace application {

#define _APPLICATION_WOSH_KIOSK_NAME		"WOSH KiosK"
#define _APPLICATION_WOSH_KIOSK_VERSION		"0.0.11"

class WoshKioskWindow :		public QGraphicsView,
							public virtual IBusListener,
							public virtual IReflection
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::application::WoshKioskWindow)
	WOSH_CLASS_ABSTRACT(false)


	public:
		WoshKioskWindow();
		virtual ~WoshKioskWindow();


	public:
		void init();
		void quitWosh();

		void refresh();

		void setManager( KioskManager* manager );


//@}
/** @name IBusListener interface
 * @{
 ******************************************************************************/
 	public:
		void busMessage( const Message& message, const Bus* source );
//@}
/** @name Internal QT events
 * @{
 ******************************************************************************/
 	protected:
		void changeEvent(QEvent *e);
		void closeEvent(QCloseEvent *event);

/** @name Configuration methods
 * @{
 ******************************************************************************/
 	protected:
		void readSettings();
		void writeSettings();

		void loadLanguage(const QString& language);
		void loadStyleSheet(const QString &sheetName);

//@}
/** @name GUI and Layout
 * @{
 ******************************************************************************/
 	protected:

//@}
/** @name WOSH fields
 * @{
 ******************************************************************************/
	protected:
		URI myUri;
		LogContext Log;
		BusConnector BusCore;
		PropertiesProvider Properties;

		bool reallyExiting;

		KioskManager* kManager;

		static WoshKioskWindow* instance;
		static WoshKioskWindow* getInstance()		{ return WoshKioskWindow::instance; }

}; // class def

  }; // namespace application
}; // namespace wosh

#endif //__WOSH_Application_WoshKioskWindow_H__
