/**
 * @class   wosh::application::WoshShopWindow
 * @brief   
 *
 * @version 0.6.614 $Id: WoshShopWindow.h 2798 2010-07-23 15:41:05Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Apps_WoshTrayServer_WoshShopWindow_H__
 #define __WOSH_Apps_WoshTrayServer_WoshShopWindow_H__

 #include <woshDefs.h>
 #include <core/BusConnector.h>

 #include <framework/gui/components/WidgetQMenu.h>
 #include <framework/gui/kiosk/KioskGadget.h>

// #include <bundles/services/DesktopNotifier/DesktopNotifierBundle.h>

 #include <QtGui/QMainWindow>
 #include <QSystemTrayIcon>
 #include <QSignalMapper>
 #include <QLabel>

using namespace wosh;
using namespace wosh::gui;

class QMdiArea;
namespace Ui {
 class WoshShopWindow;
};
namespace wosh {
 namespace gui {
  class WidgetWoshBrowser;
 };
 namespace services {
  class DesktopNotifierBundle;
 };
 namespace application {

#define _APPLICATION_WOSH_WORKSHOP_NAME			"WOSH WorkShop"
#define _APPLICATION_WOSH_WORKSHOP_VERSION		"0.6.801"

void showWidget_WoshShopWindow( QWidget* widget );

class WoshShopWindow :	public QMainWindow,
						public virtual IBusListener,
						public virtual IReflection
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::application::WoshShopWindow)
	WOSH_CLASS_ABSTRACT(false)


	public:
		WoshShopWindow( QWidget *parent = NULL );

		virtual ~WoshShopWindow();

		void init();

		static WoshShopWindow* getInstance()		{ return WoshShopWindow::instance; }

	protected:
		void changeEvent(QEvent *e);

	public:
		void busMessage( const Message& message, const Bus* source );


/** @name Gui Allocator(s)
 * @{
 ******************************************************************************/
	public:
		//void add2Workspace( QWidget* widget );
		void add2DockBar( QWidget* widget, Qt::DockWidgetArea area );

		QMdiArea* getDocumentArea();

	public slots:
		void create_GUI( const string& name, const string& type, const URI& uri, const QStringList& interfaces = QStringList() );
		void create_GADGET( const string& name, const string& type, const URI& uri );

		void quitWosh();

//@}
/** @name configuration methods
 * @{
 ******************************************************************************/
 	protected:
		void readSettings();
		void writeSettings();

		void loadStyleSheet( const QString &sheetName );
		void loadLanguage(const QString& language);

		void refreshInfo();

//@}
/** @name menu handlers
 * @{
 ******************************************************************************/
 	protected slots:

		void menu_StaticItem_triggered( QAction* action );
		void menu_CreateGUI_triggeredItem( WidgetQMenuItem * actionItem );
		void menuWOSH_Kernel_triggeredItem( WidgetQMenuItem* actionItem );

		void createTrayIcon();
		void createMenus();
		void configureStatusBar();

		void menu_Modules_update_ToolBar();

		void updateDynamicMenus();
		void updateWindowMenu();
		void updateStatusBar();


//@}
/** @name Tray Icon methods
 * @{
 ******************************************************************************/
	private slots:
		void showMessage( const QString& title, const QString& message, long timeout, QSystemTrayIcon::MessageIcon icon = QSystemTrayIcon::Information );

	public:
		QSystemTrayIcon* getTrayIcon()		{ return this->trayIcon; }
		void setDesktopNotifierBundle( wosh::services::DesktopNotifierBundle* ptr )
											{ this->desktopNotifierBundle = ptr; }

//@}
/** @name Tray Icon vars
 * @{
 ******************************************************************************/
	protected:
		QSystemTrayIcon *trayIcon;

		QMenu *trayIconMenu;
		QAction *minimizeAction;
		QAction *maximizeAction;
		QAction *restoreAction;
		QAction *quitAction;

		wosh::services::DesktopNotifierBundle* desktopNotifierBundle;

//@}

	signals:
		void s_DesktopNotifier(int state);
		void s_showMessage( const QString& title, const QString& message, long timeout, QSystemTrayIcon::MessageIcon icon );

	protected:
		QSignalMapper* windowMapper;

	private:
		Ui::WoshShopWindow *m_ui;

		static WoshShopWindow* instance;
		friend void showWidget_WoshShopWindow( QWidget* widget );

	protected:
		// status bar items
		QLabel* statusUserAtKernelLbl;
		QLabel* statusConnections;

	protected:
		// static forms
		QWidget* formScreenshoter;
		QWidget* formConfiguration;
		WidgetWoshBrowser* formWoshBrowser;
		QWidget* formLogViewer;

	protected:
		BusConnector BusGui;
		URI myUri;
		LogContext Log;


}; // class def

  }; // namespace application
}; // namespace wosh

#endif //__WOSH_Apps_WoshTrayServer_WoshShopWindow_H__
