/**
 * @class   JabberGloox09
 * @brief   Wrapper for \c gloox library version 0.9.xx. Extends common class JabberGlooxImpl.
 *
 * \b gloox library is available on APT repository of Debian-based system:
 * \verbatim
$ apt-get install libgloox libgloox-dev
\endverbatim
*
 * QMake/g++ configuration:
 * @code
 *  INCLUDEPATH += /usr/include
 *  LIBS += -L/usr/lib
 *  LIBS += -lgloox
 * @endcode
 *
 * @see     JabberGlooxImpl
 * @see     wosh::devices::JabberGlooxBundle
 * @see     http://camaya.net/gloox/
 * @see     http://code.google.com/apis/talk/libjingle 
 ****************************************************************************
 * @version 0.8.499 $Id: JabberGloox09.h 2737 2010-07-07 00:54:33Z alex $
 * @date    Released Stable on 2010-02-21
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Devices_JabberGloox_09_H__
 #define __WOSH_Bundles_Devices_JabberGloox_09_H__

// #include <core/BundleGenericWorker.h>
// #include <core/BusConnector.h>
 #include "JabberGlooxImpl.h"


class JabberGloox09 : public JabberGlooxImpl
 {
	public:
		/**
		 * @brief   Default constructor. Init vars.
		 * @param bundle [in] the parent bundle, see BundleGenericWorker interface
		 */
		JabberGloox09( wosh::BundleGeneric& bundle );
		/**
		 * @brief   Destructor. If thread is stll running, it will be stopped.
		 */
		virtual ~JabberGloox09();

	public:
		bool sendMessage( const std::string& user, const std::string& message );
		bool setPresence( bool available, bool force = false );

/** @name gloox::MessageHandler Interface
 * @{
 ******************************************************************************/
	public:
		void handleMessage( gloox::Stanza* stanza ) { handleMessage(stanza, NULL); }
		void handleMessage( gloox::Stanza* stanza, gloox::MessageSession* );
//@}
/** @name gloox::PresenceHandler Interface
 * @{
 ******************************************************************************/
	public:
		void handlePresence( gloox::Stanza* stanza );
//@}
/** @name gloox::SubscriptionHandler Interface
 * @{
 ******************************************************************************/
	public:
		void handleSubscription( gloox::Stanza* stanza );
//@}


}; // class def


#endif //__WOSH_Bundles_Devices_JabberGloox_09_H__
