/*\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
\ SendEmail.h,  Declaration for SendEmail class and SendEmailSimple function
\ Created:  2002-02-01
/ Version:	1.1, LastUpdate: 2007-01-01
\
/ Author:	David Maisonave
\			http://www.axter.com
/           Top ten expert at the Expert Exchange
\			http://www.experts-exchange.com/Programming/Programming_Languages/Cplusplus/
/
\
/ Please send comments and bugfixes to the above email address.
\
/ This code is provided AS IS, you use it at your own risk! 
\ You may use it for whatever you want.
/
\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\*/

/*
For example usage of the SendEmail class, see the bottom of the SendEmail.cpp file

Example usuage for SendEmailSimple function:
	int main(int , char*)
	{
		const char* Hostname = "axter_x.com";
		const char* UserID = "axter_x";
		const char* To = "maisonave@axter_x.com";
		const char* Subject = "This is a test email";
		const char* Body = "Test body";
		const char* UserPassword = "abc123";
		SendEmailSimple(Hostname, UserID, To, Subject, Body, UserPassword);
		return 0;
	}	
*/


#ifndef SENDEMAIL_HEADER_GAURD
#define SENDEMAIL_HEADER_GAURD

#define _CRT_SECURE_NO_WARNINGS

#if  defined(WIN32) || defined(__WIN32__)

	//For VC++ ******************************
	#ifdef _MSC_VER
		#include <afxsock.h>
		#pragma warning(disable:4786)
	#endif //_MSC_VER
	//For Borland C++ ******************************
	#ifdef __BORLANDC__  
		#include <winsock2.h>
		#pragma option -x	
	#endif //__BORLANDC__
	//For GNU C++ ******************************
	#ifdef __GNUC__
		#include <winsock.h>
	#endif //__GNUC__

#else //defined(WIN32) || defined(__WIN32__)
	//For Unix and Linux ******************************

	typedef unsigned int           SOCKET; 

	#include <string.h>
	#include <stdlib.h>
	#include <sys/types.h>
	#include <sys/socket.h>
	#include <netdb.h>
	#include <netinet/in.h>

	#include <sys/socketvar.h>
	#include <arpa/inet.h>

	#define SOCKET_ERROR -1
	#define INVALID_SOCKET -1

#endif //defined(WIN32) || defined(__WIN32__)

#include <string>
#include <vector>
#include <fstream>
#include <list>
#include <time.h>
#include <ctype.h>

#if defined(__GNUC__) && __GNUC__  < 3  && __GNUC_MINOR__ < 96
#include <strstream> 
class StrngStream
{
public:
	StrngStream():m_stream(new std::ostrstream){}
	~StrngStream(){delete m_stream;}
	StrngStream& operator<<(const char* Src)
	{
		*m_stream << Src;
		return *this;
	}
	StrngStream& operator<<(const std::string &Src)
	{
		*m_stream << Src.c_str();
		return *this;
	}
	std::string str() const
	{
		return std::string(m_stream->str(), m_stream->str()+m_stream->pcount());
	}
	void str(const char *Src)
	{
		if (Src)
		{
			delete m_stream;
			m_stream = new std::ostrstream;
		}
	}
private:
	std::ostrstream *m_stream;
};
#else //defined(__GNUC__) && __GNUC__  < 3  && __GNUC_MINOR__ < 96
#include <sstream> 
typedef std::ostringstream StrngStream;
#endif //defined(__GNUC__) && __GNUC__  < 3  && __GNUC_MINOR__ < 96



class SendEmail 
{//SendEmail by David Maisonave (david@axter.com)
public:
	enum {SMTP_OUTGOING_MAIL_PORT = 25, BASE64_MAXLINE  = 76, MAX_LINE_SIZE = 4094, AUTHENTICATION_PORT = 113};
	enum LoginMethod{LOGIN_AUTH_METHOD, LOGIN_PLAIN_METHOD, LOGIN_CRAM_MD5_METHOD};
	SendEmail(
		std::string EmailSenderHostName, 
		std::string Auth_UserID, 
		std::string Auth_Password = "", 
		std::string Sender = "", 
		std::string ReplyToAddr = "", 
		LoginMethod LoginMethod_ = LOGIN_AUTH_METHOD, 
		std::string X_Mailer = "X-Mailer: SendEmail By David Maisonave\r\n", 
		std::string X_Phone = "",
		std::string EmailAddrSeperator = ", ",
		int ServerPortNo = SMTP_OUTGOING_MAIL_PORT);
	~SendEmail();
	std::string operator()(
		const std::list<std::string>	&To, 
		const std::string	&Subject,
		const std::string	&Body,
		const std::list<std::string>	*CC_List = 0, 
		const std::list<std::string>	*BCC_List = 0, 
		const std::list<std::string>	*AttachmentPath_List = 0,
		const std::string *MessageID = 0);
	std::string operator()(
		const std::string	&To, 
		const std::string	&Subject,
		const std::string	&Body,
		const std::list<std::string>	*CC_List = 0, 
		const std::list<std::string>	*BCC_List = 0, 
		const std::list<std::string>	*AttachmentPath_List = 0,
		const std::string *MessageID = 0);
	std::string Mail(
		const std::list<std::string> &To, 
		const std::string	&Subject,
		const std::string	&Body,
		const std::list<std::string>	&CC_List , 
		const std::list<std::string>	&BCC_List , 
		const std::list<std::string>	*AttachmentPath_List /* = 0 */,
		const std::string *MessageID = 0);
	std::string Mail(
		const std::list<std::string> &To, 
		const std::string	&Subject,
		const std::string	&Body,
		const std::list<std::string>	*CC_List = 0, 
		const std::list<std::string>	*BCC_List = 0, 
		const std::list<std::string>	*AttachmentPath_List = 0,
		const std::string *MessageID = 0);
	std::string Mail(
		const std::string	&To, 
		const std::string	&Subject,
		const std::string	&Body,
		const std::list<std::string>	*CC_List = 0, 
		const std::list<std::string>	*BCC_List = 0, 
		const std::list<std::string>	*AttachmentPath_List = 0,
		const std::string *MessageID = 0);

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
std::string CheckMail(
					 const std::string EmailSenderHostName, 
					 const std::string Auth_UserID, 
					 const std::string Auth_Password = "", 
					 LoginMethod LoginMethod_ = LOGIN_AUTH_METHOD);
					
	void DoLogin(void);
	void SetServerPortNo(int PortNo){m_HostServerPort = PortNo;}
	void AppendHostNameToEmailAddress(bool True_){m_AppendHostNameToEmailAddress = True_;}
	static const char* NO_ERRORS;
	const std::vector<std::string> &RecentErrorLog;
private:
	std::list<std::string>* StrLstToELst(const std::list<std::string> *Src, std::list<std::string> &Dest);
	enum{TO_BLOCK, CC_BLOCK, BCC_BLOCK};
	std::string Mail(
		const std::string	&RCPT_To, 
		const std::list<std::string>	&To, 
		const std::string	&Subject,
		const std::string	&Body,
		const std::list<std::string> &CC_List, 
		const std::list<std::string>	*AttachmentPath_List,
		const std::string &MessageID);
	std::vector<std::string> m_RecentErrorLog;
	LoginMethod m_LoginMethod;
	std::string m_EmailSenderHostName;
	int m_HostServerPort;
	bool m_AppendHostNameToEmailAddress;
	std::string m_Sender;
	std::string m_ReplyToAddr;
	std::string m_Auth_UserID;
	std::string m_Auth_Password;
	std::string m_EmailAddrSeperator;
	SOCKET m_socket;
	bool m_InitiateGood;
	char m_EmailBuffer[MAX_LINE_SIZE + 1];
	const std::string m_X_Mailer;
	const std::string m_X_Phone;
	int m_UniqueContentID;
	bool m_UseReplyToAddr_ForMailFrom;
	std::string ReturnValueFromReceive;
	static const char* m_base64Table;
	static const char* CHK_220_REPLY;
	static const char* CHK_221_REPLY;
	static const char* CHK_235_REPLY;
	static const char* CHK_250_REPLY;
	static const char* CHK_334_REPLY;
	static const char* CHK_354_REPLY;
	
	static const char* CHK_plusOK_REPLY;

	
	static const char* EMAIL_ADDR_DIV_CHR;
	static const std::string CRLF_SET;
	bool SendStr(const StrngStream &Src);
	bool SendStr(const char* Buffer);
	bool VerifyReply(const char* szReplyString);
	bool Receive(char* szBuffer, int nLenMax, int nFlags, const char* szReplyString);
	bool Send(const char *szBuffer, size_t nLen);
	bool Encode64File(const std::string &FileName);
	std::string Encode64String(const std::string &Src);
	char Encode64Byte(unsigned char uc);
	static std::string EmailAddrFormat(const std::string &Src);
	const char* Encode64(const std::vector<unsigned char> & vBytes, std::string *Dest = NULL, bool EndWithCR = false);
};

bool SendEmailSimple(const char* HostName, const char* UserID, const char* To, const char* Subject, const char* Body, const char* UserPassword = NULL, const char* FileName_Attach = NULL, int ServerPortNo = SendEmail::SMTP_OUTGOING_MAIL_PORT);


std::string SimpleCheckMail(const char* HostName, const char* UserID, const char* UserPassword);

#endif //!SENDEMAIL_HEADER_GAURD
