/**
 * @class   wosh::Mail
 * @brief   Rapresentation of an email message
 *
 *
 * @todo    import from MimeParser-1.1
 *
 *
 * @version	$Id: Mail.h 1979 2010-02-27 11:53:39Z alex $
 * @author	Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2009, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Mail_H__
 #define __WOSH_Mail_H__

 #include <woshConfig.h>
 #include <core/Serialization.h>

 #include <string>
 #include <map>


using namespace std;
namespace wosh {


class Mail   {

	protected:

		int id;									///< Unique ID
		string uid;								///< server's uid

		bool read;								///< flag message as read

		mutable map<string,string> headers;		///< map (hash table) containing key-value pairs

		string sender_name;						///< name of the sender (may be empty)
		string sender_add;						///< address of the sender (may be faked)

		string destination_mail;				///< destination email address

		string subject;							///< subject of the message
		string body;							///< text of the message


		long size;								///< size of the message (and attachments)

		long timestamp;							///< timestamp of the message

		void* attachments;						///< TODO...


	public:

		Mail( int id = -1 );

		virtual ~Mail();

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

		void setID( long value )							{ this->id = value; }

		void setUID( const string& value )					{ this->uid = value; }

		void setRead( bool value = true )					{ this->read = value; }


		void clear();

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

		const int& getID() const							{ return this->id; }

		const string& getUID() const						{ return this->uid; }

		const string& getSenderName() const					{ return this->sender_name; }
		const string& getSenderAddress() const				{ return this->sender_add; }
		const string& getSubject() const					{ return this->subject; }
		const string& getBody() const						{ return this->body; }

		const string& getDestinationAddress() const			{ return this->destination_mail; }

		bool isRead() const									{ return this->read; }

		long getSize() const								{ return this->size; }

		long getTimeStamp() const							{ return this->timestamp; }


		// bool hasAttachments() const


//@}

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual string getNameTag() const					{ return "Mail"; };

		WRESULT loadFromPlain( const string& data );

//@}

	protected:

		WRESULT parseHeaders( const string& pop3_headers );

/*
		const int MaxSize = 32767;

		const char constTextPlain[]	= "text/plain";
		const char constTextHtml[]	= "text/html";
		const char constRfc822[]	= "message/rfc822";
		const char constMultipart[]	= "multipart";
		const char constBoundary[]	= "boundary";
		const char constCharset[]	= "charset";
*/
}; // class def

}; // namespace wosh

#endif //__WOSH_Mail_H__
