/**
 * @class   wosh::devices::TestDeviceBundle
 * @brief   Virual device designed for debugging and diagnostic purposes.
 *
 * @ingroup Devices
 ****************************************************************************
 * @version 0.8.013 $Id: TestDeviceBundle.h 2561 2010-05-27 03:30:39Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Devices_TestDevice_H__
 #define __WOSH_Bundles_Devices_TestDevice_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>

 #include "ITestDeviceListener.h"
 
 #include <interfaces/Device.h>
 #include <interfaces/devices/TestDevice.h>

class TestDeviceImpl;

using namespace std;
namespace wosh {
 namespace devices {

 #define _TestDevice_NAME						"TestDevice"
 #define _TestDevice_VERSION					0.01


class TestDeviceBundle :	public BundleGeneric,
							public virtual wosh::interfaces::Device,
							public virtual ITestDeviceListener
 {
	WOSH_BUNDLE(wosh::devices::TestDeviceBundle)
	WOSH_BUNDLE_INTERFACE("wosh::devices::Device")
	WOSH_BUNDLE_SINGLETON(0,0)

	public:
		/**
		 * @brief   Empty constructor, init bundle and vars, Properties and Methods.
		 */
		TestDeviceBundle();
		/**
		 * @brief   Empty destructor, may (synch) stop bundle and internal threads when still running
		 */
		virtual ~TestDeviceBundle();


	public:
		WRESULT bundleStart();
		WRESULT bundleStop();
		WRESULT bundleDiagnostic();

		void busMessage( const Message& message, const Bus* source );

//@}
/** @name Device Interface
 * @{
 ******************************************************************************/
	public:
		wosh::interfaces::Device::DEVICE_STATE getDeviceState() const { return this->devState; }

//@}
/** @name TestDevice Interface
 * @{
 ******************************************************************************/
	public:

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

//@}
/** @name Modem Events
 * @{
 ******************************************************************************/
 	public:

//@}
/** @name IPropertiesProviderListener interface
 * @{
 ******************************************************************************/
 	public:
		bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );
//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

//@}
	private:
		Response* mmDoForceState( const Request* request );
		Response* mmDoRaiseEvent( const Request* request );
		Response* mmDoWaitMethod( const Message* requestMessage );

	protected:
		BusConnector BusDevices;					///< register myself & work as Device too
		TestDeviceImpl* devWorker;					///< real implementation

		wosh::interfaces::Device::DEVICE_STATE devState;		///< current state of device


}; // class def

 }; // namespace devices
}; // namespace wosh

#endif //__WOSH_Bundles_Devices_TestDevice_H__
