/**
 * @class   wosh::services::AlarmBundle
 * @brief   
 *
 *
 * Flow:
 *  - the bundle listen for Core, Building, Devices
 *  - messages are evaluated by current enabled triggers
 *  - triggers may update the alarm level
 *  - the brain evaluates the new state
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 * @ingroup Services
 ****************************************************************************
 * @version $Id: AlarmBundle.h 2282 2010-04-07 06:20:24Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_Alarm_H__
 #define __WOSH_Bundles_Services_Alarm_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>

 #include <interfaces/Service.h>
 #include <interfaces/services/Alarm.h>

 #include <framework/security/AlarmListeners.h>


namespace wosh {
 using namespace security;
 namespace services {

 #define _Alarm_VERSION				0.01

 #define _Alarm_KEY_Configuration	"Configuration"

 #define _Alarm_KEY_AlarmState		"AlarmState"
 #define _Alarm_KEY_AlarmStateTs	"AlarmStateTs"

 #define _Alarm_KEY_Events			"Events"
 #define _Alarm_KEY_LastEvent		"LastEvent"
 #define _Alarm_KEY_LastEventTs		"LastEventTs"

 #define _Alarm_KEY_Errors			"Errors"
 #define _Alarm_KEY_LastError		"LastError"
 #define _Alarm_KEY_LastErrorTs		"LastErrorTs"

 #define _Alarm_KEY_DB				"DataBaseFile"


 #define _Alarm_EVENT_Armed			"Armed"
 #define _Alarm_EVENT_Disarmed		"Disarmed"

 #define _Alarm_EVENT_AlarmEvent	"AlarmEvent"

 #define _Alarm_EVENT_StateChanged	"StateChanged"

 #define _Alarm_ERROR_BRAIN			1040
 

class AlarmBundle :		public BundleGeneric,
						public virtual wosh::interfaces::services::Alarm,
						public virtual IAlarmEngineListener
 {
	WOSH_BUNDLE(wosh::services::AlarmBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::Alarm")
	WOSH_BUNDLE_SINGLETON(1,0)


	public:
		AlarmBundle();

		virtual ~AlarmBundle();


	public:
		WRESULT bundleStart();
		WRESULT bundleStop();

		void busMessage( const Message& message, const Bus* source );

		bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );

//@}
/** @name Alarm Interface
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT arm()				{ return bundleStart(); }
		virtual WRESULT disarm()			{ return bundleStop(); }

		virtual WRESULT loadConfiguration();
		virtual WRESULT saveConfiguration();

//@}
/** @name AlarmEngine Interface
 * @{
 ******************************************************************************/
 	public:
		virtual void alarm_armed( AlarmEngine* source );
		virtual void alarm_disarmed( AlarmEngine* source );

		virtual void postMessage( Message* message, AlarmEngine* source );

		virtual void changed_AlarmState( int state, AlarmState* source );

		virtual void raised_AlarmEvent( const AlarmEvent* alarm_event, AlarmState* source );
		virtual void raised_AlarmBrain_Error( int error, const std::string& description, AlarmBrain* source );

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		Response* mmDoArmAlarm( const Request* request );
		Response* mmDoDisarmAlarm( const Request* request );
		Response* mmDoGetAlarmTriggers( const Request* request );
		Response* mmDoGetAlarmEvents( const Request* request );

//@}
	protected:
		AlarmEngine* alarmEngine;	///< real implementation

		BusConnector BusBuilding;
		BusConnector BusDevices;


}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_Alarm_H__
