/**
 * @class   wosh::services::FestivalBundle
 * @brief   
 *
 *
 *
 * Installation on Debian (Requirements) [italian example]:
 * \code
 *   $ apt-get install festival festvox-italp16k festvox-itapc16k alsa-oss oss-compat
 * \endcode
 *
 * .festivalrc
 * .festivalvarsrc
 *
 * Test Festival:
 
 * \code
 *   $ echo "Hello World"| festival --tts
 *
 *   $ festival
 *    festival> (SayText "Ciao, evviva Debian");
 * \endcode
 *
 * http://www2.pd.istc.cnr.it/FESTIVAL/home/
 *
 * @ingroup Services
 ****************************************************************************
 * @version $Id: FestivalBundle.h 2282 2010-04-07 06:20:24Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_Festival_H__
 #define __WOSH_Bundles_Services_Festival_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>

 #include <interfaces/Service.h>
 #include <interfaces/services/Festival.h>
 #include <interfaces/services/InterpreterService.h>

 #include <framework/communication/NotificationConversion.h>


class FestivalImpl;

using namespace wosh::communication;
namespace wosh {
 namespace services {

 #define _Festival_VERSION				0.20



class FestivalBundle :		public BundleGeneric,
							public virtual wosh::interfaces::services::InterpreterService,
							public virtual wosh::interfaces::services::Festival
 {
	WOSH_BUNDLE(wosh::services::FestivalBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::Festival")
	WOSH_BUNDLE_SINGLETON(1,0)

	public:
		FestivalBundle();

		virtual ~FestivalBundle();


	public:
		WRESULT bundleStart();
		WRESULT bundleStop();

		bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );

		void busMessage( const Message& message, const Bus* source );

//@}
/** @name Festival Interface
 * @{
 ******************************************************************************/
	public:
		bool validateTranslation( const NotificationConversion* conversion );

		WRESULT enqueueTranslation( const NotificationConversion* conversion, const URI* uri );

//@}
/** @name Festival Interface for Automation callbacks
 * @{
 ******************************************************************************/
	public:

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		Response* mmDoConvert( const Message* requestMessage );
		Response* mmDoListVoices( const Message* requestMessage );
		Response* mmDoEvalCommand( const Message* requestMessage );


//@}
	private:

		static FestivalBundle* getInstance();
		static FestivalBundle* ms_FestivalBundle_instance;

	protected:

		FestivalImpl* autoWorker;	///< real implementation



}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_Festival_H__
