/** @file    GardenerWorker.cpp
 *  @author  Alessandro Polo
 *  @version $Id: GardenerWorker.cpp 2803 2010-07-26 14:28:07Z alex $
 *  @brief
 * File containing methods for the GardenerWorker class.
 * The header for this class can be found in GardenerWorker.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "GardenerWorker.h"

 #include <algorithm>


using namespace std;
using namespace wosh;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

GardenerWorker::GardenerWorker( BundleGeneric& bundle )
	: BundleGenericWorker(bundle) {
	this->listener = NULL;
 }


GardenerWorker::~GardenerWorker() {
	if ( isThreadRunning() ) {
		quitThread(10000);
	 }
	Log(LOG_VERBOSE, ":~GardenerWorker() : Freeing Programs.." );
	this->programs.clear();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD

void GardenerWorker::runThread() {
	Log(LOG_DEBUG, "runThread() : Entering Loop.." );
	tStrGardenerProgramMap::ConstIterator it;

	while(this->running) {
		ThreadImpl::sleepForMSec(200);

		this->programs.transactionBeginRead();
		for ( it=this->programs.begin(); it!=this->programs.end(); ++it ) {
			if ( it->second == NULL ) continue;
			if ( !it->second->isEnabled() ) continue;
			
			//if ( !it->second->isActive() ) continue;

		 }
		this->programs.transactionEnd();

	 }

	Log(LOG_DEBUG, "runThread() : Exiting Loop.." );
 }

////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT GardenerWorker::addProgram( GardenerProgram* program ) {
	if ( program == NULL ) return WRET_ERR_PARAM;
	Log(LOG_INFO, "addProgram(%s) : Adding Program#%"PRId64"..", program->getName().c_str(), program->getID() );
	this->programs.set(program->getName(), program);
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void GardenerWorker::evaluateWeatherUpdate( const wosh::xtended::WeatherInfo* updated_info, double similarity ) {
	if ( updated_info == NULL ) return;
	Log(LOG_INFO, "evaluateWeatherUpdate() : Forwarding news.." );
	tStrGardenerProgramMap::ConstIterator it;
	this->programs.transactionBeginRead();
	for ( it=this->programs.begin(); it!=this->programs.end(); ++it ) {
		if ( it->second == NULL ) continue;
		if ( !it->second->isEnabled() ) continue;
		it->second->evaluateWeatherUpdate(updated_info, similarity);
	 }
	this->programs.transactionEnd();
 }

WRESULT GardenerWorker::controlProgram( const std::string& target_program, bool activate, bool exclusive ) {
	this->programs.transactionBeginRead();
	if ( !this->programs.exists(target_program) ) {
		this->programs.transactionEnd();
		return WRET_ERR_PARAM;
	 }
	Log(LOG_INFO, "controlProgram(%s) : [activate=%i,exclusive=%i]", target_program.c_str(), activate, exclusive );
	tStrGardenerProgramMap::ConstIterator it;
	for ( it=this->programs.begin(); it!=this->programs.end(); ++it ) {
		if ( it->second == NULL ) continue;
		if ( !it->second->isEnabled() ) continue;
		if ( target_program != it->first ) {
			if ( exclusive )
				it->second->setActive(false);
			continue;
		 }
		it->second->setActive(activate);
	 }
	this->programs.transactionEnd();
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
