/**
 * @class   GardenerZone
 * @brief   Interface for 
 *
 ****************************************************************************
 * @version $Id: GardenerZone.h 2803 2010-07-26 14:28:07Z alex $
 * @author Alessandro Polo
 * @date   Released on 2009-06-04
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_SERVICES_GardenerZone_H__
 #define __WOSH_SERVICES_GardenerZone_H__

 #include <woshDefs.h>
 #include <core/Object.h>
 #include <core/LogContext.h>
 #include <core/Utilities.h>


using namespace std;
using namespace wosh;

class GardenerZone : public Object
 {
	WOSH_CLASSNAME(GardenerZone)

	public:
		enum SOURCE_TYPE {
			SOURCE_UNKNOWN	= 0x00,
			SOURCE_SENSOR	= 0x01,
			SOURCE_WEB		= 0x02,
			SOURCE_USER		= 0x04
		 };

	public:
		GardenerZone()
			: Object() {
			this->uid = Utilities::randomLong();
			this->uid = _GET_Abs(this->uid);
			Object::setName("GardenerZone", this->uid);
			Log.setContext(Object::getName());
			Log.setIndent( 4 );
			Log.setLevel( LOG_INFO );
			Log.setLevel( LOG_DEBUG );
			clear();
		 }

		virtual ~GardenerZone() { }


	public:
		inline int64 getID() const						{ return this->uid; }
		bool isEnabled() const							{ return this->enabled; }
		long getPriority() const						{ return this->priority; }

	public:
		void setEnabled( bool value )					{ this->enabled = value; }
		void setPriority( long value )					{ this->priority = value; }

	protected:
		void clear() {
			this->enabled = true;
			this->priority = 0;
		 }

	protected:
		int64 uid;					///<
		LogContext Log;				///<
		bool enabled;				///<
		long priority;				///<

	protected:
		long time_min;				///< minimum time in seconds foreach session. 0=disabled
		long time_max;				///< maximum time in seconds foreach session. 0=disabled
		long time_period_repeat;	///< period in seconds between each session

	protected:


}; // class def


#endif //__WOSH_SERVICES_GardenerZone_H__
