/**
 * @class   wosh::services::HeyuBundle
 * @brief   Wrapper to Heyu software, supports monitoring and controling X10 devices.
 *          POSIX systems only.
 *
 *
 *
 *
 *
 *
 *
 *
 * @ingroup Services
 ****************************************************************************
 * @version 0.8.378 $Id: HeyuBundle.h 2768 2010-07-13 16:58:32Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_Heyu_H__
 #define __WOSH_Bundles_Services_Heyu_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>

 #include <interfaces/Service.h>
 #include <interfaces/services/Heyu.h>

 #include "IHeyuBindingListener.h"


namespace wosh {
 namespace services {

 #define _Heyu_NAME						"Heyu"
 #define _Heyu_VERSION					0.82

 #define _Heyu_DEFAULT_heyubin			"heyu"
 

class HeyuBundle :	public BundleGeneric,
					public virtual wosh::interfaces::services::Heyu,
					public virtual IHeyuBindingListener
 {
	WOSH_BUNDLE(wosh::services::HeyuBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::Heyu")
	WOSH_BUNDLE_SINGLETON(1,0)


	public:
		/**
		 * @brief   Empty constructor, init bundle and vars, Properties and Methods.
		 */
		HeyuBundle();
		/**
		 * @brief   Empty destructor, may (synch) stop bundle and internal threads when still running
		 */
		virtual ~HeyuBundle();


	public:
		WRESULT bundleStart();
		WRESULT bundleStop();

		bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );

		void busMessage( const Message& message, const Bus* source );

//@}
/** @name HeyuBinding Events
 * @{
 ******************************************************************************/
 	public:
		void device_event( DeviceX10* device, DeviceX10::MODULE_ACTION event, int arg_value, int event_source, HeyuBinding* x10d );
		void driver_event( int event_id, HeyuBinding* x10d );

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:
		void updateHeyuProps();
		void registerDevices();
		void unRegisterDevices();
		WRESULT reloadHeyuDevices();

		Message* busMessageDevice( const Message& message, DeviceX10* dev );

//@}
	private:
		Response* mmDoUpdate( const Request* request );
		Response* mmDoEvalCommand( const Request* request );

	protected:
		BusConnector BusDevices;	///< Connect 'Device' Bus
		HeyuBinding* heyuBinding;	///< real implementation (wrapper to Heyu software)


}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_Heyu_H__
