/**
 * @file    test_HeyuBinding.cpp
 * @brief   Test classes: wosh::devices::HeyuBinding
 *
 * @version $Id: test_HeyuBinding.cpp 2574 2010-05-27 09:04:30Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

 #include <woshDefs.h>
 #include "HeyuBinding.h"
 
bool exitRequest = false;
//////////////////////////////////////////////////////////////////
#ifdef _OS_POSIX
 #include <fcntl.h>
 #include <unistd.h>
 #include <signal.h>

void test_HeyuBinding_signal_handler(int sig) {
	switch(sig) {
		case SIGHUP:
		case SIGTERM:
		case SIGINT:
			exitRequest = true;
			break;
	}
 }
void test_HeyuBinding_registerSignals()  {
	signal (SIGINT, test_HeyuBinding_signal_handler);
	signal (SIGTERM, test_HeyuBinding_signal_handler);
	signal (SIGHUP, test_HeyuBinding_signal_handler);
 }
#endif
//////////////////////////////////////////////////////////////////


using namespace std;
using namespace wosh;
using namespace wosh::services;


void test_class_HeyuBinding();


int test_HeyuBinding()
 {
	Log::log( "test_HeyuBinding() : Starting TEST" );

	test_class_HeyuBinding();


	Log::log( "test_HeyuBinding() : Finished TEST" );
	return 0;
 }



class HeyuBindingstdout : public IHeyuBindingListener  {
	public:
		virtual void driverX10_event( HeyuBinding* x10d, const string& code, const string& status )
		 {
//qDebug() << "driverX10_event:: " << code.c_str() << status.c_str();
		 }

		HeyuBindingstdout() { }
		virtual ~HeyuBindingstdout() { }

};

 

void test_class_HeyuBinding()
 {
	Log::log( " test_class_HeyuBinding() : Starting TEST for class 'HeyuBinding'" );

#ifdef _OS_POSIX
	test_HeyuBinding_registerSignals();
#endif

	Logger log;

	HeyuBindingstdout x10event_handler;
	HeyuBinding x10d;
	// setup
	x10d.setHeyuBindingListener(&x10event_handler);
	x10d.setLogger( &log );
	x10d.setHeyuBinary( "heyu" );

	x10d.heyuEngineCheckState();
qDebug() << "State" << (int)x10d.getState();


//	WRESULT ret_upd = x10d.updateAll( false );
//	ASSERT ( ret_upd == WRET_OK )
//qDebug() << "Dump of D" << x10d.getHouseStatus( 'D' ).c_str();

	x10d.setLogger( &log );

	// connect
	x10d.startThread();


	// loop
	qDebug() << "HERE!";
	while( true ) {
		ThreadImpl::sleepForMSec(100);
		if ( exitRequest ) break;
	 }

	// exit
	x10d.quitThread();

	Log::log( " test_class_HeyuBinding() : Finished TEST for class 'HeyuBinding'" );
 }




int main()
 {
	return test_HeyuBinding();
 }
