/**
 * @class   SessionProtocolHuman
 * @brief   
 *
 ****************************************************************************
 * @version $Id: SessionProtocolHuman.h 2274 2010-04-06 01:40:21Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_SessionProtocolHuman_H__
 #define __WOSH_SessionProtocolHuman_H__

 #include <core/DataList.h>
 #include <framework/communication/Session.h>
 #include <framework/communication/SessionProtocol.h>
 #include <framework/communication/ShellCommand.h>

 #include <map>

namespace wosh {
 namespace services {
   class HumanInterpreterBundle;
  };
};


class SessionProtocolHuman : public wosh::communication::SessionProtocol  {

	public:
		SessionProtocolHuman();
		virtual ~SessionProtocolHuman();

	public:

		virtual std::string getProtocolName() const			{ return "human"; }

		virtual double getProtocolVersion() const 			{ return 0.1; }


		// build prompt 
		virtual void printPrompt( wosh::communication::Session& session ) const;

		virtual bool getLoginData( wosh::communication::Session& session, std::string& user, std::string& pwd );

		virtual void parse_command( wosh::communication::Session& session, const std::string& dataline );

		virtual void evalMessage( wosh::communication::Session& session, const wosh::Message* message );


	protected:
	
		bool internal_command( wosh::communication::Session& session, wosh::communication::ShellCommand* command );

		void printDataList( wosh::communication::Session& session, const wosh::DataList* list, const std::string& header );

WRESULT evalAnswerHumanMessage( wosh::communication::Session& session, const std::string& message, std::string& response, int preferred_max_size = 255 );
WRESULT evalAnswerHumanRequest( wosh::communication::Session& session, const std::string& sentence, std::string& response, int preferred_max_size = 255 );

WRESULT translateMessageToHumanLanguage( const wosh::Message& message, std::string& response, int preferred_max_size = 255 );


	protected:

		void setHumanInterpreterBundle( wosh::services::HumanInterpreterBundle* hib )	{ this->hiParent = hib; }


		wosh::services::HumanInterpreterBundle* hiParent;

		friend class wosh::services::HumanInterpreterBundle;


}; // class def

#endif //__WOSH_SessionProtocolHuman_H__
