/**
 * @class   wosh::SystemHost
 * @brief   
 *

 *
 *
 * @version	$Id: SystemHost.h 2634 2010-06-09 23:01:02Z alex $
 * @author	Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_SystemHost_H__
 #define __WOSH_SystemHost_H__

 #include <core/Object.h>
 #include <core/Utilities.h>


using namespace std;
namespace wosh {

 template <class T> class DataFieldCollector;

class SystemHost : public Object
 {
	WOSH_SERIALIZABLE(wosh::SystemHost)


	public:
		SystemHost()
			: Object() {
			clear();
		 }

		SystemHost( const SystemHost& m )
			: Object() {
			clear();
			*this = m;
		 }

		virtual ~SystemHost() { }


	public:
		SystemHost& operator=(const SystemHost& m);
		bool operator==(const SystemHost &other) const;

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		bool setMAC( const string& value ) {
			this->mac = value;
			if ( value.size() != 10 ) return false;
			return true;
		 }
		void setHostName( const string& value )		{ this->hostname = value; }
		void setIP( const string& value )			{ this->ip = value; }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		const std::string& getMAC() const			{ return this->mac; }
		std::string& getMAC()						{ return this->mac; }

		const std::string& getIP() const			{ return this->ip; }
		std::string& getIP()						{ return this->ip; }

		const std::string& getHostName() const		{ return this->hostname; }
		std::string& getHostName()					{ return this->hostname; }

		virtual long getID() const					{ return this->id; }

		static DataFieldCollector<SystemHost>* getDataFields();

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const		{ return ""; }
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}
	protected:
		virtual void clear();

	protected:
		long id;					///< unique ID of 
		std::string hostname;
		std::string ip;
		std::string mac;


}; // class def

}; // namespace wosh

#endif //__WOSH_CronJob_H__
