/**
 * @class   wosh::services::SystemServicesBundle
 * @brief   Expose standardized OS-dependent functionalities.
 *
 * Features may be implemented:
 *  - (on-request) monitor file/folder change
 *  - FileSystem-mounting, cooperate with wosh::FileSystem
 *  - Time-synchronization
 *
 *
 * @ingroup Services
 ****************************************************************************
 * @version 0.6.030 $Id: SystemServicesBundle.h 2488 2010-05-21 01:48:27Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_SystemServices_H__
 #define __WOSH_Bundles_Services_SystemServices_H__

 #include <core/BundleGeneric.h>
 #include <core/Containers.h>

 #include <interfaces/Service.h>
 #include <interfaces/services/SystemServices.h>

 #include "SystemHost.h"


namespace wosh {
 namespace services {

 #define _SystemServices_VERSION				0.12


class SystemServicesBundle :	public BundleGeneric,
								public virtual wosh::interfaces::services::SystemServices
 {
	WOSH_BUNDLE(wosh::services::SystemServicesBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::SystemServices")
	WOSH_BUNDLE_SINGLETON(1,0)

	typedef MapT<std::string, SystemHost*> tStrSystemHostMap;


	public:
		SystemServicesBundle();

		virtual ~SystemServicesBundle();


	public:
		WRESULT bundleStart();
		WRESULT bundleStop();

		void busMessage( const Message& message, const Bus* source );


/** @name SystemServices Interface
 * @{
 ******************************************************************************/
	public:
		WRESULT loadObjects();
		WRESULT saveObjects();

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:

//@}
/** @name Modem Events
 * @{
 ******************************************************************************/
	public:

//@}
/** @name Thread Events
 * @{
 ******************************************************************************/

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:
		WRESULT addObject_( SystemHost* obj );
		WRESULT removeObject_( SystemHost* obj );

//@}
	private:
		Response* mmDoListSystemHosts( const Message* requestMessage );
		Response* mmDoWakeOnLAN( const Message* requestMessage );

	protected:
		tStrSystemHostMap objects;
		//MapT<std::string, PropertiesProvider*> info;


}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_SystemServices_H__
