/**
 * @class   wosh::services::TaskManagerBundle
 * @brief   Manage Automations and generic tasks within WOSH.
 *
 *
 *
 * @see     wosh::automations
 * @ingroup Services
 ****************************************************************************
 * @version 0.8.013 $Id: TaskManagerBundle.h 2770 2010-07-14 01:33:48Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_TaskManager_H__
 #define __WOSH_Bundles_Services_TaskManager_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>

 #include <framework/automations/Automation.h>
 #include <framework/xtended/Scheduler.h>

 #include <interfaces/Service.h>
 #include <interfaces/services/TaskManager.h>
 
class TaskManagerImpl;

using namespace wosh::automations;
namespace wosh {
 namespace services {

 #define _TaskManager_VERSION				0.10


class TaskManagerBundle :	public BundleGeneric,
							public virtual wosh::interfaces::services::TaskManager,
							public virtual wosh::xtended::ISchedulerListener
 {
	WOSH_BUNDLE(wosh::services::TaskManagerBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::TaskManager")
	WOSH_BUNDLE_SINGLETON(1,0)

	typedef MapT<int64, Automation*> tInt64AutomationMap;


	public:
		TaskManagerBundle();
		virtual ~TaskManagerBundle();

	public:
		WRESULT bundleStart();
		WRESULT bundleStop();

		bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );

	public:
		WRESULT addAutomation( Automation* automation );

		WRESULT loadAutomations();
		WRESULT saveAutomations();

/** @name ISchedulerListener interface
 * @{
 ******************************************************************************/
	public:
		void taskEvent( wosh::xtended::SchedulerTask* task, int64 time, wosh::xtended::Scheduler* source );
//@}
	protected:
		WRESULT addAutomation_( Automation* automation );
		WRESULT execAutomation_( Automation* automation );

	private:
		Response* mmDoExecTask( const Message* messageRequest );
		Response* mmDoReloadTasks( const Request* request );
		Response* mmDoScheduleTask( const Message* messageRequest );

	protected:
		BusConnector BusAutomation;

		tInt64AutomationMap automations;

		wosh::xtended::Scheduler* scheduler;


}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_TaskManager_H__
