/**
 * @class   TelnetServerImpl
 * @brief   Service providing TelnetServer features
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 ****************************************************************************
 * @version $Id: TelnetServerImpl.h 2488 2010-05-21 01:48:27Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Services_TelnetServer_Impl_H__
 #define __WOSH_Services_TelnetServer_Impl_H__

 #include <core/Object.h>
 #include <core/BundleGenericWorker.h>

 #include <framework/communication/Session.h>
 #include <framework/communication/SessionProtocol.h>
 #include <framework/communication/ShellCommand.h>
 #include <interfaces/services/TelnetServer.h>

 #include <framework/network/SocketTcp.h>
 #include <framework/network/ServerTcp.h>

 #include "TelnetSession.h"


using namespace std;
using namespace wosh::network;

 #define _Channel_KEY_BindAddress		"BindAddress"
 #define _Channel_KEY_Listening			"Listening"


class TelnetServerImpl :	public BundleGenericWorker,
							public virtual wosh::interfaces::services::TelnetServer,
							public virtual IServerTcpListener
 {

	public:
		TelnetServerImpl( BundleGeneric& bundle );
		virtual ~TelnetServerImpl();

	public:
		WRESULT listen( const std::string& addressport, unsigned long timeout = LIMIT_ULONG_MAX );
		WRESULT stopListening( unsigned long timeout = LIMIT_ULONG_MAX );

		bool isListening() const;


	public:
		bool session_post_message( Session* session, Message* message );
		void forwardMessage_( const Message& message, long id );













		ListT<TelnetSession*>& getSessions()	{ return this->sessions; }

		WRESULT getSessionsName( std::vector<std::string>& sessions_names ) const;

		WRESULT getSessionsUnixStyle( std::vector<std::string>& sessions_UnixStyle ) const;


	public:

		virtual Session* getSession_( long id );

		virtual WRESULT registerSession( Session* session );
		virtual WRESULT unRegisterSession( Session* session );

		virtual bool isUserLogged( const std::string& username ) const		{ return (getSessionIDByUser(username) != 0); }

		virtual long getSessionIDByUser( const std::string& username ) const;

		

		virtual WRESULT registerProtocol( SessionProtocol* session_proto );
		virtual WRESULT unRegisterProtocol( SessionProtocol* session_proto );

	public:

		bool session_is_auth( Session* session, const string& user, const string& pwd );

		virtual WRESULT changeSessionProtocol( Session* session, const string& protocol_name );

		friend class Session;


		WRESULT setAddressPort( const string& address_port );



/** @name IServerTcpListener
 * @{
 ******************************************************************************/
	public:
		virtual void serverTcp_incomingConnection( int socket_id, wosh::network::ServerTcp* source );
//@}
	protected:
		bool initializingThread();
		void runThread();
		void exitingThread();

	protected:
		wosh::network::ServerTcp* tcpServer;

		bool ready;

		string bindAddressPort;


//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		/**
		 * @brief   
		 * @see     running
		 * @see     
		 */
		virtual void runThread();

//@}

	protected:

		SessionProtocol* protocolDefault;
		std::vector<SessionProtocol*> protocols;

		ListT<TelnetSession*> sessions;

}; // class def


#endif //__WOSH_Services_TelnetServer_Impl_H__
