/**
 * @class   TelnetSession
 * @brief   S
 *
 *
 *
 *
 ****************************************************************************
 * @version $Id: TelnetSession.h 2574 2010-05-27 09:04:30Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Services_TelnetSession_H__
 #define __WOSH_Services_TelnetSession_H__

 #include <framework/communication/Session.h>
 #include <core/Thread.h>
 #include <framework/network/SocketTcp.h>

class TelnetServerImpl;

using namespace std;
using namespace wosh;
using namespace wosh::network;
using namespace wosh::communication;


class TelnetSession :	public wosh::communication::Session,
						protected wosh::ThreadImpl,
						public virtual ISocketTcpListener
 {
	WOSH_SESSION(TelnetSession)


	public:

		TelnetSession();

		virtual ~TelnetSession();

WRESULT disconnect();
bool isConnected() const;

/** @name Session Interface
 * @{
 ******************************************************************************/
	public:
		WRESULT sessionStart( unsigned long timeout = LIMIT_ULONG_MAX );
		WRESULT sessionStop( unsigned long timeout = LIMIT_ULONG_MAX );

		inline std::istream& istream()				{ return this->inputStream; }
		inline std::ostream& ostream()				{ return this->outputStream; }

//@}
/** @name ISocketTcpListener
 * @{
 ******************************************************************************/
	public:
		void tcpDataAvailable( int data_len, SocketTcp* socket );
		void tcpSocketError( int error_code, const char* description, SocketTcp* socket );
//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:
		void runThread();

//@}
	protected:
		std::stringstream outputStream;
		std::stringstream inputStream;

		SocketTcp* socket;

	protected:
		char* msgBufferOut;
		int msgBufferOutSize;
		MutexRW msgBufferOutMux;
		
		char* msgBufferIn;
		int msgBufferInSize;


}; // class def


#endif //__WOSH_Services_TelnetSession_H__
