/**
 * @class   wosh::services::WeatherBundle
 * @brief   
 *
 * to ADD: multiple data source, data-fusion
 *
 * http://www.ilmeteo.it/portale/taxonomy/term/2/0/feed?meteo=Mori
 * \code
apt-get install libcurl4-dev
apt-get install libxslt1-dev
 * \endcode
 *
 *
 * @ingroup Services
 * @see     wosh::NetworkManager
 * @see     wosh::WoshHost
 * @see     wosh::services::Weather
 ****************************************************************************
 * @version $Id: WeatherBundle.h 2804 2010-07-27 04:39:07Z alex $
 * @author  Alessandro Polo
 * @date    Released on 2009-06-04
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Services_Weather_H__
 #define __WOSH_Bundles_Services_Weather_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>

 #include <interfaces/Service.h>
 #include <interfaces/services/Weather.h>

 #include "WeatherWorker.h"


using namespace std;
namespace wosh {
 namespace services {

 #define _Weather_VERSION					0.01


class WeatherBundle :	public BundleGeneric,
						public virtual wosh::interfaces::services::Weather,
						public virtual IWeatherWorker
 {
	WOSH_BUNDLE(wosh::services::WeatherBundle)
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::Weather")
	WOSH_BUNDLE_SINGLETON(1,0)


	public:
		/**
		 * @brief   Empty constructor, init bundle and vars, Properties and Methods.
		 */
		WeatherBundle();
		/**
		 * @brief   Empty destructor, may (synch) stop bundle and internal threads when still running
		 */
		virtual ~WeatherBundle();


	public:
		WRESULT bundleStart();
		WRESULT bundleStop();

		void busMessage( const Message& message, const Bus* source );

		bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );

//@}
/** @name Getters
 * @{
 ******************************************************************************/
 	public:

//@}
/** @name Setters
 * @{
 ******************************************************************************/
 	public:
		void updatedWeather( const WeatherInfo* newInfo, double similarity, WeatherWorker* source );
		void updatedForecast( const vector<WeatherInfo*>& newForecast, WeatherWorker* source );

//@}
	private:
		Response* mmDoProcessDebug( const Request* request );
		Response* mmDoExport( const Request* request );

	protected:
		WeatherWorker* myWorker;

		double currentWeatherSimilarityThreshold;

}; // class def

 }; // namespace services
}; // namespace wosh

#endif //__WOSH_Bundles_Services_Weather_H__
