/**
 * @class   WeatherSourceHttp
 * @brief   
 *

 ****************************************************************************
 * @version $Id: WeatherSourceHttp.h 2258 2010-04-02 13:02:21Z alex $
 * @author Alessandro Polo
 * @date   Released on 2009-06-04
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_SERVICES_WeatherSourceHttp_H__
 #define __WOSH_SERVICES_WeatherSourceHttp_H__

 #include "WeatherSource.h"
 #include <framework/xtended/WeatherInfo.h>
 #include <framework/network/ClientHttp.h>
 #include <core/Thread.h>
 #include <core/FileSystem.h>


using namespace wosh;
using namespace wosh::network;

class WeatherSourceHttp :		public WeatherSource,
								public virtual IClientHttpListener
 {
	WOSH_CLASSNAME(WeatherSourceHttp)

	public:
		WeatherSourceHttp() : WeatherSource() {
			Object::setName("WeatherSourceHttp");
			Log.setContext( "WeatherSourceHttp" );
			this->httpClient = new ClientHttp();
			this->httpClient->setListener(this);
			FileSystem::reserveTempFile( this->xmlLocalTemp );
		 }
		virtual ~WeatherSourceHttp() {
			delete this->httpClient; this->httpClient = NULL;
			FileSystem::freeTempFile( this->xmlLocalTemp );
		 }

	public:
		virtual WRESULT updateData( unsigned long timeout = LIMIT_ULONG_MAX, bool* newWeatherAvailable = NULL ) {
			WRESULT ret = this->httpClient->downloadFile( this->xmlUrl, this->xmlLocalTemp );
			if ( WFAILED(ret) || timeout == 0 ) return ret;
			timeout = (unsigned long)(timeout / 500) + 1;
			unsigned long i = 0;
			while( ++i < timeout ) {
				ThreadImpl::sleepForMSec(500);
				if( !this->httpClient->isDownloading() ) { //lame
					if ( this->httpClient->isRequestAborted() ) {
						setLastOperation(-1,WRET_ERR_INTERNAL);
						return WRET_ERR_INTERNAL;
					 }
					if ( newWeatherAvailable != NULL )
						*newWeatherAvailable = true;
					setLastOperation(-1,WRET_OK);
					return WRET_OK;
				 }
			 }
			return ret;
		 }

		virtual SOURCE_TYPE getSourceType() const	{ return WeatherSource::SOURCE_WEB; }

	public:
		virtual WRESULT setXmlURL( const std::string& xml_url ) {
			this->xmlUrl = xml_url;
			return WRET_OK;
		 }

	public:
		virtual void httpRequestInitiated( ClientHttp* ) { }
		virtual void httpRequestCompleted( ClientHttp* ) { }
		virtual void httpRequestError( ClientHttp* ) { }
		virtual void httpDataReadProgress( long, long, ClientHttp* ) { }

	protected:
		wosh::network::ClientHttp* httpClient;
		std::string xmlUrl;
		std::string xmlLocalTemp;


}; // class def


#endif //__WOSH_SERVICES_WeatherSourceHttp_H__
