/** @file    MessageContent.cpp
 *  @author  Alessandro Polo
 *  @version $Id: MessageContent.cpp 2432 2010-05-07 21:40:29Z alex $
 *  @brief
 * File containing methods for the wosh::MessageContent class.
 * The header for this class can be found in MessageContent.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/MessageContent.h>

 #include <core/ObjectAllocator.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/Fact.h>
 #include <core/Notification.h>
 #include <core/SystemError.h>


using namespace std;
namespace wosh {

 WOSH_REGISTER_INTERFACE(wosh::MessageContent, "wosh::ISerializable", 1.01, _static_MessageContent)
 WOSH_REGISTER_INTERFACE(wosh::MessageContentGeneric, "wosh::MessageContent", 1.01, _static_MessageContentGeneric)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// CASTING HELPERS

const Request* MessageContent::asRequest() const {
	if ( !isRequest() ) return NULL;
	return static_cast<const Request*>( this );
 }

const Response* MessageContent::asResponse() const {
	if ( !isResponse() ) return NULL;
	return static_cast<const Response*>( this );
 }

const Fact* MessageContent::asFact() const {
	if ( !isFact() ) return NULL;
	return static_cast<const Fact*>( this );
 }

const Notification* MessageContent::asNotification() const {
	if ( !isNotification() ) return NULL;
	return static_cast<const Notification*>( this );
 }

const SystemError* MessageContent::asSystemError() const {
	if ( !isSystemError() ) return NULL;
	return static_cast<const SystemError*>( this );
 }

///////////////////////////////////////////////////////////////////////////////////////////// CASTING HELPERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////// STATIC

std::string MessageContent::getTypeAsString(TYPE type)
 {
	switch(type) {
		case MessageContent::GENERIC:		return "GENERIC";
		case MessageContent::SYSTEM:		return "SYSTEM";
		case MessageContent::FACT:			return "FACT";
		case MessageContent::ACTION:		return "ACTION";
		case MessageContent::AUTOMATION:	return "AUTOMATION";
		case MessageContent::NOTIFICATION:	return "NOTIFICATION";
		case MessageContent::RESPONSE:		return "RESPONSE";
		case MessageContent::PROPERTY:		return "PROPERTY";
		case MessageContent::UNKNOWN:
		default:							return "UNKNOWN";
	 }
 }

MessageContent::TYPE MessageContent::getTypeFromString( const std::string& type_string )
 {
	if ( type_string == "GENERIC")
		return MessageContent::GENERIC;
	else if ( type_string == "SYSTEM")
		return MessageContent::SYSTEM;
	else if ( type_string == "FACT")
		return MessageContent::FACT;
	else if ( type_string == "ACTION")
		return MessageContent::ACTION;
	else if ( type_string == "AUTOMATION")
		return MessageContent::AUTOMATION;
	else if ( type_string == "NOTIFICATION")
		return MessageContent::NOTIFICATION;
	else if ( type_string == "RESPONSE")
		return MessageContent::RESPONSE;
	else if ( type_string == "PROPERTY")
		return MessageContent::PROPERTY;
	else if ( type_string == "UNKNOWN")
		return MessageContent::UNKNOWN;

	return MessageContent::UNKNOWN;
 }

////////////////////////////////////////////////////////////////////////////////////////////////////// STATIC
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
