/**
 * @class   wosh::MethodArguments
 * @brief   Store the arguments' definition of a Method
 *
 *
 *


 - expose arguments of a method (its interface)
 - basic input validation
 - ???????????

 - map Message (Request) to a (real) membeer function


 - MethodGeneric  template con ret/args comuni
 - una specializzazione con WRESULT => mappa ret#code a response string error













 *
 *
 ****************************************************************************
 * @version $Id: MethodArguments.h 2551 2010-05-26 21:55:42Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR MethodS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_MethodArguments_H__
 #define __WOSH_Core_MethodArguments_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>
 #include <core/Variant.h>

 #include <vector>


namespace wosh {


class MethodArgument : public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::MethodArgument)
	WOSH_COMPARE_RTTI(wosh::MethodArgument)

	public:

		MethodArgument() {
			this->typeName = "";
			this->optional = false;
			this->ordered = -1;
		 }

		MethodArgument( Variant::DATA_TYPE type, bool is_optional = false, int index = -1 ) {
			set(type, is_optional, index);
		 }

		MethodArgument( const char* classname, bool is_optional = false, int index = -1 ) {
			set(classname, is_optional, index);
		 }

		MethodArgument( const MethodArgument& m ) {
			*this = m;
		 }

		virtual ~MethodArgument() { }


	public:
		void set( Variant::DATA_TYPE type, bool is_optional = false, int index = -1 );
		void set( const char* classname, bool is_optional = false, int index = -1 );

/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		MethodArgument& operator=(const MethodArgument& m);

		bool operator==(const MethodArgument &other) const;
		bool operator!=(const MethodArgument &other) const	{ return !(*this == other);  }

		bool operator>(const MethodArgument& other) const;
		bool operator<(const MethodArgument& other) const;

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}
	public:
		std::string typeName;
		bool optional;
		int ordered;
		std::string fieldName;

}; // class def



class MethodArguments : public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::MethodArguments)
	WOSH_COMPARE_RTTI(wosh::MethodArguments)

	public:
		MethodArguments() { }

		MethodArguments( const MethodArguments& m ) {
			*this = m;
		 }

		virtual ~MethodArguments() { }


	public:



//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

		void set( unsigned int i, const MethodArgument& arg ) {
			if ( this->args.size() <= i ) {
				this->args.resize(i+1);
			 }
			this->args.at(i) = arg;
		 }

		void set( unsigned int i, Variant::DATA_TYPE type, bool optional = false ) {
			set( i,  MethodArgument(type,optional,i) );
		 }

		void set( unsigned int i, const char* classname, bool optional = false ) {
			set( i,  MethodArgument(classname,optional,i) );
		 }

		void setCount( unsigned int i ) {
			this->args.resize(i);
		 }

		MethodArgument& get( unsigned int i ) {
			if ( this->args.size() <= i ) {
				this->args.resize(i+1);
			 }
			return this->args.at(i);
		 }

		void clear() {
			this->args.clear();
		 }
//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:


//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		MethodArguments& operator=(const MethodArguments& m);
		bool operator==(const MethodArguments &other) const;
		bool operator!=(const MethodArguments &other) const	{ return !(*this == other);  }
//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}
	protected:
		std::vector<wosh::MethodArgument> args;


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_MethodArguments_H__
