/**
 * @class   wosh::SecurityManager
 * @brief   CoreModule which cares of system security and auditing.
 *
 *
The most common operation performed by a security subsystem is proving a
users identity. In a common example, a user asserts their identity with their
login user-name and password (the step of authentication), and then asserts
(by authorization)

One of the less-considered, but equally important matters that a security subsystem
should take care of is peer authentication. It is readily accepted that the
user must prove their identity to a server, but with a few exceptions (mostly
involving secure websites), users do not expect a server to prove its identity
to them.
Proof of server identity is a very important issue, as soon as any of the
information given by that server needs to be trusted. Often called mutual authentication,
a solution to this problem ensures that the server is trusted not
to provide malicious data.

Once the identities of a server and client has been established, there must be
some way to ensure that only that server or client can continue to communicate
with the other. Otherwise, it may be possible for an impostor to take over
a connection and impersonate one party to the other. (connection hijacking)
The practice of data integrity, also known as signing, typically requires a
cryptographic calculation designed such that both sides can prove these facts:
This signature could only have be performed by the other party, and could only
have been constructed over the data received. If the output of that calculation
does not match the value supplied by the other party, the message must be
considered compromised.



The components of distributed systems, whether object-oriented or not, have to communicate
via a network. If hosts are directlyconnected to public networks, such as the Internet,
any network traffic to or from the host is publicly accessible.
Individuals wishing to launch security attacks can obtain connections to a public network, because there are no restrictions
on connecting machines to a public network. This means that an attacker can use a public
network to send messages to distributed system components in an attempt to compromise
the system security.
An obvious approach to avoiding security problems is to physically separate the network
from the public network.

 --------------------------------------------------------------------------

There are different forms of security threat. [Colouris et al., 1994] classify them into four
different categories and we followthis classification.
 - The first class is concerned with leakage of information to non-authorized parties.
 - The second class includes threats of non-authorized parties modifying information that is stored ormanipulated in a distributed system.
 - The third class is concerned with the unauthorized use of resources, such as network bandwidth, disk space or CPU time.
 - The last, and probablymost serious, threat involves the destruction of information or the distributed system itself by unauthorized parties..

 --------------------------------------------------------------------------

Centralized applications that are written using object-oriented programming languages
often deal with security at a session level. They request users to identify themselves during
an authentication procedure at session start and then they trust that the user will not make
the session available to unauthorized users.
Distributed objects may communicate over insecure networks, where attackers can
openly access network traffic.

Hence, it is not sufficient to authenticate at a session
level but individual requests might have to be authenticated. Moreover, server objects
have to be able to decide whether a user or client that has requested execution of an
operation is authorized to execute that request.




 --------------------------------------------------------------------------
 Auditing
 =========
 SecurityAuditingEngine (implemented within wosh::security)
  monitors security events and may apply countermeasures..
 
 --------------------------------------------------------------------------
 Authorization 
 =============

 idea: un component che fa da trust=> genera securitytoken criptato con dentro le info
 quindi un messaggio da communicator ha le info necessarie x valutare che sessione aprire
 e ereditare le credenziali
 
 gli oggetti hanno permission ma  legata a credenziali?
 hanno anche riferimento e credenziali?
 ---------------------------------------------------------------------------




OTTIMO:  http://msdn.microsoft.com/en-us/library/ms788756.aspx



















 Identity and access management (IAM) remains a challenge in distributed systems.
 
 Security tokens are used to prove one's identity electronically (as in the case of a customer trying to access their bank account). The token is used in addition to or in place of a password to prove that the customer is who they claim to be. The token acts like an electronic key to access something.
 
 Use brokered authentication with a security token (ref http://msdn.microsoft.com/en-us/library/aa480563.aspx)
 
 PAM (Pluggable authentication modules)
  
 ----------------------------------
 s
 
 
 
 
 
 
 
 
 
 
 
 
 specs:
  - support multiple authentication modules (providers)
 
 
 
 *
 *
 *
 *
 * \image html diagrams/SecurityManager.png "wosh::SecurityManager UML diagram"
 *
 *
 * \par Initialization:
 * \msc
SecurityManager,BusManager;
SecurityManager->SecurityManager [label="WoshModule::onKernelInitializing()", URL="\ref wosh::WoshModule::onKernelInitializing()" ];
|||;
SecurityManager=>BusManager [ label="create wosh.Bus.Security", URL="\ref wosh::BusManager::createBus()" ];
SecurityManager<=SecurityManager [ label="connect wosh.Bus.Security", URL="\ref wosh::BusConnector::connect()" ];
 * \endmsc
 *
 *
 * @ingroup CoreModules
 ****************************************************************************
 * @version 0.6.030 $Id: SecurityManager.h 2732 2010-07-06 03:18:11Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! \page page_core
 * \htmlonly <hr/> \endhtmlonly
 * \section page_core_security Security
 *
 * \warning WOSH Security architecture is still under planning.
 *
 * wosh::SecurityManager offers common security-related operations (such as
 * wosh::Permission matching) and generates wosh::SecurityToken objects which
 * are associated with wosh::Object and wosh::User actions (messages).
 *
 * \see wosh::SecurityManager
 * \see wosh::SecurityToken
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR UserS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_SecurityManager_H__
 #define __WOSH_Core_SecurityManager_H__

 #include <woshDefs.h>
 #include <core/WoshModule.h>

 #include <core/Permission.h>
 #include <core/SecurityToken.h>
 #include <core/SecurityFirewall.h>


namespace wosh {


class SecurityAuditingEngine {
 public:
	SecurityAuditingEngine() {}
	virtual ~SecurityAuditingEngine() {}
 };

class SecurityTokenSignature
 {
 public:
	SecurityTokenSignature() {}
	virtual ~SecurityTokenSignature() {}

 public:
	int64 token_id;
	int64 crc32;

};


 #define _SecurityManager_NAME						"SecurityManager"
 #define _SecurityManager_URI						":Security"

 #define _SecurityManager_KEY_EventLogMaxSize		"EventLogMaxSize"
 #define _SecurityManager_KEY_EventLogLocallyOnly	"EventLogLocallyOnly"

 #define _SecurityManager_DEFAULT_EventLogMaxSize	400

 #define _SecurityManager_METHOD_fake_event			"fake_event"
 #define _SecurityManager_METHOD_eventslog			"eventslog"
 #define _SecurityManager_METHOD_clear_log			"clear_log"

class WoshKernel;
class WoshHost;


class SecurityManager : public WoshModule
 {
	WOSH_MODULE(wosh::SecurityManager)

	typedef ListT<wosh::Message*> tMessageList;
	typedef ListT<wosh::SecurityAuditingEngine*> tSecurityAuditingEngineList;

	typedef MapT<int64, wosh::SecurityTokenSignature*> tInt64SecurityTokenSignatureMap;

	typedef MapT<std::string, wosh::SecurityTokenAuthenticator*> tStrSecurityTokenAuthenticatorMap;


	protected:
		/**
		 * @brief  Default constructor. Call base constructor and initialize some properties and hosted methods.
		 * @param parent [in] WoshKernel singleton, passed to WoshModule constructor
		 * @note   Called by WoshKernel only. Static getter (gamma singleton) will return NULL until kernel is created.
		 */
		SecurityManager( const WoshKernel* parent = NULL );

		/**
		 * @brief  Call base class implementation and create the Security Bus (wosh.Bus.Security).
		 * @return result of WoshModule::onKernelInitializing() + bus-creation result
		 */
		WRESULT onKernelInitializing();

		friend class wosh::WoshKernel;

	public:
		/**
		 * @brief  Destructor. Events dataset is destroyed.
		 */
		virtual ~SecurityManager();


/** @name Public methods (bundles)
 * @{
 ******************************************************************************/
	public:
///@ todo how to do AUDITING (tracing which clients accessed which resources and how)
		static bool isTrustedHost( const WoshHost* )  { return false; }
//		static bool evalMessage( const Message& message, const WoshHost* fromHost, , const WoshHost* toHost );

		static WRESULT validate( const WoshHost* source, const WoshHost* target, const NetworkChannel* channel );

		static WRESULT validate( const Message* message, const NetworkChannel* channel );
		static WRESULT validate( const Message* message, const Bus* source, const Bus* target );



		static bool compareMask( short mask_pattern, short operation );

		static bool evalAuthorization( const SecurityToken* security, const Permission* permissions, Permission::PERMISSION_TYPE operation );

		static WRESULT validateToken( const SecurityToken* security );

		// TEMP, TO BE REMOVED
		static SecurityToken* generateToken( const Object* object );
		// TEMP, TO BE REMOVED
		static SecurityToken* generateToken( const std::string& user, const Object* object = NULL );

//		static SecurityAuthentication* generateCredentials( const std::string& user, const std::string& sa_type, const std::string& request_entity_id );

//@}
/** @name Message Events (incoming message)
 * @{
 ******************************************************************************/
	public:
		void busMessage( const Message& message, const Bus* source );
//@}
/** @name IPropertiesProviderListener interface
 * @{
 ******************************************************************************/
 	protected:
		bool updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source );
//@}
/** @name Internal Mehods and Events management
 * @{
 ******************************************************************************/
	private:
		/**
		 * @brief  Access the SecurityManager singleton. Gamma Pattern.
		 *         Static reference is allocated and detroyed by WoshKernel on constructor/destructor respectively
		 * @return pointer to SecurityManager singleton. May be NULL (if kernel has not been allocated yet)
		 */
		static SecurityManager* getInstance();
		/**
		 * @brief  Append a device-event to event-log cache. Reentrant.
		 * @param message [in] the message representing the devic-event (from Device bus, containing wosh::Fact)
		 * @see    eventsLog
		 */
		void logEvent( const Message& message );
//@}
	private:
		Response* mmDoEventsLog( const Request* request );
		Response* mmDoFakeSecurityEvent( const Request* request );
		WRESULT clearEventsLog();

	public:
		static SecurityTokenProviderImplicit& getCredentialImplicit() {
			static SecurityTokenProviderImplicit static_credential_implicit;
			return static_credential_implicit;
		 }
		static SecurityTokenProviderProxy& getCredentialProxy() {
			static SecurityTokenProviderProxy static_credential_proxy;
			return static_credential_proxy;
		 }
	//	friend wosh::BundleGeneric;

	private:
		BusConnector BusSecurity;	///< Bus of Security, network-shared, created by myself on kernel-init

		tSecurityAuditingEngineList auditingEngine;

		static tStrSecurityTokenAuthenticatorMap& getAuthenticators() {
			static tStrSecurityTokenAuthenticatorMap static_authenticators;
			return static_authenticators;
		 }

		static tInt64SecurityTokenSignatureMap& getTokenSignatures() {
			static tInt64SecurityTokenSignatureMap static_signatures;
			return static_signatures;
		 }

		tMessageList eventsLog;		///< cache of last events (Message.Fact) monitored on Security bus.
		long eventLogMaxSize;		///< size of the EventLog cache. 0 will disable logging
		bool eventLogLocallyOnly;	///< when enabled (default), filter only local events

}; // class def

}; // namespace wosh

#endif //__WOSH_Core_SecurityManager_H__
