/** @file    SessionProtocol.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SessionProtocol.cpp 2634 2010-06-09 23:01:02Z alex $
 *  @brief
 * File containing methods for the wosh::communication::SessionProtocol class.
 * The header for this class can be found in SessionProtocol.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/SessionProtocol.h>

 #include <core/ObjectAllocator.h>
 #include <core/Session.h>
 #include <core/DataModels.h>


using namespace std;
namespace wosh {

 WOSH_REGISTER_INTERFACE(wosh::SessionProtocol, "", 1.0, _static_SessionProtocol )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

WRESULT SessionProtocol::postMessage( Message* message, Session* session ) {
	if ( session == NULL ) return WRET_ERR_PARAM;
	if ( message == NULL ) return WRET_ERR_PARAM;
	return session->postMessage(message);
 }

void SessionProtocol::printMessage( Session* session, const std::string& message ) {
	if ( session == NULL ) return;
	session->printMessage(message);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

DataFieldCollector<SessionProtocol>* SessionProtocol::getDataFields() {
	DataFieldCollector<SessionProtocol>* fields = new DataFieldCollector<SessionProtocol>();
	fields->add<const char*, Variant>(&SessionProtocol::getClassName, "Type", 0);
	return fields;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
