/**
 * @class   wosh::SharedMemoryDsm
 * @brief   
 *
 *
 *
 ****************************************************************************
 * @version 0.8.499 $Id: SharedMemoryDsm.h 2665 2010-06-17 20:10:44Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_SharedMemoryDsm_H__
 #define __WOSH_Core_SharedMemoryDsm_H__

 #include <woshDefs.h>
 #include <core/SharedMemory.h>
 #include <core/SharedTransaction.h>
 #include <core/Thread.h>


namespace wosh {

 #define _SharedMemoryDsm_METHOD_transaction	"transaction"

typedef ListT<Transaction*> tTransactionList;


class SharedMemoryDsm :		public ThreadImpl,
							public SharedMemory
 {
	WOSH_CLASSNAME(wosh::SharedMemoryDsm)
	WOSH_CLASS_ABSTRACT(false)

	public:


	public:
		SharedMemoryDsm( const std::string& name = "", Object* parent = NULL );
		virtual ~SharedMemoryDsm();


	public:
		const tTransactionList& getTransactions() const		{ return this->transations; }
		tTransactionList& getTransactions()					{ return this->transations; }

		const tTransactionList& getTransactionsHistory() const	{ return this->transationsHistory; }
		int getTransactionsHistorySize() const					{ return this->transationsHistorySize; }

/** @name Flow control
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT init( unsigned long timeout = LIMIT_ULONG_MAX );
		virtual WRESULT deinit( unsigned long timeout = LIMIT_ULONG_MAX );

//@}
/** @name Setters (reentrant)
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT setMode( AGENT_MODE mode, bool raise_update = true );
		virtual WRESULT setPropagation( UPDATE_PROPAGATION propagation, bool raise_update = true );

//@}
/** @name IBusListener interface
 * @{
 ******************************************************************************/
	public:
		virtual void busMessage( const Message& message, const Bus* source );
//@}
	protected:
		virtual void removeTransaction_( Transaction* transaction );
		virtual WRESULT evaluateTransactionRequest( const Message& message );
		virtual WRESULT evaluateTransactionResponse( const Message& message );

	protected:
		bool initializingThread();
		void runThread();
		void exitingThread();

	protected:
		tTransactionList transations;
		tTransactionList transationsHistory;
		int transationsHistorySize;


}; // class def




}; // namespace wosh

#endif //__WOSH_Core_SharedMemoryDsm_H__
