/** @file    Tree.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Tree.cpp 2551 2010-05-26 21:55:42Z alex $
 *  @brief
 * File containing methods for the wosh::Tree class.
 * The header for this class can be found in Tree.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <core/Tree.h>

 #include <core/ObjectAllocator.h>
 #include <core/Utilities.h>


using namespace std;
namespace wosh {

 WOSH_REGISTER(wosh::Tree, "wosh::ISerializable", 1.01, _static_Tree)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

TreeNode& TreeNode::operator=( const TreeNode& m ) {
	if (this == &m) return *this; // same object?
	// not optimized
	free(true, true);
	if ( m.getContent() == NULL )
		setContent( NULL, true );
	else
		setContent( m.getContent()->clone(), true );

	TreeNodeT<ISerializable*>::ChildConstIterator it;
	const TreeNodeT<ISerializable*>::ChildConstIterator it_end = m.end();
	for( it=m.begin(); it!=it_end; ++it) {
		const TreeNode* trn = reinterpret_cast<const TreeNode*>(*it);
		if ( trn == NULL ) continue;
		TreeNode* trc = trn->clone();
		trc->setParent(this, true);
	 }
	return *this;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool TreeNode::updateHash( DataHashContext* hash_context ) const {
	if ( hash_context == NULL ) return false;
	bool ret = true;
	if ( this->content != NULL )
		ret = this->content->updateHash(hash_context) & ret;
	TreeNodeT<ISerializable*>::ChildConstIterator it;
	const TreeNodeT<ISerializable*>::ChildConstIterator it_end = this->children.end();
	for( it=this->children.begin(); it!=it_end; ++it) {
		const TreeNode* trn = reinterpret_cast<const TreeNode*>(*it);
		if ( trn == NULL ) continue;
		ret = trn->updateHash(hash_context) & ret;
	 }
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Tree::updateHash( DataHashContext* hash_context ) const {
	if ( hash_context == NULL ) return false;
	bool ret = true;
	ret = hash_context->update( this->tree_name ) & ret;
	if ( this->rootNode != NULL )
		ret = this->rootNode->updateHash(hash_context) & ret;
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
