/**
 * @file    Tree.h
 * @brief   Includes container-classes representing a tree of objects.
 ****************************************************************************
 * @version $Id: Tree.h 2793 2010-07-23 12:20:47Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Tree_H__
 #define __WOSH_Core_Tree_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>
 #include <core/Containers.h>


namespace wosh {

/**
 * @class   wosh::TreeNode
 * @brief   
 ****************************************************************************
 *
 * @ingroup DataModels
 ****************************************************************************
 * @version 0.8.384 $Id $
 * @author  Alessandro Polo
 ****************************************************************************/

class TreeNode : public TreeNodeT<ISerializable*>, public ISerializable
 {
	WOSH_SERIALIZABLE(wosh::TreeNode)
	WOSH_COMPARE_RTTI(wosh::TreeNode)

	//typedef ListT<TreeNode*> tTreeNodeList;
	//typedef tTreeNodeList::Iterator ChildIterator;
	//typedef tTreeNodeList::ConstIterator ChildConstIterator;

	public:
		TreeNode( TreeNode* node_parent, ISerializable* data = NULL )
			: TreeNodeT<ISerializable*>(node_parent, data) { }

		TreeNode( const TreeNode& m )
			: TreeNodeT<ISerializable*>(NULL, NULL) {
			*this = m;
		 }

		virtual ~TreeNode() {
			free(true, true);
		 }

	public:

/*		using TreeNodeT<ISerializable*>::addChild;

		inline ChildConstIterator begin() const		{ return this->children.begin(); }
		inline ChildIterator begin()				{ return this->children.begin(); }

		inline ChildConstIterator end() const		{ return this->children.end(); }
		inline ChildIterator end()					{ return this->children.end(); }
*/

/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual TreeNode& operator=( const TreeNode& m );

		virtual bool operator==( const TreeNode& ) const {
			return false;
		 }

		virtual bool operator!=( const TreeNode& other ) const	{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const {
			std::string temp;
			return temp;
		 }
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}



}; // class def



/**
 * @class   wosh::Tree
 * @brief   
 ****************************************************************************
 *
 * @ingroup DataModels
 ****************************************************************************
 * @version 0.8.384 $Id $
 * @author  Alessandro Polo
 ****************************************************************************/

class Tree : public TreeT<TreeNode>, public ISerializable
 {
	WOSH_SERIALIZABLE(wosh::Tree)
	WOSH_COMPARE_RTTI(wosh::Tree)


	public:
		/**
		 * @brief  Empty constructor.
		 */
		Tree() : TreeT<TreeNode>() { }
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		Tree( const Tree& m ) : TreeT<TreeNode>(m) {
			*this = m;
		 }
		virtual ~Tree() {
			free(true, true);
		 }

	public:

		using TreeT<TreeNode>::getRoot;

/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual Tree& operator=( const Tree& m ) {
			if (this == &m) return *this; // same object?
			// not optimized
			if ( m.getRoot() == NULL )
				setRoot( NULL, true );
			else
				setRoot( m.getRoot()->clone(), true );
			return *this;
		 }

		virtual bool operator==( const Tree& other ) const {
			if ( other.rootNode == NULL && this->rootNode == NULL )
				return true;
			if ( other.rootNode != NULL && this->rootNode != NULL )
				return (*other.rootNode == *this->rootNode);
			return false;
		 }

		virtual bool operator!=( const Tree& other ) const	{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const {
			std::string temp;
			return temp;
		 }
		virtual bool updateHash( DataHashContext* hash_context ) const;

//@}

}; // class def



}; // namespace wosh

#endif //__WOSH_Core_Tree_H__
