/**
 * @file    posix/processes.h
 * @brief   multi-platform implementation of process-related functions
 *          for POSIX systems.
 *
 ****************************************************************************
 * @version $Id: processes.h 2324 2010-04-22 18:02:17Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_POSIX_Processes_H__
 #define __WOSH_Core_POSIX_Processes_H__

 #include <map>
 #include <string>


/**
 * @brief  Retrieve PID of current process.
 * @return returns the process ID of the current process.
 * @note   using pid_t getpid(void);
 */
long getCurrentProcessPID();

/**
 * @brief  Retrieve PID of parent process.
 * @return returns the process ID of the parent of the current process.
 * @note   using pid_t getppid(void);
 */
long getParentProcessPID();

/**
 * @brief  Retrieve process-info, using \c ios against '/proc/self/stat'
 * @param pid [in] actually not supported
 * @param fields [out] map of fields=>value pairs
 * @return false when retrieving failed; true else
 * @note   works on some platforms, only SELF process
 */
bool retrieveProcessInfo( long pid, std::map<std::string, std::string>& fields );


#endif //__WOSH_Core_POSIX_Processes_H__
