/**
 * @file    posix/thread.h
 * @brief   
 *
 ****************************************************************************
 * @version $Id: pthread_thread.h 2628 2010-06-08 23:30:40Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_POSIX_pthread_thread_H__
 #define __WOSH_Core_POSIX_pthread_thread_H__

 #include <core/Thread.h>
 #include <pthread.h>


void* ThreadImpl_PThread_runner_helper(void* ptr);

class ThreadImpl_PThread : public wosh::Thread
 {
	WOSH_CLASSNAME(wosh::ThreadImpl) // faking classname cos of typedef
	WOSH_CLASS_ABSTRACT(true)

	public:
		/**
		 * @brief   Initialize thread and register thread to global list
		 * @param owner [in] owner of the thread, may be NULL. Used for debug/log purpose (and not instance-ownership)
		 * @param threadName [in] optional name of the thread, may be empty.. Used for debug/log purpose
		 * @see     wosh::ThreadManager
		 */
		ThreadImpl_PThread( const wosh::Object* owner, const std::string& threadName = "" );
		/**
		 * @brief   unregister thread from global list
		 * @see     wosh::ThreadManager
		 */
		virtual ~ThreadImpl_PThread();

	public:
		virtual WRESULT startThread( unsigned long synch_timeout = LIMIT_ULONG_MAX );
		virtual WRESULT quitThread( unsigned long synch_timeout = LIMIT_ULONG_MAX );
		bool waitThread( unsigned long time = LIMIT_ULONG_MAX );
		void terminateThread();

	public:
		static inline void sleepForSec( int sec )				{ sleep(sec); }
		static inline void sleepForMSec( int msec )				{
		//msleep(msec);
		}

	protected:
		bool isThreadRunningInternal() const;

	private:
		void run();

		pthread_t p_thread;

		friend void* ThreadImpl_PThread_runner_helper(void* ptr);

}; // class def

#endif //__WOSH_Core_POSIX_pthread_thread_H__
