/** @file    win32/desktop.cpp
 *  @author  Alessandro Polo
 *  @version $Id: desktop.cpp 2733 2010-07-06 03:48:05Z alex $
 *  @brief   multi-platform implementation of desktop/user-related functions.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include "desktop.h"
#include "utils.h"

#include <windows.h>

using namespace std;

bool getComputerName( std::string& computer_name ) {
	DWORD lpnSize = 64;
	TCHAR lpBuffer[64+1];
	memset(lpBuffer, 0, 64+1);
	BOOL ret = GetComputerName( lpBuffer, &lpnSize );
	if ( ret == FALSE ) return false;
#ifndef UNICODE
	computer_name.assign( lpBuffer );
#else
	ConvertToString(lpBuffer, computer_name);
#endif
	return true;
 }

bool getOSName( std::string& os_name ) {
	DWORD dwVersion = 0; DWORD dwBuild = 0;
	DWORD dwMajorVersion = 0; DWORD dwMinorVersion = 0; 
	dwVersion = GetVersion();
	dwMajorVersion = (DWORD)(LOBYTE(LOWORD(dwVersion)));
	dwMinorVersion = (DWORD)(HIBYTE(LOWORD(dwVersion)));
	if (dwVersion < 0x80000000) dwBuild = (DWORD)(HIWORD(dwVersion));
	char buffer[47];
	sprintf_s(buffer, 46, "Windows %d.%d (%d)", dwMajorVersion, dwMinorVersion, dwBuild );
	os_name.assign(buffer);
	return true;
 }

bool getSystemUserName( std::string& username ) {
	DWORD lpnSize = 64;
	TCHAR lpBuffer[64+1];
	memset(lpBuffer, 0, 64+1);
	BOOL ret = GetUserName( lpBuffer, &lpnSize );
	if ( ret == FALSE ) return false;
#ifndef UNICODE
	username.assign( lpBuffer );
#else
	ConvertToString(lpBuffer, username);
#endif
	return true;
 }

bool getMouseLocation( long& x, long& y ) {
	POINT mouseLoc;
	BOOL ret = GetCursorPos( &mouseLoc );
	if ( ret == FALSE ) return false;
	x = mouseLoc.x;
	y = mouseLoc.y;
	return true;
 }

