/**
 * @class   wosh::automations::AutomationMessage
 * @brief   wosh::automations::Automation implementation providing a wosh::Message based
 *          automation, triggered by one or more messages (as wosh::MessageFilter)
 *          and generating one or more effects (as wosh::Message).
 *
 *
 ****************************************************************************
 * @version $Id: AutomationMessage.h 2557 2010-05-26 23:29:03Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Automations_AutomationMessage_H__
 #define __WOSH_Automations_AutomationMessage_H__

 #include <framework/automations/Automation.h>

 #include <core/Message.h>
 #include <core/MessageFilter.h>


namespace wosh {
 namespace automations {

 #define _AutomationMessage_VERSION			0.900


class AutomationMessage : public Automation
 {
	WOSH_SERIALIZABLE(wosh::automations::AutomationMessage)
	WOSH_COMPARE_RTTI(wosh::automations::AutomationMessage)

	typedef ListT<wosh::MessageFilter*> tMessageFilterList;
	typedef ListT<wosh::Message*> tMessageList;


	public:
		/**
		 * @brief  Empty constructor.
		 */
		AutomationMessage();
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		AutomationMessage( const AutomationMessage& m );
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~AutomationMessage();


	public:
		virtual WRESULT addTigger( MessageFilter* trigger );
		virtual WRESULT addEffect( Message* effect );

//@}
/** @name Automation Interface - 
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT execute( const Message* message = NULL, int64 time = 0 );

		virtual bool isTrigger( const Message* message = NULL, int64 time = 0 ) const;

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual AutomationMessage& operator=(const AutomationMessage& m);
		virtual bool operator==(const AutomationMessage &other) const;
//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}
	protected:
		virtual WRESULT replaceVariables( Message* effect, const Message* trigger );
		virtual bool replaceVariable( std::string& var, const Message* trigger );

		virtual WRESULT addTigger_( MessageFilter* trigger );
		virtual WRESULT addEffect_( Message* effect );

	protected:
		tMessageFilterList triggers;		///< list of the triggers
		tMessageList effects;				///< list of the effects

}; // class def

 }; // namespace automations
}; // namespace wosh

#endif //__WOSH_Automations_AutomationMessage_H__
