/**
 * @class   wosh::building::Fan
 * @brief   BuildingObject representing a generic (real) light.
 *          This class provides standard Methods and properties to 
 *          control the mapped wosh::interfaces::devices::Switch or 
 *          wosh::interfaces::devices::Dimmer
 *          wosh::building::Fan inherits wosh::building::Appliance
 *          and it's usually a child of wosh::building::Room
 *
 ****************************************************************************
 * @version 0.8.399 $Id: Fan.h 2380 2010-04-29 00:21:21Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Building_Fan_H__
 #define __WOSH_Building_Fan_H__

 #include <framework/building/Appliance.h>


namespace wosh {
 namespace building {

 #define _Fan_VERSION				0.8399


class Fan : public Appliance
 {
	WOSH_BUILDINGOBJECT(wosh::building::Fan)

	public:
		/**
		 * @brief  Empty constructor.
		 */
		Fan() : Appliance() {
			BuildingObject::setName( Utilities::format("wosh.building.Fan#%ld", Utilities::randomLong()) );
		 }
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		Fan( const Fan& m ) : Appliance(m) { }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Fan() { }


}; // class def

 }; // namespace building
}; // namespace wosh

#endif //__WOSH_Building_Fan_H__
