/** @file    FileMessage.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FileMessage.cpp 2621 2010-06-04 12:23:48Z alex $
 *  @brief
 * File containing methods for the wosh::communication::FileMessage class.
 * The header for this class can be found in FileMessage.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <framework/communication/FileMessage.h>

 #include <core/ObjectAllocator.h>
 #include <core/Utilities.h>


using namespace std;
namespace wosh {
 namespace communication {

 WOSH_REGISTER(wosh::communication::FileMessage, "wosh::Notification", 1.0, _static_FileMessage )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

FileMessage::FileMessage()
	: Notification() {
	clear();
 }

FileMessage::FileMessage( const std::string& filename )
	: Notification() {
	clear();
	this->res_path = filename;
 }

FileMessage::FileMessage( const FileMessage& m )
	: Notification(m) {
	clear();
	*this = m;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// CONFIGURATION

void FileMessage::clear() {
	Notification::clear();
	this->protocol = Notification::PROTOCOL_HUMAN;
	this->res_path.clear();
 }

FileMessage& FileMessage::operator=( const FileMessage& m ) {
	if (this == &m) return *this; // same object?
	(Notification&)*this = (const Notification&)m;
	this->res_path = m.res_path;
	this->optPlain = m.optPlain;
	return *this;
 }

bool FileMessage::operator==( const FileMessage &other ) const {
	if ( (Notification&)*this != (const Notification&)other )
		return false;
	if ( this->res_path != other.res_path )
		return false;
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// DEBUG & INFO
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FileMessage::fromNotification( const Notification* notif ) {
	if ( notif == NULL ) return false;

	setFlowID( notif->getFlowID() );

	this->sender_id = notif->getSender_ID();
	this->sender_user = notif->getSender_User();
	this->recipent_id = notif->getRecipent_ID();
	this->recipent_user = notif->getRecipent_User();
	this->recipent_group = notif->getRecipent_Group();

	// still keep text
	this->optPlain = notif->getPlain();

	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// SERIALIZATION

std::string FileMessage::toString() const {
	return Utilities::format("Notification[FileMessage]#%ld", this->uid);
 }

bool FileMessage::updateHash( DataHashContext* hash_context ) const {
	if ( hash_context == NULL ) return false;
	bool ret = Notification::updateHash( hash_context );
	ret = this->res_path.updateHash( hash_context ) & ret;
	ret = hash_context->update( this->optPlain ) & ret;
	return ret;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// SERIALIZATION
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace communication
}; // namespace wosh
