/**
 * @class   wosh::communication::FileMessage
 * @brief   
 ****************************************************************************
 * @version $Id: FileMessage.h 2556 2010-05-26 23:16:48Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_FileMessage_H__
 #define __WOSH_Communication_FileMessage_H__

 #include <woshDefs.h>
 #include <core/Notification.h>
 #include <core/ResourcePath.h>


namespace wosh {
 namespace communication {

class FileMessage : public Notification
 {
	WOSH_SERIALIZABLE(wosh::communication::FileMessage)
	WOSH_COMPARE_RTTI(wosh::communication::FileMessage)

	public:


	public:
		/**
		 * @brief  Empty constructor.
		 */
		FileMessage();
		/**
		 * @brief  Constructor with initializer(s), provided for convenience.
		 * @param filename [in] the (wosh) path of the audio file
		 * @see    setFile()
		 */
		FileMessage( const std::string& filename );
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		FileMessage( const FileMessage& m );
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~FileMessage() { }

	public:
		virtual bool fromNotification( const Notification* notif );


	public:

		virtual const std::string& getPlain() const				{ return this->optPlain; }

		virtual std::string getSummary( int maxsize = 1023 ) const { (void)maxsize; return this->optPlain;; }

		virtual void setPlain( const std::string& text )		{ this->optPlain = text; }


//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setFile( const ResourcePath& path )				{ this->res_path = path; }


//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		const ResourcePath& getFile() const						{ return this->res_path; }
		ResourcePath& getFile()									{ return this->res_path; }

		virtual Notification::TYPE getNotificationType() const	{ return Notification::TYPE_DATA; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual FileMessage& operator=(const FileMessage& m);

		virtual bool operator==(const FileMessage &other) const;
		virtual bool operator!=(const FileMessage &other) const { return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}
	protected:
		virtual void clear();

	protected:
		ResourcePath res_path;
		std::string optPlain;


}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_FileMessage_H__
