/**
 * @class   wosh::communication::ImageMessage
 * @brief   wosh::Notification representing an image message (based on image-file).
 *
 * @see     wosh::interfaces::devices::PlayerAudio
 ****************************************************************************
 * @version $Id: ImageMessage.h 2556 2010-05-26 23:16:48Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_ImageMessage_H__
 #define __WOSH_Communication_ImageMessage_H__

 #include <woshDefs.h>
 #include <framework/communication/FileMessage.h>


namespace wosh {
 namespace communication {

class ImageMessage : public wosh::communication::FileMessage
 {
	WOSH_SERIALIZABLE(wosh::communication::ImageMessage)
	WOSH_COMPARE_RTTI(wosh::communication::ImageMessage)

	public:
		/**
		 * @brief  Empty constructor.
		 */
		ImageMessage();
		/**
		 * @brief  Constructor with initializer(s), provided for convenience.
		 * @param filename [in] the (wosh) path of the image
		 * @see    setFile()
		 */
		ImageMessage( const std::string& filename );
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		ImageMessage( const ImageMessage& m );
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~ImageMessage() { }

	public:
		virtual bool fromNotification( const Notification* notif );


//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		virtual Notification::TYPE getNotificationType() const { return Notification::TYPE_IMAGE; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual ImageMessage& operator=(const ImageMessage& m);

		virtual bool operator==(const ImageMessage &other) const;
		virtual bool operator!=(const ImageMessage &other) const { return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}
	protected:
		virtual void clear();

	protected:


}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_ImageMessage_H__
