/** @file    SessionIM.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SessionIM.cpp 2732 2010-07-06 03:18:11Z alex $
 *  @brief
 * File containing methods for the wosh::communication::SessionIM class.
 * The header for this class can be found in SessionIM.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/communication/SessionIM.h>

 #include <core/ObjectAllocator.h>
 #include <core/Utilities.h>
 #include <core/WoshKernel.h>
 #include <core/Request.h>
 #include <core/SecurityManager.h>
 #include <core/SessionProtocol.h>

 #include <framework/communication/TextMessage.h>

 #define _ENDLINE	"\r\n"

 
using namespace std;
using namespace wosh::communication;
namespace wosh {
 namespace communication {

 WOSH_REGISTER(wosh::communication::SessionIM, "wosh::Session", 1.0, _static_SessionIM )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SessionIM::SessionIM()
	: Session() {
	this->timeout_ts = 0;
 }

SessionIM::~SessionIM() {
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionIM::open( unsigned long )
 {
	if ( getProtocol() == NULL )
		return WRET_ERR_ILLEGAL_USE;

	return login( getUsername(), new wosh::SecurityTokenProviderProxy() );
 }

WRESULT SessionIM::close( unsigned long ) {
	return logout();
 }

void SessionIM::busMessage( const Message& message, const Bus* source ) {
	if ( message.isEmpty() ) return;
	if ( message.getContent()->isNotification() ) {
		const Notification* notif = message.getContent()->asNotification();
		if ( notif->getRecipent_User() == "wosh" ) {
			parseInput( notif->getPlain() );
			return;
		 }
	 }
	Session::busMessage( message, source );
 }

void SessionIM::printMessage( const std::string& message ) {
	TextMessage* txtmsg = new TextMessage();
	txtmsg->setSender_User( "wosh" );
	txtmsg->setRecipent_User( getUsername() );
	txtmsg->setProtocol( Notification::PROTOCOL_SHELL );
	txtmsg->setLanguage( "en_GB" );
	txtmsg->setPlain( message );
	txtmsg->setPreferredCommunicator(getCreatorURI());
	Message* msg_notif = new Message(txtmsg);
	msg_notif->getDestination().fromString("*:Communication");
	WRESULT sent = postMessage( msg_notif );
	(void)sent;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace communication
}; // namespace wosh
