/**
 * @class   wosh::communication::SessionProtocolDefault
 * @brief   SessionProtocol using the shell syntax, very similar to POSIX.
 *
 * @see     wosh::Session
 * @see     wosh::SessionManager
 ****************************************************************************
 * @version $Id: SessionProtocolDefault.h 2488 2010-05-21 01:48:27Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_SessionProtocolDefault_H__
 #define __WOSH_Communication_SessionProtocolDefault_H__

 #include <core/SessionProtocol.h>
 #include <framework/communication/ShellCommand.h>

 #include <map>


namespace wosh {
 class List;
 class Table;
 class Tree;
 class TreeNode;
 namespace communication {

#define _SessionProtocolDefault_VERSION		"0.8.384"

class SessionProtocolDefault : public SessionProtocol
 {
	WOSH_CLASSNAME(wosh::communication::SessionProtocolDefault)
	WOSH_CLASS_ABSTRACT(false)

	public:
		SessionProtocolDefault();
		virtual ~SessionProtocolDefault();

	public:
		std::string buildPrompt( Session* session ) const;

		WRESULT parseInput( const std::string& dataline, Session* source_session );
		WRESULT parseShellCommand( ShellCommand* command, Session* source_session );

		WRESULT renderMessage( const Message* message, const Bus* source, Session* source_session );

	protected:
		void vprintMessage( Session* session, const char* format, ... );


	protected:
		std::map<std::string, ShellCommand*> aliases;
		ShellCommand lastCommand;


}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_SessionProtocolDefault_H__
