/**
 * @file    framework/entertainment.h
 * @brief   Include all components of the entertainment framework
 ****************************************************************************
 * @version 0.6.030 $Id: entertainment.h 2258 2010-04-02 13:02:21Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! @namespace wosh::entertainment
 * @brief   Contains classes regarding Entertainment. Representing media objects, playlist and more.
 * @see     Refer to wosh::services::MediaDirectorBundle
 * @see     \ref page_framework_entertainment
 *
 ****************************************************************************/
/*! \page page_framework
 * \htmlonly <hr/> \endhtmlonly
 * \section page_framework_entertainment WOSH Entertainment Framework
 *
 * WOSH Entertainment namespace is designed to define interfaces and provide
 * a set of standard implementations for multimedia functionalities.
 * 
 * The smallest block is wosh::entertainment::MediaTrack class, which holds
 * information of a generic media resource: the path as wosh::ResourcePath,
 * media information are saved in wosh::PropertiesProvider.
 * Media fields are filled by the playing service/device, so they might be
 * missing.
 * 
 * Everybody knows about playlist, wosh::entertainment::MediaList is the
 * representation of a generic playlist. Implementation is based on wosh::List
 * template of MediaTrack* objects, moreover it holds properties (wosh::PropertiesProvider)
 * associated with the playlist itself (for example, used to save album name).
 *
 *
 * While a generic \c Player service/device outputs to a specific device (output),
 * MediaDirector service and WOSH itself support multi-zone management.
 *
 * Since there may be multiple zone within a wosh::building::Room (or generic location)
 * but users might want more device to work synchronously, another scope of
 * 'location' must be defined: wosh::entertainment::MediaZone.
 *
 * MediaZones enable also media mobility, managed by MediaDirector.
 *
 * [..]
 *
 * \warning wosh::entertainment namespace and MediaDirector service are actaully
 *          under development.
 *
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_FrameWork_Entertainment_H__
 #define __WOSH_FrameWork_Entertainment_H__

 #include <woshDefs.h>

namespace wosh {
 namespace entertainment { };
 };

 #include <framework/entertainment/MediaList.h>
 #include <framework/entertainment/MediaRecord.h>
 #include <framework/entertainment/MediaTrack.h>
 #include <framework/entertainment/MediaZone.h>


#endif //__WOSH_FrameWork_Entertainment_H__
