/**
 * @file    EntertainmentEvents.h
 * @brief   
 *
 *
 ****************************************************************************
 * @version $Id: EntertainmentEvents.h 2683 2010-06-21 09:17:31Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Entertainment_EntertainmentEvents_H__
 #define __WOSH_Entertainment_EntertainmentEvents_H__

 #include <woshDefs.h>
 #include <core/Fact.h>
 #include <interfaces/devices/PlayerAudio.h>

 
namespace wosh {
 namespace entertainment {

class PlayerAudioEvent_StateChanged : public wosh::Fact
 {
	WOSH_SERIALIZABLE(wosh::entertainment::PlayerAudioEvent_StateChanged)

	public:
		PlayerAudioEvent_StateChanged()
			: wosh::Fact(_Player_EVENT_PlayStateChanged) { }

		PlayerAudioEvent_StateChanged( int playstate )
			: wosh::Fact(_Player_EVENT_PlayStateChanged, new Variant(playstate) ) { }

		PlayerAudioEvent_StateChanged( int playstate, int volume, bool muted = false );

		virtual ~PlayerAudioEvent_StateChanged() { }

	public:
		int getPlayState() const;
		int getVolume() const;
		int getMuted() const;

		inline const char* getPlayStateAsString() const {
			return getPlayStateAsString( getPlayState() );
		 }

		static const char* getPlayStateAsString( int playstate );
		static int getPlayStateFromString( const char* playstate );

}; // class def


class MediaTrack;

class PlayerAudioEvent_MediaTrackChanged : public wosh::Fact
 {
	WOSH_SERIALIZABLE(wosh::entertainment::PlayerAudioEvent_MediaTrackChanged)

	public:
		PlayerAudioEvent_MediaTrackChanged()
			: wosh::Fact(_Player_EVENT_MediaTrackChanged) { }

		PlayerAudioEvent_MediaTrackChanged( const MediaTrack* track );

		virtual ~PlayerAudioEvent_MediaTrackChanged() { }

	public:


}; // class def



class MediaList;

class PlayerAudioEvent_MediaListChanged : public wosh::Fact
 {
	WOSH_SERIALIZABLE(wosh::entertainment::PlayerAudioEvent_MediaListChanged)

	public:
		PlayerAudioEvent_MediaListChanged()
			: wosh::Fact(_Player_EVENT_MediaListChanged) { }

		PlayerAudioEvent_MediaListChanged( const MediaList* list );

		virtual ~PlayerAudioEvent_MediaListChanged() { }

	public:


}; // class def




 }; // namespace entertainment
}; // namespace wosh


#endif //__WOSH_Entertainment_EntertainmentEvents_H__
