/**
 * @class   wosh::gui::FormContainer
 * @brief   
 *          
 *          
 *
 ****************************************************************************
 * @version $Id: FormContainer.h 2184 2010-03-20 13:19:23Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormContainer_H__
 #define __WOSH_Gui_FormContainer_H__

 #include <framework/gui/IWidget.h>

 #include <QtGui/QWidget>
 #include <QtGui/QTabWidget>
 #include <QtGui/QVBoxLayout>


namespace wosh {
 namespace gui {


class FormContainer :  public IWidgetGeneric
 {
	Q_OBJECT


	public:
		FormContainer( QWidget *parent = NULL ) : IWidgetGeneric(parent) {
			this->setObjectName(QString::fromUtf8("FormContainer"));
			this->resize(500, 377);

			this->verticalLayout = new QVBoxLayout(this);
			this->verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
			this->verticalLayout->setSpacing(3);
			this->verticalLayout->setMargin(0);

			this->tabWidget = new QTabWidget(this);
			this->tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
			this->tabWidget->setTabPosition(QTabWidget::South);
			//QT4.1 not supported this->tabWidget->setDocumentMode(true);

			this->verticalLayout->addWidget(tabWidget);
		 }
		virtual ~FormContainer() { }


	public:
		virtual WRESULT init() {
			WRESULT ret = WRET_OK;
			for(int i=0; i<this->iWidgets.size(); i++) {
				if ( this->iWidgets.at(i) == NULL ) continue;
				ret += this->iWidgets.at(i)->init();
			 }
			return ret;
		 }

	public:
		virtual void busMessage( const Message& , const Bus* ) { }

		virtual void refreshAll() {
			for(int i=0; i<this->iWidgets.size(); i++) {
				if ( this->iWidgets.at(i) == NULL ) continue;
				this->iWidgets.at(i)->refreshAll();
			 }
		 }

	public:
		virtual WRESULT addIWidget( IWidget* iwidget, bool just_add = false ) {
			if ( iwidget == NULL ) return WRET_ERR_PARAM;
			if ( iwidget->asQWidget() == NULL ) return WRET_ERR_PARAM;

			this->iWidgets.append(iwidget);
			if ( !just_add ) {
				iwidget->setTargetURI( this->getTargetURI() );
			 }

			QWidget* widget = iwidget->asQWidget();
			if ( widget == NULL )
				return WRET_ERR_INTERNAL;

			this->tabWidget->addTab(widget, widget->windowTitle());
			this->tabWidget->setCurrentIndex(this->tabWidget->count()-1);

			return WRET_OK;
		 }

		virtual void setTargetURI( const URI& target_uri ) {
			IWidgetGeneric::setTargetURI( target_uri );
			for(int i=0; i<this->iWidgets.size(); i++) {
				if ( this->iWidgets.at(i) == NULL ) continue;
				this->iWidgets.at(i)->setTargetURI( target_uri );
			 }
		 }

	protected:
		QVBoxLayout *verticalLayout;
		QTabWidget *tabWidget;

		QList<IWidget*> iWidgets;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormContainer_H__
