/**
 * @class   wosh::gui::FormCommunicationManager
 * @brief   Graphic interface of wosh::interfaces::services::CommunicationManager bundle interface.
 *
 * Official implementation is wosh::services::CommunicationManagerBundle
 *
 * @ingroup Gui
 ****************************************************************************
 * @version $Id: FormCommunicationManager.h 2649 2010-06-11 09:18:58Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormCommunicationManager_H__
 #define __WOSH_Gui_FormCommunicationManager_H__

 #include <framework/gui/IWidget.h>
 #include <framework/gui/WoshTableModel.h>
 #include <QtGui/QWidget>

 #include <interfaces/services/CommunicationManager.h>


namespace Ui {
    class FormCommunicationManager;
}
using namespace std;
namespace wosh {
 namespace gui {


class FormCommunicationManager :  public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormCommunicationManager)
	WOSH_CLASS_ABSTRACT(false)


	public:
		FormCommunicationManager(QWidget *parent = 0);
		virtual ~FormCommunicationManager();

	public:
		virtual void busMessage( const Message& message, const Bus* source );

	public slots:
		virtual void refreshAll();

	protected slots:
		void cmbSendMessageType_activated( const QString& value  );
		void btnSendMessageSend_clicked();
		void btnSendMessageReset_clicked();
		void btnSendMessageOptions_clicked();

	protected:
		void changeEvent(QEvent *e);

		Notification* getCurrentNotification();


	private:
		Ui::FormCommunicationManager *m_ui;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormCommunicationManager_H__
