/**
 * @class   wosh::gui::FormDiscoveryUdp
 * @brief   Graphic interface of wosh::interfaces::services::DiscoveryUdp bundle interface.
 *
 * Official implementation is wosh::services::DiscoveryUdpBundle
 *
 * @ingroup Gui
 ****************************************************************************
 * @version $Id: FormDiscoveryUdp.h 2488 2010-05-21 01:48:27Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormDiscoveryUdp_H__
 #define __WOSH_Gui_FormDiscoveryUdp_H__

 #include <framework/gui/IWidget.h>
 #include <framework/gui/widgets/wCore_PropertiesProvider.h>
 #include <framework/gui/widgets/wCore_Variant.h>
 #include <core/MethodsCommon.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <interfaces/services/DiscoveryUdp.h>

 #include <QtGui/QWidget>
 #include "ui_FormDiscoveryUdp.h"

using namespace std;
namespace wosh {
 namespace gui {


class FormDiscoveryUdp :  public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormDiscoveryUdp)
	WOSH_CLASS_ABSTRACT(false)


	public:
		FormDiscoveryUdp(QWidget *parent = 0) : IWidgetGeneric(parent) {
			this->m_ui = new Ui::FormDiscoveryUdp();
			this->m_ui->setupUi(this);

			this->setWindowTitle("DiscoveryUdp");
			this->getWidgetURI().setName( Utilities::format("FormDiscoveryUdp#%10lu", Utilities::randomLong()) );

			connect(this->m_ui->wUpBarConfiguration, SIGNAL(updateClicked()), this, SLOT(refreshAll()));

			this->propMapperCfg = new PropertiesMapper(this, true);
			connect(this->propMapperCfg, SIGNAL(send_request(const QString&, ISerializable*)), this, SLOT(executeRemoteMethod(const QString&, ISerializable*)));
			this->propMapperCfg->set(_DiscoveryUdp_KEY_BindAddressPort, this->m_ui->editorBindAddress);
			this->propMapperCfg->set(_DiscoveryUdp_KEY_BroadcastAddressPort, this->m_ui->editorBroadcastAddresses);
			this->propMapperCfg->set(_Discovery_KEY_NotifyFrequency, this->m_ui->sliderNotifyFrequency);

			mapMethodButton(_Discovery_METHOD_discover, this->m_ui->btnDiscover);
		 }
 
		virtual ~FormDiscoveryUdp() { }

	public:

		virtual void busMessage( const Message& message, const Bus* ) {
			if ( message.isEmpty() ) return;
			if ( !message.getContent()->isResponse() ) return;
			if ( !isMessageForMeOrFromTarget(message) )
				return;
			const wosh::Response* response = message.getContent()->asResponse();
			if ( this->propMapperCfg->evalResponse(response) )
				this->m_ui->wUpBarConfiguration->sayReady();
			promptMethodResponse(message);
		 }

	public slots:

		virtual void refreshAll() {
			refreshConfiguration();
		 }
 
		void refreshConfiguration() {
			List* dlProps = this->propMapperCfg->createKeysList();
			executeRemoteMethod( _METHOD_GetProperties, dlProps );
			this->m_ui->wUpBarConfiguration->sayRetrieving();
		 }

	protected:
		PropertiesMapper* propMapperCfg;

	private:
		Ui::FormDiscoveryUdp *m_ui;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormDiscoveryUdp_H__
