/** @file    FormHeyu.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormHeyu.cpp 2649 2010-06-11 09:18:58Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormHeyu class.
 * The header for this class can be found in FormHeyu.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormHeyu.h"
 #include "ui_FormHeyu.h"

 #include <core/UserManager.h>
 #include <core/WoshKernel.h>
 #include <core/MethodsCommon.h>
 #include <core/Request.h>
 #include <core/Response.h>

 #include <interfaces/services/Heyu.h>

 #include <QtGui>
 #include <framework/gui/GuiUtilities.h>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormHeyu, "wosh::gui::IWidgetGeneric", 1.0, _static_FormHeyu )
 WOSH_GUI_MAP(wosh::gui::FormHeyu, "wosh::interfaces::services::Heyu", 0.0, _static_Map_FormHeyu )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormHeyu::FormHeyu(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormHeyu();
	this->m_ui->setupUi(this);

	connect(this->m_ui->tableView, SIGNAL(postMessage(Message*)),this, SLOT(postMessage(Message*)) );

	this->setWindowTitle("Heyu");
	this->getWidgetURI().setName( Utilities::format("FormHeyu#%10lu", Utilities::randomLong()) );
	this->getTargetURI().fromString(":Bundles/Heyu");

	mapMethodButton(_Heyu_METHOD_update_cache, this->m_ui->btnForceUpdate);

	connect(this->m_ui->btnRunCommand, SIGNAL(clicked()), this, SLOT(btnRunCommand_clicked()));
 }

FormHeyu::~FormHeyu()
 {
	delete this->m_ui;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormHeyu::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormHeyu::refreshAll() {
	this->setWindowTitle( QString("Heyu [%1]").arg(this->getTargetURI().getKernelName().c_str()) );
	this->m_ui->tableView->refreshData();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormHeyu::btnRunCommand_clicked() {
	if ( this->m_ui->txtCommand->text().isEmpty() ) return;
	executeRemoteMethod( _Heyu_METHOD_eval_cmd, new Variant( string(this->m_ui->txtCommand->text().toLatin1().data()) ) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormHeyu::busMessage( const Message& message, const Bus* )
 {
	if ( !isMessageForMeOrFromTarget(message) )
		return;

	if ( message.getContent()->isResponse() ) {
		const wosh::Response* response = message.getContent()->asResponse();

		this->m_ui->tableView->evalResponse(response);

		if ( response->getMethod() == _Heyu_METHOD_eval_cmd ) {
			if ( WFAILED(response->getReturnValue()) )
				QMessageBox::warning(this, this->windowTitle(), tr("FAILED#%1 executing Command").arg(response->getReturnValue()) );
			else
				QMessageBox::information(this, this->windowTitle(), tr("Command Executed") );
		 }
		else
			promptMethodResponse(message);
	 }
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
