/** @file    FormTaskManager.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormTaskManager.cpp 2808 2010-07-28 20:44:00Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormTaskManager class.
 * The header for this class can be found in FormTaskManager.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormTaskManager.h"
 #include "ui_FormTaskManager.h"

 #include <core/UserManager.h>
 #include <core/WoshKernel.h>
 #include <core/MethodsCommon.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/List.h>

 #include <interfaces/services/TaskManager.h>

 #include <QtGui>
 #include <framework/gui/GuiUtilities.h>
 #include <framework/gui/components/WidgetUpdateBar.h>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormTaskManager, "wosh::gui::IWidgetGeneric", 1.0, _static_FormTaskManager )
 WOSH_GUI_MAP(wosh::gui::FormTaskManager, "wosh::interfaces::services::TaskManager", 0.0, _static_Map_FormTaskManager )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormTaskManager::FormTaskManager(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormTaskManager();
	this->m_ui->setupUi(this);

	this->setWindowTitle("Automation Manager");
	this->getWidgetURI().setName( Utilities::format("FormTaskManager#%10lu", Utilities::randomLong()) );
	this->getTargetURI().clearPathName();
	this->getTargetURI().setName("Automations");

	this->m_ui->tableViewAuto->getTableView()->setContextMenuPolicy(Qt::CustomContextMenu);
	connect(this->m_ui->tableViewAuto, SIGNAL(postMessage(Message*)),this, SLOT(postMessage(Message*)) );
	connect(this->m_ui->tableViewAuto->getTableModel(), SIGNAL(tableCell_changed(unsigned int,unsigned int, ISerializable*)), this, SLOT(cell_valueChanged(unsigned int,unsigned int, ISerializable*)));


//	mapMethodButton(_TaskManager_METHOD_save, this->m_ui->btnDumpAuto);

 }

FormTaskManager::~FormTaskManager() {
	delete this->m_ui;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormTaskManager::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormTaskManager::refreshAll() {
	this->setWindowTitle( QString("Automation Manager [%1]").arg(this->getTargetURI().getKernelName().c_str()) );
	this->m_ui->tableViewAuto->refreshData();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormTaskManager::busMessage( const Message& message, const Bus* )
 {
	if ( !isMessageForMeOrFromTarget(message) )
		return;

	if ( message.getContent()->isResponse() ) {
		const wosh::Response* response = message.getContent()->asResponse();

		this->m_ui->tableViewAuto->evalResponse(response);

		promptMethodResponse(message);
	 }

 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormTaskManager::cell_valueChanged( unsigned int row, unsigned int col, ISerializable* newValue )
 {
	if ( newValue == NULL )
		return;
	if ( col != 2 )
		return;
	ISerializable* objName = this->wTableModelAuto->getDataTable()->get(row, 1 );
	if ( objName == NULL || !objName->isKindOf<Variant>() ) {
		return;
	 }
	const wosh::Variant* nameData = dynamic_cast<const wosh::Variant*>(objName);

	if ( !newValue->isKindOf<Variant>() )
		return;
	const wosh::Variant* valueData = dynamic_cast<const wosh::Variant*>(newValue);
	
	bool enabled = valueData->toBoolean(true);
	List* dL = new List();
	dL->push_back( valueData->clone() );
	dL->push_back<Variant>( enabled );

	Request* request_properties = new Request( _METHOD_SetProperty, dL );
	Message* message = new Message(request_properties);
	message->setDestinationBus( _Bus_Core );
	message->getDestination().setName(nameData->toString());
	postMessageForTarget(message);
	this->m_ui->tableViewAuto->getUpdateBar()->showMessage( QString(tr("Applying changes of '%1'..")).arg(nameData->toString().c_str()) );
 }


//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
