/** @file    FormWeather.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormWeather.cpp 2748 2010-07-11 22:07:06Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormWeather class.
 * The header for this class can be found in FormWeather.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormWeather.h"
 #include "ui_FormWeather.h"

 #include <core/UserManager.h>
 #include <core/WoshKernel.h>
 #include <core/MethodsCommon.h>
 #include <core/Request.h>
 #include <core/Response.h>
 #include <core/List.h>

 #include <interfaces/services/Weather.h>

 #include <QtGui>
 #include <framework/gui/GuiUtilities.h>
 #include <framework/gui/components/WidgetUpdateBar.h>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormWeather, "wosh::gui::IWidgetGeneric", 1.0, _static_FormWeather )
 WOSH_GUI_MAP(wosh::gui::FormWeather, "wosh::interfaces::services::Weather", 0.0, _static_Map_FormWeather )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormWeather::FormWeather(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormWeather();
	this->m_ui->setupUi(this);

	this->setWindowTitle("Weather Service");

	this->m_ui->tableViewForecast->setMethodList(_Weather_METHOD_get_forecast_tbl);
	this->m_ui->tableViewForecast->setMethodRetrieve(_METHOD_Retrieve);
	connect(this->m_ui->tableViewForecast, SIGNAL(postMessage(Message*)),this, SLOT(postMessage(Message*)) );

	connect(this->m_ui->tableViewSources, SIGNAL(postMessage(Message*)),this, SLOT(postMessage(Message*)) );

	connect(this->m_ui->wUpdateBarCondition, SIGNAL(updateClicked()), this, SLOT(refreshCondition()));

	mapMethodButton("asd", this->m_ui->btnReloadCondition);
	mapMethodButton("qwe", this->m_ui->btnReloadForecast);

 }

FormWeather::~FormWeather() {
	delete this->m_ui;
 }

void FormWeather::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormWeather::refreshAll() {
	this->setWindowTitle( QString("Weather Service [%1]").arg(this->getTargetURI().getKernelName().c_str()) );
	refreshCondition();
	this->m_ui->tableViewForecast->refreshData();
	this->m_ui->tableViewSources->refreshData();
 }

void FormWeather::refreshCondition() {
	executeRemoteMethod(_Weather_METHOD_get_weather);
	this->m_ui->wUpdateBarCondition->sayRetrieving();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormWeather::busMessage( const Message& message, const Bus* ) {
	if ( message.isEmpty() || !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) ) return;
	const wosh::Response* response = message.getContent()->asResponse();

	if ( response->getMethod() == _Weather_METHOD_get_weather ) {
		if ( response->hasData() && response->getData()->isKindOf<wosh::xtended::WeatherInfo>() ) {
			this->m_ui->wConditionCurrent->loadObject(response->getData()->as<wosh::xtended::WeatherInfo>());
			this->m_ui->wUpdateBarCondition->sayReady();
		 }
		else {
			this->m_ui->wConditionCurrent->loadObject(NULL);
			this->m_ui->wUpdateBarCondition->showError("Invalid DataType");
		 }
		return;
	 }
	this->m_ui->tableViewForecast->evalResponse(response);
	this->m_ui->tableViewSources->evalResponse(response);


 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
