/** @file    BusConnectorQt.cpp
 *  @author  Alessandro Polo
 *  @version $Id: BusConnectorQt.cpp 2437 2010-05-08 15:56:40Z alex $
 *  @brief
 * File containing methods for the wosh::gui::BusConnectorQt class.
 * The header for this class can be found in BusConnectorQt.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/gui/components/BusConnectorQt.h>

 #include <core/ObjectAllocator.h>
 #include <core/BusManager.h>

#ifdef _ENABLE_PROFILING
 #include <core/SystemProfiler.h>
#endif


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::BusConnectorQt, "wosh::BusConnector", 1.00, _static_BusConnectorQt )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

BusConnectorQt::BusConnectorQt() : QObject(), BusConnector()
 {
	QObject::connect(this, SIGNAL(signal_new_message(Message*)), this, SLOT(slot_new_message(Message*)), Qt::QueuedConnection );
	QObject::connect(this, SIGNAL(signal_event(Bus::BUS_STATE)), this, SLOT(slot_event(Bus::BUS_STATE)), Qt::QueuedConnection );

 }
 
BusConnectorQt::~BusConnectorQt()
 {
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS

void BusConnectorQt::busEvent( Bus::BUS_STATE state, const Bus* )
 {
	 if ( state == Bus::STATE_DESTROYED )
		this->bus = NULL;
	if ( this->handler == NULL )
		return;
	emit signal_event(state);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void BusConnectorQt::busMessage( const Message& message, const Bus* )
 {
	if ( this->handler == NULL )
		return;
	emit signal_new_message(message.clone());
 }

////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void BusConnectorQt::slot_new_message( Message* message )
 {
	if ( message == NULL ) return;
	if ( this->handler == NULL ) {
		delete message;
		return;
	 }
	PROFILING_ENTER(this, 0)
	this->handler->busMessage( *message, this->bus );
	delete message;
	PROFILING_EXIT(this, 0)
 }

void BusConnectorQt::slot_event( Bus::BUS_STATE state )
 {
	if ( this->handler == NULL )
		return;
	this->handler->busEvent( state, this->bus );
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// POSTING
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
