/** @file    SessionQt.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SessionQt.cpp 2804 2010-07-27 04:39:07Z alex $
 *  @brief
 * File containing methods for the wosh::gui::SessionQt class.
 * The header for this class can be found in SessionQt.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/gui/components/SessionQt.h>

 #include <framework/gui/GuiUtilities.h>
 #include <core/ObjectAllocator.h>
 #include <core/Utilities.h>
 #include <core/WoshKernel.h>
 //#include <termios.h>

 
using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::SessionQt, "wosh::Session", 1.0, _static_SessionQt )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SessionQt::SessionQt() : QObject(), Session() {
	Object::setName("SessionQt", getID());

	Log.setContext( "SessionQt", true );

 }

SessionQt::~SessionQt() {
	Log(LOG_VERBOSE, ":~SessionQt() Destroying.." );
	close();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionQt::open( unsigned long ) {
	return login( &GuiUtilities::getSessionInfo() );
 }

WRESULT SessionQt::close( unsigned long ) {
	emit onClosed();
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionQt::setCurrentLocation( const URI& uri ) {
	Session::setCurrentLocation(uri);
	emit onLocationChanged();
 }
void SessionQt::setCurrentBus( const std::string& value ) {
	Session::setCurrentBus(value);
	emit onBusChanged( QString(value.c_str()) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
