/**
 * @class   wosh::gui::SessionQt
 * @brief   The class represent and manage life of an local shell session.
 *          
 *
 * @see     woshsrv
 * @see     wosh::Session
 * @see     wosh::SessionManager
 ****************************************************************************
 * @version $Id: SessionQt.h 2437 2010-05-08 15:56:40Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_SessionQt_H__
 #define __WOSH_Communication_SessionQt_H__

 #include <core/Session.h>
 #include <framework/communication/ShellCommand.h>
 
 #include <QObject>


namespace wosh {
 namespace gui {

class SessionQt :	public QObject,
					public wosh::Session
 {
 	Q_OBJECT

	WOSH_SESSION(wosh::gui::SessionQt)


	public:
		SessionQt();
		virtual ~SessionQt();

	public:
		WRESULT open( unsigned long timeout = LIMIT_ULONG_MAX );
		WRESULT close( unsigned long timeout = LIMIT_ULONG_MAX );

		void setCurrentLocation( const URI& uri );
		void setCurrentBus( const std::string& value );

	protected:
		inline void printMessage( const std::string& output_text ) {
			emit printMessage( QString::fromStdString(output_text) );
		 }

	public slots:
		void parseInput( const QString& message ) {
			Session::parseInput( message.toStdString() );
		 }

	signals:
		void printMessage( const QString& message );
		void onClosed();
		void onLocationChanged();
		void onBusChanged( const QString& busname );

}; // class def

 }; // namespace gui
}; // namespace wosh

#endif //__WOSH_Communication_SessionQt_H__
