/** @file    DialogLogin.cpp
 *  @author  Alessandro Polo
 *  @version $Id: DialogLogin.cpp 2618 2010-06-04 09:48:15Z alex $
 *  @brief
 * File containing methods for the wosh::gui::DialogLogin class.
 * The header for this class can be found in DialogLogin.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "DialogLogin.h"
 #include "ui_DialogLogin.h"

 #include <QtGui/QMessageBox>
 #include <core/UserManager.h>
 #include <framework/gui/GuiUtilities.h>
 #include <framework/security/SecurityAuthenticationShell.h>
 #include <framework/security/SecurityAuthenticationOS.h>


using namespace std;
using namespace wosh::security;
namespace wosh {
 namespace gui {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

DialogLogin::DialogLogin(QWidget *parent) : QDialog(parent)
 {
	this->m_ui = new Ui::DialogLogin();
	this->m_ui->setupUi(this);

	this->security = NULL;

	QObject::connect(this->m_ui->buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
	QObject::connect(this->m_ui->buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

	setUserName( GuiUtilities::getSessionInfo().getUserName() );

	if ( !this->m_ui->txtOS_WoshUserName->text().isEmpty() )
		this->m_ui->tabWidget->setCurrentIndex(1);
 }

DialogLogin::~DialogLogin() {
	delete this->m_ui;
	if ( this->security != NULL )
		delete this->security;
 }

void DialogLogin::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void DialogLogin::setUserName( const std::string& value ) {
	this->m_ui->txtPWD_UserName->setText( QString(value.c_str()) );
	string os_user, wosh_user;
	if ( WSUCCEEDED(SecurityTokenProviderOS::findLoggedUser(wosh_user, &os_user)) ) {
		this->m_ui->txtOS_OsUserName->setText( QString(os_user.c_str()) );
		this->m_ui->txtOS_WoshUserName->setText( QString(wosh_user.c_str()) );
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void DialogLogin::accept() {
	switch( this->m_ui->tabWidget->currentIndex() ) {
		case 0: {
				this->username = this->m_ui->txtPWD_UserName->text().toStdString();
				this->security = new wosh::security::SecurityTokenProviderShell();// this->m_ui->txtPWD_Password->text().toStdString() );
				break;
			 }
		case 1: {
				this->username = this->m_ui->txtOS_OsUserName->text().toStdString();
				this->security = new wosh::security::SecurityTokenProviderOS();
				break;
			 }
		default: {
				this->username = "";
				this->security = NULL;
				break;
			 }
	 }
	QDialog::accept();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT DialogLogin::waitValidUser( std::string& username, const wosh::SecurityTokenProvider* default_securityAuth, wosh::SecurityTokenProvider** securityAuth ) {
	WRESULT auth = UserManager::isAuthorized( username, default_securityAuth );
	if ( WSUCCEEDED(auth) ) {
		*securityAuth = default_securityAuth->clone();
		return WRET_OK;
	 }

	DialogLogin* dSuser = new DialogLogin();
	dSuser->setUserName(username);
	///@todo setup default
	int ret = QDialog::Accepted;
	while ( ( ret = dSuser->exec() ) == QDialog::Accepted ) {
		WRESULT auth = UserManager::isAuthorized( dSuser->getUserName(), dSuser->getSecurityTokenProvider() );
		if ( WSUCCEEDED(auth) )
			break;
	 }
	if ( ret == QDialog::Accepted ) {
		username = dSuser->getUserName();
		*securityAuth = dSuser->security;
		dSuser->security = NULL;
		return WRET_OK;
	 }
	*securityAuth = NULL;
	delete dSuser;
	return WRET_ERR_SECURITY;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
