/**
 * @class   wosh::gui::KioskDialogLogin
 * @brief   
 *          
 *          
 *
 ****************************************************************************
 * @version $Id: KioskDialogLogin.h 2282 2010-04-07 06:20:24Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
#ifndef __WOSH_Gui_KioskDialogLogin_H__
 #define __WOSH_Gui_KioskDialogLogin_H__

 #include <QtGui/QGroupBox>
 #include "ui_KioskDialogLogin.h"

#if QT_VERSION >= 0x040600
# include <QPropertyAnimation>
#endif


using namespace std;
namespace wosh {
 namespace gui {


class KioskDialogLogin : public QGroupBox {

	Q_OBJECT
	Q_PROPERTY(QRect geometry READ geometry WRITE setGeometry)

	public:
		KioskDialogLogin(QWidget *parent = 0) : QGroupBox(parent) {
			this->m_ui = new Ui::KioskDialogLogin();
			this->m_ui->setupUi(this);
i = 0;
			connect(this->m_ui->btnLogin, SIGNAL(clicked()), this, SLOT(btnLogin_clicked()));
			connect(this->m_ui->btnSystem, SIGNAL(clicked()), this, SIGNAL(system_dialog_request()));
		 }

		virtual ~KioskDialogLogin() {
			delete this->m_ui;
		 }

	public slots:
		void logged() {
		 }
		void denied( const QString& optional_message ) {
			(void)optional_message;

#if QT_VERSION >= 0x040600
			QPropertyAnimation* animation = new QPropertyAnimation(this, "geometry");
			animation->setDuration(350);
			animation->setLoopCount(2);
			animation->setEasingCurve(QEasingCurve::OutBounce);
			animation->setStartValue( this->geometry() );
			animation->setKeyValueAt( 0.4, this->geometry().adjusted(-10,0,-10,0) );
			animation->setKeyValueAt( 0.6, this->geometry().adjusted(+10,0,+10,0) );
			animation->setEndValue( this->geometry() );
			animation->start();
#endif
		 }

	protected slots:
		void btnLogin_clicked() {
			this->m_ui->btnLogin->setText( QString("clicked %1").arg(++i) );
			emit login_request( this->m_ui->txtUsername->text() , this->m_ui->txtPassword->text() );
			this->m_ui->txtPassword->setText("");
		 }

	signals:
		void login_request( const QString& username, const QString& password );
		void system_dialog_request();

	private:
		Ui::KioskDialogLogin *m_ui;
int i;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_KioskDialogLogin_H__
