/**
 * @class   wosh::gui::FormPersistenceManager
 * @brief   Graphic interface of wosh::PersistenceManager core-module.
 *
 * \image html extras/screenshots/WOSH.GUI.FormPersistenceManager.1.png "FormPersistenceManager::Tab 1"
 * \image html extras/screenshots/WOSH.GUI.FormPersistenceManager.2.png "FormPersistenceManager::Tab 2"
 *
 * @ingroup Gui
 ****************************************************************************
 * @version 0.8.499 $Id: FormPersistenceManager.h 2648 2010-06-11 08:32:52Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormPersistenceManager_H__
 #define __WOSH_Gui_FormPersistenceManager_H__

 #include <framework/gui/IWidget.h>
 #include <framework/gui/IWidgetSerializable.h>
 #include <framework/gui/widgets/wCore_Object.h>

 #include <QtGui/QMenu>


namespace Ui {
 class FormPersistenceManager;
};
using namespace std;
namespace wosh {
 namespace gui {


class FormPersistenceManager :  public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormPersistenceManager)
	WOSH_CLASS_ABSTRACT(false)


	public:
		FormPersistenceManager(QWidget *parent = 0);
		virtual ~FormPersistenceManager();

	public:
		void busMessage( const Message& message, const Bus* source );

	public slots:
		void refreshAll();

	protected slots:
		void refreshEncoders();
		void refreshSandObjectTypeList();

		void btnDbForceSave_clicked();
		void tableViewDataBases_ContextMenu( const QPoint& pos, const QModelIndex& index );


		void btnProtocolUninstall_clicked();
		void btnProtocolInstall_clicked();
		void tableViewProtocols_ContextMenu( const QPoint& pos, const QModelIndex& index );
		//void tableViewSupportedProtocols_ContextMenu( const QPoint& pos, const QModelIndex& index );

		void checkAllocateSandBuffer();
		void btnSandObjectAllocate_clicked();
		void btnSandObjectDestroy_clicked();
		void btnSandObjectGui_clicked();
		void btnSandToString_clicked();
		void btnSandEncode_clicked();
		void btnSandDecode_clicked();

	protected:
		void changeEvent(QEvent *e);

	protected:
		void buildMenus();
		QMenu* menuDataBases;
		QAction* menuDataBasesTitle;

		QMenu* menuProtocol;
		QAction* menuProtocolTitle;
		QAction* menuProtocolCreateGui;
		QAction* menuProtocolStart;
		QAction* menuProtocolStop;

		char* sand_buffer;
		int64 sand_buffer_size;
		ObjectTypeInfoListModel* sand_obj_types_model;
		ISerializable* sand_obj;
		IWidgetSerializable* sand_obj_gui;

	private:
		Ui::FormPersistenceManager *m_ui;

}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormPersistenceManager_H__
