/**
 * @class   wosh::gui::FormThreadManager
 * @brief   Graphic interface of wosh::ThreadManager core-module.
 *
 * \image html extras/screenshots/WOSH.GUI.FormThreadManager.1.png "FormThreadManager::Tab 1"
 * \image html extras/screenshots/WOSH.GUI.FormThreadManager.2.png "FormThreadManager::Tab 2"
 *
 * @ingroup Gui
 ****************************************************************************
 * @version 0.8.499 $Id: FormThreadManager.h 2457 2010-05-12 03:15:17Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_FormThreadManager_H__
 #define __WOSH_Gui_FormThreadManager_H__

 #include <framework/gui/IWidget.h>
 #include <framework/gui/widgets/wCore_PropertiesProvider.h>


namespace Ui {
 class FormThreadManager;
};
using namespace std;
namespace wosh {
 namespace gui {


class FormThreadManager :  public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::FormThreadManager)
	WOSH_CLASS_ABSTRACT(false)


	public:
		FormThreadManager(QWidget *parent = 0);
		virtual ~FormThreadManager();

	public:
		void busMessage( const Message& message, const Bus* source );

	public slots:
		void refreshAll();

	protected slots:
		void refreshConfiguration();

		void btnThreadStart_clicked();
		void btnThreadStop_clicked();
		void btnThreadTerminate_clicked();

	protected:
		void changeEvent(QEvent *e);

	protected:
		PropertiesMapper* propMapperCfg;

	private:
		Ui::FormThreadManager *m_ui;


}; // class def

 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_FormThreadManager_H__
