/** @file    wBuilding_BuildingObject.cpp
 *  @author  Alessandro Polo
 *  @version $Id: wBuilding_BuildingObject.cpp 2618 2010-06-04 09:48:15Z alex $
 *  @brief
 * File containing methods for the wosh::gui::wBuilding_BuildingObject class.
 * The header for this class can be found in wBuilding_BuildingObject.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "wBuilding_BuildingObject.h"

 #include <core/ObjectAllocator.h>
 #include <framework/gui/GuiUtilities.h>

 #include "ui_wBuilding_BuildingObject.h"


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::wBuilding_BuildingObject, "wosh::gui::IWidgetSerializable", 1.0, _static_wBuilding_BuildingObject )
 WOSH_GUI_MAP(wosh::gui::wBuilding_BuildingObject, "wosh::building::BuildingObject", 0.0, _static_Map_wBuilding_BuildingObject )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

wBuilding_BuildingObject::wBuilding_BuildingObject( QWidget* parent )
	: QWidget(parent) {

	this->m_ui = new Ui::wBuilding_BuildingObject();
	this->m_ui->setupUi(this);

	setWindowIcon( GuiUtilities::createIconFor( getSerializableType() ) );
	setWindowTitle( getSerializableType() );

	connect(this->m_ui->cmbDebugLevel, SIGNAL(activated(const QString &)), this, SLOT(cmbDebugLevel_activated(const QString &)));

	this->m_ui->widgetProperties->init();
	loadObject(NULL);
 }

wBuilding_BuildingObject::~wBuilding_BuildingObject() {
	delete this->m_ui;
 }

void wBuilding_BuildingObject::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT wBuilding_BuildingObject::loadObject( const wosh::building::BuildingObject* bobj ) {
	if ( bobj == NULL ) {
		setWindowIcon( GuiUtilities::createIconFor( getSerializableType() ) );
		setWindowTitle( getSerializableType() );

		this->m_ui->lblName->setText("");
		this->m_ui->lblType->setText("");
		this->m_ui->lblURI->setText("");

		this->m_ui->widgetProperties->setTargetURI( URI() );
		this->m_ui->widgetProperties->loadObject(NULL);
		this->m_ui->widgetGeometry->loadObject(NULL);

	 }
	else {
		setWindowIcon( GuiUtilities::createIconFor( bobj->getInterfaces().getImplementedClass().getClassType().c_str() ) );
		setWindowTitle( QString("%1 [%2]").arg(bobj->getName().c_str()).arg(bobj->getInterfaces().getImplementedClass().getClassType().c_str()) );

		this->m_ui->lblName->setText( bobj->getName().c_str() );
		this->m_ui->lblType->setText( bobj->getInterfaces().getImplementedClass().getClassType().c_str() );
		this->m_ui->lblURI->setText( bobj->getURI().toString().c_str() );

		this->m_ui->widgetProperties->setTargetURI( bobj->getURI() );
		this->m_ui->widgetProperties->loadObject(&bobj->getProperties());
		this->m_ui->widgetGeometry->loadObject(&bobj->getGeometry());

	 }
	return WRET_OK;
 }

WRESULT wBuilding_BuildingObject::saveObject( wosh::building::BuildingObject* bobj ) const {
	if ( bobj == NULL ) return WRET_ERR_PARAM;
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void wBuilding_BuildingObject::cmbDebugLevel_activated(const QString& text) {
	List* dlProp = new List(2);
	dlProp->push_back<Variant>( _KEY_DebugLevel );
	dlProp->push_back<Variant>( string(text.toLatin1().data()) );
	Request* request_info = new Request( _METHOD_SetProperty, dlProp );
//	postMessageForTarget( new Message(request_info) );
//	this->m_ui->wUpBarInformation->showMessage("Changing DebugLevel..");
 }

 }; // namespace gui
}; // namespace wosh
