/**
 * @file    wCore_List.h
 * @brief   
 *          
 *          
 *
 ****************************************************************************
 * @version $Id: wCore_List.h 2505 2010-05-22 02:03:13Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_Widgets_Core_List_H__
 #define __WOSH_Gui_Widgets_Core_List_H__

 #include <framework/gui/IWidgetSerializable.h>
 #include <framework/gui/GuiUtilities.h>
 #include <QtGui/QListView>
 #include <QAbstractListModel>

 #include <core/List.h>



namespace wosh {
 namespace gui {


// ----------------------------------------------------------------------------------------

class ListModel_Core_List :  public QAbstractListModel
 {
	Q_OBJECT

	public:
		ListModel_Core_List(QObject *parent = 0);
		virtual ~ListModel_Core_List();

	public:
		WRESULT loadObject( const wosh::List* list );
		WRESULT acquireObject( wosh::List* list );

	public:
		int rowCount(const QModelIndex &parent = QModelIndex()) const;
		QVariant data(const QModelIndex &index, int role) const;

	public: // editable 
		Qt::ItemFlags flags(const QModelIndex &index) const {
			 if (!index.isValid())
				 return Qt::ItemIsEnabled;
			 return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
		 }

		bool setData(const QModelIndex &index, const QVariant &value, int role);

	protected:
		wosh::List* dataList;

}; // class def

// ----------------------------------------------------------------------------------------


class wCore_List :	public QListView,
						public virtual IWidgetSerializableT<wosh::List>
 {
	Q_OBJECT

	WOSH_WIDGET_SERIALIZABLE(wosh::gui::wCore_List)

	public:
		wCore_List(QWidget* parent = NULL);
		virtual ~wCore_List();

	public:
		virtual WRESULT loadObject( const wosh::List* object );
		virtual WRESULT saveObject( wosh::List* object ) const;

	public:
		inline const ListModel_Core_List* getListModel() const	{ return this->listModel; }
		inline ListModel_Core_List* getListModel()				{ return this->listModel; }


	signals:
		void updatedObject();

	protected:
		ListModel_Core_List* listModel;


}; // class def



// ----------------------------------------------------------------------------------------




 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_Widgets_Core_List_H__
