/**
 * @class   wosh::gui::wCore_MethodsProvider
 * @brief   Shows Properties (MethodsProvider) and allow user
 *          to edit then (asynch, remote through bus messages).
 *
 ****************************************************************************
 * @version $Id: wCore_MethodsProvider.h 2488 2010-05-21 01:48:27Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_wCore_MethodsProvider_H__
 #define __WOSH_Gui_wCore_MethodsProvider_H__

#include <core/MethodsProvider.h>
#include <core/List.h>
#include <core/Message.h>

#include <framework/gui/IWidget.h>
#include <framework/gui/WoshTableModel.h>

 #include <core/Property.h>
 #include <core/Table.h>
 #include <core/MethodsCommon.h>
 #include <QMap>
 #include <QSignalMapper>


using namespace wosh;

namespace Ui {
    class wCore_MethodsProvider;
}
namespace wosh {
 namespace gui {



class wCore_MethodsProvider :  public IWidgetGeneric
 {
	Q_OBJECT

	WOSH_CLASSNAME(wosh::gui::wCore_MethodsProvider)
	WOSH_CLASS_ABSTRACT(false)


	public:
		wCore_MethodsProvider(QWidget *parent = 0);
		virtual ~wCore_MethodsProvider();

	public slots:
		WRESULT loadObject( const wosh::Table* table );
		WRESULT saveObject( wosh::Table* table ) const;

	public:
		virtual void busMessage( const Message& message, const Bus* source );

		virtual void setTargetURI( const URI& uri );
 
	protected:
		void changeEvent(QEvent *e);

	protected:
		virtual void contextMenuEvent( QContextMenuEvent* menu_event );

		void refreshData( bool force_all );

	public slots:

		virtual void refreshAll()			{ refreshData(true); }
		virtual WRESULT refreshSmart()		{ refreshData(false); return WRET_OK; }

		virtual WRESULT reset();

	protected slots:
		void cell_valueChanged( unsigned int row, unsigned int col, ISerializable* newValue );

	signals:
		void updateClicked();


	protected:
		void buildMenus();

		QAction* menuTitle;

		QMenu* menuKey;
		QAction* actionKeyRefresh;
		QAction* actionKeyCopyTo;
		QAction* actionKeyNew;
		QAction* actionKeyMan;
		QAction* actionKeyDelete;

		QMenu* menuValue;
		QAction* actionValueRefresh;
		QAction* actionValueEdit;

		QMenu* menuPerm;
		QAction* actionPermRefresh;
		QAction* actionPermEdit;

	private:
		Ui::wCore_MethodsProvider *m_ui;

}; // class def



 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_wCore_MethodsProvider_H__
