/**
 * @file    wCore_Object.h
 * @brief   
 *          
 *          
 *
 ****************************************************************************
 * @version $Id: wCore_Object.h 2505 2010-05-22 02:03:13Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_Widgets_Core_Object_H__
 #define __WOSH_Gui_Widgets_Core_Object_H__

 #include <framework/gui/IWidgetSerializable.h>
 #include <framework/gui/GuiUtilities.h>
 #include <QAbstractListModel>
 #include <QAbstractTableModel>

 #include <core/ObjectAllocator.h>

namespace wosh {
 namespace gui {


// ----------------------------------------------------------------------------------------

class ObjectTypeInfoListModel :  public QAbstractListModel
 {
	Q_OBJECT

	public:
		ObjectTypeInfoListModel(QObject *parent = 0)
			: QAbstractListModel(parent) { }
		virtual ~ObjectTypeInfoListModel() { }

		std::vector<ObjectTypeInfo>& getData() { return this->class_list; }

		using QAbstractListModel::reset;

	public:
		int rowCount( const QModelIndex &parent = QModelIndex() ) const {
			(void)parent; return this->class_list.size();
		 }

		QVariant data( const QModelIndex &index, int role ) const;

	protected:
		std::vector<ObjectTypeInfo> class_list;

}; // class def
// ----------------------------------------------------------------------------------------

class ObjectTypeInfoTableModel :  public QAbstractTableModel
 {
	Q_OBJECT

	public:
		ObjectTypeInfoTableModel(QObject *parent = 0)
			: QAbstractTableModel(parent) { }
		virtual ~ObjectTypeInfoTableModel() { }

		std::vector<ObjectTypeInfo>& getData() { return this->class_list; }

		using QAbstractTableModel::reset;

		int rowCount( const QModelIndex &parent = QModelIndex() ) const {
			(void)parent; return this->class_list.size();
		 }

		int columnCount(const QModelIndex &parent = QModelIndex()) const { (void)parent; return 3; }

		QVariant data(const QModelIndex &index, int role) const;

		QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;

	public: // editable
		 Qt::ItemFlags flags(const QModelIndex &index) const {
			return QAbstractItemModel::flags(index) | Qt::ItemIsEnabled;// | Qt::ItemIsEditable;
		  }

	protected:
		std::vector<ObjectTypeInfo> class_list;

}; // class def

// ----------------------------------------------------------------------------------------


 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_Widgets_Core_Object_H__
