/**
 * @file    wCore_Variant.h
 * @brief   
 *          
 *          
 *
 ****************************************************************************
 * @version $Id: wCore_Variant.h 2455 2010-05-12 02:46:32Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Gui_Widgets_Core_Variant_H__
 #define __WOSH_Gui_Widgets_Core_Variant_H__

 #include <framework/gui/IWidgetSerializable.h>
 #include <framework/gui/GuiUtilities.h>
 #include <QAbstractListModel>

 #include <core/Variant.h>

 #include <QtGui/QVBoxLayout>
 #include <QDateTime>
 #include <QtGui/QMenu>
 #include <QtGui/QContextMenuEvent>
 #include <QtGui/QLineEdit>
 #include <QtGui/QComboBox>
 #include <QtGui/QLabel>
 #include <QtGui/QSlider>
 #include <QtGui/QCheckBox>
 #include <QtGui/QSpinBox>


namespace wosh {
 namespace gui {


// ----------------------------------------------------------------------------------------

class IwCore_Variant :  public virtual IWidgetSerializableT<wosh::Variant>
 {
	public:
		IwCore_Variant() { }
		virtual ~IwCore_Variant() { }

	public:
		virtual WRESULT setData( const Variant& new_data ) {
			this->myData = new_data;
			this->timestamp = QDateTime::currentDateTime();
			applyData(new_data);
			return WRET_OK;
		 }

		virtual void applyData( const Variant& new_data ) = 0;

		virtual const Variant& getData() const				{ return this->myData; }
		virtual Variant& getData()							{ return this->myData; }

	public:
		virtual WRESULT loadObject( const wosh::Variant* object ) {
			this->myData.clear();
			if ( object == NULL ) return WRET_OK;
			return setData(*object);
		 }

		virtual WRESULT saveObject( wosh::Variant* object ) const {
			if ( object == NULL ) return WRET_ERR_PARAM;
			*object = getData();
			return WRET_OK;
		 }

		inline QString getDataType() const				{ return QString(this->myData.getTypeAsString()); }

	protected:
		wosh::Variant myData;
		QDateTime timestamp;

}; // class def

// ----------------------------------------------------------------------------------------

class wCore_Variant :	public QWidget,
						public virtual IWidgetSerializableT<wosh::Variant>
 {
	Q_OBJECT

	WOSH_WIDGET_SERIALIZABLE(wosh::gui::wCore_Variant)

	public:
		wCore_Variant(QWidget* parent = NULL);
		virtual ~wCore_Variant() { }

	public:
		virtual WRESULT setData( const Variant& new_data );

		inline const Variant& getData() const {
			if ( this->wEditor == NULL ) return Variant::EmptyVariant;
			return this->wEditor->getData();
		 }

		inline QString getDataType() {
			if ( this->wEditor == NULL ) return QString("Variant::EMPTY");
			return this->wEditor->getDataType();
		 }
		
	public:
		virtual WRESULT loadObject( const wosh::Variant* object );

		virtual WRESULT saveObject( wosh::Variant* object ) const;

	public slots:
		void changedType();

	protected:
		IwCore_Variant* wEditor;

		QVBoxLayout* vLayout;
		QComboBox* cmbType;

	signals:
		void updatedObject();

}; // class def


// ----------------------------------------------------------------------------------------

class VariantTypeModel :  public QAbstractListModel
 {
	Q_OBJECT

	public:
		VariantTypeModel(QObject *parent = 0)
			: QAbstractListModel(parent) { }

	public:
		int rowCount( const QModelIndex &parent = QModelIndex() ) const {
			(void)parent; return 12;
		 }

		QVariant data( const QModelIndex &index, int role ) const;

}; // class def

// ----------------------------------------------------------------------------------------


class wCore_VariantLabel :	public QLabel,
							public IwCore_Variant
 {
	Q_OBJECT

	WOSH_WIDGET_SERIALIZABLE(wosh::gui::wCore_VariantLabel)

	public:
		wCore_VariantLabel(QWidget* parent = NULL)
			: QLabel(parent), IwCore_Variant() {
		 }

	public:
		void applyData( const Variant& new_data ) {
			QVariant qvar = GuiUtilities::convertVariant2QVariant( &new_data, Variant::STRING );
			this->setText( qvar.toString() );
			this->setToolTip(tr("%1 [updated at %2]").arg(getDataType()).arg(this->timestamp.toString()));
		 }

}; // class def

// ----------------------------------------------------------------------------------------


class wCore_VariantEditor :		public QLineEdit,
								public IwCore_Variant
 {
	Q_OBJECT

	WOSH_WIDGET_SERIALIZABLE(wosh::gui::wCore_VariantEditor)

	public:
		wCore_VariantEditor(QWidget* parent = NULL)
			: QLineEdit(parent), IwCore_Variant() {
			connect(this, SIGNAL(editingFinished()), this, SLOT(check_update()));
		 }

	public:
		void applyData( const Variant& new_data ) {
			QVariant qvar = GuiUtilities::convertVariant2QVariant( &new_data, Variant::STRING );
			this->setText( qvar.toString() );
			this->setToolTip(tr("%1 [updated at %2]").arg(getDataType()).arg(this->timestamp.toString()));
		 }

	protected:
		void contextMenuEvent( QContextMenuEvent* menu_event ) {
			QMenu* menu = createStandardContextMenu();
			QMenu* menuType = new QMenu(this);
			menuType->setTitle("Type");
			menuType->addAction("String (STR)");
			menuType->addAction("Integer (INT)");

			QAction* sep = menu->insertSeparator( menu->actions().first());
			menu->insertMenu(sep, menuType);

			menu->exec(menu_event->globalPos());
			delete menu;
		 }

	protected slots:
		void check_update() {
			if ( !myData.isString() || myData.asString() != text().toLatin1().data() ) {
				myData.set( text().toLatin1().data() );
				emit updatedObject();
			 }
		 }

	signals:
		void updatedObject();

}; // class def

// ----------------------------------------------------------------------------------------


class wCore_VariantComboBox :	public QComboBox,
								public IwCore_Variant
 {
	Q_OBJECT

	WOSH_WIDGET_SERIALIZABLE(wosh::gui::wCore_VariantComboBox)

	public:
		wCore_VariantComboBox(QWidget* parent = NULL)
			: QComboBox(parent), IwCore_Variant() {
			connect(this, SIGNAL(activated(int)), this, SLOT(check_update(int)));
		 }

	public:
		void applyData( const Variant& new_data ) {
			if ( new_data.isNumeric() )
				this->setCurrentIndex( new_data.toInteger(0) );
			else {
				for(int i=0;i<count(); i++) {
					if ( itemText(i) != QString(new_data.toString().c_str()) ) continue;
					this->setCurrentIndex( i );
					break;
				 }
			 }
		 }

	protected slots:
		void check_update( int i ) {
			if ( myData.toLong(0) != i ) {
				myData.set( (long)i );
				emit updatedObject();
			 }
		 }

	signals:
		void updatedObject();

}; // class def

// ----------------------------------------------------------------------------------------


class wCore_VariantSlider :		public QSlider,
								public IwCore_Variant
 {
	Q_OBJECT

	WOSH_WIDGET_SERIALIZABLE(wosh::gui::wCore_VariantSlider)

	public:
		wCore_VariantSlider(QWidget* parent = NULL)
			: QSlider(parent), IwCore_Variant() {
		 }

	public:
		void applyData( const Variant& new_data ) {
			if ( new_data.isInteger() )
				this->setValue( new_data.toInteger(0) );
			else if ( new_data.isLong() )
				this->setValue( new_data.toLong(0) );
			else if ( new_data.isInt64() )
				this->setValue( new_data.toInt64(0) );
		 }

	protected slots:
		void check_update() {
		 }

	signals:
		void updatedObject();

}; // class def

// ----------------------------------------------------------------------------------------


class wCore_VariantSpinBox :	public QSpinBox,
								public IwCore_Variant
 {
	Q_OBJECT

	WOSH_WIDGET_SERIALIZABLE(wosh::gui::wCore_VariantSpinBox)

	public:
		wCore_VariantSpinBox(QWidget* parent = NULL)
			: QSpinBox(parent), IwCore_Variant() {
			connect(this, SIGNAL(editingFinished()), this, SLOT(check_update()));
		 }

	public:
		void applyData( const Variant& new_data ) {
			if ( new_data.isInteger() )
				this->setValue( new_data.toInteger(0) );
			else if ( new_data.isLong() )
				this->setValue( new_data.toLong(0) );
			else if ( new_data.isInt64() )
				this->setValue( new_data.toInt64(0) );
		 }

	protected slots:
		void check_update() {
			if ( myData.toLong(0) != value() ) {
				myData.set( (long)value() );
				emit updatedObject();
			 }
		 }

	signals:
		void updatedObject();

}; // class def

// ----------------------------------------------------------------------------------------

class wCore_VariantCheckBox :	public QCheckBox,
								public IwCore_Variant
 {
	Q_OBJECT

	WOSH_WIDGET_SERIALIZABLE(wosh::gui::wCore_VariantCheckBox)

	public:
		wCore_VariantCheckBox(QWidget* parent = NULL)
			: QCheckBox(parent), IwCore_Variant() {
			connect(this, SIGNAL(clicked()), this, SLOT(check_update()));
		 }

	public:
		void applyData( const Variant& new_data ) {
			this->setChecked( new_data.toBoolean(false) );
		 }

	protected slots:
		void check_update() {
			if ( !this->myData.isBoolean() || this->myData.toBoolean(false) != isChecked() ) {
				this->myData.set( (bool)isChecked() );
				emit updatedObject();
			 }
		 }

	signals:
		void updatedObject();

}; // class def

// ----------------------------------------------------------------------------------------




 }; // namespace gui
}; // namespace wosh


#endif //__WOSH_Gui_Widgets_Core_Variant_H__
