/**
 * @class   wosh::network::NetworkChannelTcpServer
 * @brief   The class provides a TCP server for wosh::network::NetworkProtocolTcp
 *          and manages incoming connections.
 *
 * @see     wosh::NetworkManager
 ****************************************************************************
 * @version $Id: NetworkChannelTcpServer.h 2610 2010-05-29 14:39:50Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Network_NetworkChannelTcpServer_H__
 #define __WOSH_Network_NetworkChannelTcpServer_H__

 #include <core/Thread.h>
 #include <core/LogContext.h>

 #include <framework/network/ServerTcp.h>



namespace wosh {
 namespace network {


class NetworkChannelTcpServer;

class INetworkChannelTcpServerListener {
	public:
		virtual void channelTcpServer_incomingConnection( wosh::network::NetworkChannelTcpServer* source, int socket_id ) = 0;

	public:
		virtual ~INetworkChannelTcpServerListener () { }
};


class NetworkChannelTcpServer :	protected wosh::ThreadImpl,
								public virtual wosh::network::IServerTcpListener  {

	public:

		NetworkChannelTcpServer( const std::string& name = "" );

		virtual ~NetworkChannelTcpServer();


	public:

		WRESULT listen( const std::string& addressport, unsigned long timeout = LIMIT_ULONG_MAX );
		WRESULT stopListening( unsigned long timeout = LIMIT_ULONG_MAX );

		bool isListening() const;

		void setListener( wosh::network::INetworkChannelTcpServerListener* tcp_listener );

		inline wosh::network::INetworkChannelTcpServerListener* getListener() { return this->listener; }


/** @name IServerTcpListener
 * @{
 ******************************************************************************/
	public:
		void serverTcp_incomingConnection( int socket_id, wosh::network::ServerTcp* source );
//@}
	protected:
		bool initializingThread();
		void runThread();
		void exitingThread();

	protected:
		std::string bindAddress;

		wosh::network::ServerTcp* tcpServer;

		wosh::LogContext Log;

		wosh::network::INetworkChannelTcpServerListener* listener;


}; // class def

 }; // namespace network
}; // namespace wosh

#endif //__WOSH_Network_NetworkChannelTcpServer_H__
