/** @file    SocketTcp.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SocketTcp.cpp 2282 2010-04-07 06:20:24Z alex $
 *  @brief
 * File containing methods for the wosh::network::SocketTcp class.
 * The header for this class can be found in SocketTcp.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <framework/network/SocketTcp.h>

 #include <core/ObjectAllocator.h>


namespace wosh {
 namespace network {

  WOSH_REGISTER_INTERFACE(wosh::network::SocketTcp, "", 1.00, _static_SocketTcp )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

SocketTcp::SocketTcp() {
	this->listener = NULL;

	this->address = "";
	this->port = 0;
	this->address_port = "";

#ifdef _SocketTcp_DEBUG_ENABLED
	Log.setContext( "SocketTcp" );
	Log.setLevel( LOG_INFO );
	Log.setIndent( 5 );
#endif
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SocketTcp::connectTo( const std::string& address_port, unsigned long timeout  ) {
	if ( !setAddressPort(address_port) ) {
#ifdef _SocketTcp_DEBUG_ENABLED
		Log(LOG_CRITICAL, ":connectTo(%s) FAILED Setting address", address_port.c_str() );
#endif
		return WRET_ERR_PARAM;
	 }
	return connect(timeout);
 }

WRESULT SocketTcp::connectTo( const std::string& address_ip, int port_value, unsigned long timeout  ) {
	if ( !setAddressPort(address_ip, port_value) ) {
#ifdef _SocketTcp_DEBUG_ENABLED
		Log(LOG_CRITICAL, ":connectTo(%s:%i) FAILED Setting address", address_ip.c_str(), port_value );
#endif
		return WRET_ERR_PARAM;
	 }
	return connect(timeout);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SocketTcp::setAddressPort( const std::string& add_value, int port_value ) {
	if ( isConnected() ) {
#ifdef _SocketTcp_DEBUG_ENABLED
		Log(LOG_CRITICAL, ":setAddressPort(%s, %i) Not permitted while Binded!", add_value.c_str(), port_value );
#endif
		return false;
	 }
	this->address = add_value;
	this->port = port_value;
	this->address_port = Utilities::format("%s:%i", this->address.c_str(), this->port);
#ifdef _SocketTcp_DEBUG_ENABLED
	Log(LOG_INFO, ":setAddressPort() set to '%s'", this->address_port.c_str() );
#endif
	return true;
 }

bool SocketTcp::setAddressPort( const std::string& value ) {
	if ( isConnected() ) {
#ifdef _SocketTcp_DEBUG_ENABLED
		Log(LOG_CRITICAL, ":setAddressPort(%s) Not permitted while Binded!", value.c_str() );
#endif
		return false;
	 }
	this->address = wosh::network::NetworkUtilities::getAddressFromDestination(value);
	this->port = wosh::network::NetworkUtilities::getPortFromDestination(value);
	this->address_port = value;
#ifdef _SocketTcp_DEBUG_ENABLED
	Log(LOG_INFO, ":setAddressPort() set to '%s'", this->address_port.c_str() );
#endif
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace network
}; // namespace wosh
