/**
 * @class   wosh::network::qt::SocketTcpQt
 * @brief   The class provides a TCP socket.
 *
 * @note    Introduce Rate-Control, see http://doc.trolltech.com/qq/qq17-ratecontrol.html
 ****************************************************************************
 * @version $Id: SocketTcpQt.h 2628 2010-06-08 23:30:40Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Network_Qt_SocketTcpQt_H__
 #define __WOSH_Network_Qt_SocketTcpQt_H__

 #include <framework/network/SocketTcp.h>

 #include <QtNetwork/QTcpSocket>


namespace wosh {
 namespace network {
  namespace qt {

//#define _SocketTcpQt_DEBUG_ENABLED

class ServerTcpQt;

class SocketTcpQt :		private QTcpSocket,
						public wosh::network::SocketTcp
 {
	Q_OBJECT
	WOSH_CLASSNAME(wosh::network::qt::SocketTcpQt)
	WOSH_CLASS_ABSTRACT(false)


	public:

		SocketTcpQt();

		virtual ~SocketTcpQt();

	public:

		using QTcpSocket::readAll;

	public:

		WRESULT connect( unsigned long timeout = LIMIT_ULONG_MAX );

		WRESULT disconnect( unsigned long timeout = LIMIT_ULONG_MAX );

		inline int64 read(char* data, int64 maxlen ) {
			return QTcpSocket::read(data, maxlen);
		 }

		int sendDatagram( const char* data, int size );

		bool waitWritten( unsigned long timeout = LIMIT_ULONG_MAX );

		bool waitResponse( char* data, int max_size, long& size, unsigned long timeout = LIMIT_ULONG_MAX );

		inline bool isConnected() const		{ return ( isOpen() && state() == QAbstractSocket::ConnectedState); }

		WRESULT setSocket( int socket_descriptor );

/** @name Getters
 * @{
 ******************************************************************************/
	public:
		std::string getPeerAddressPort() const;

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:

//@}
/** @name Internal vars
 * @{
 ******************************************************************************/
	protected:
		friend class wosh::network::qt::ServerTcpQt;

//@}
	private slots:
		void readClient();
		void socket_error( QAbstractSocket::SocketError socketError );


}; // class def

  }; // namespace qt
 }; // namespace network
}; // namespace wosh

#endif //__WOSH_Network_Qt_SocketTcpQt_H__
