/** @file    EncodersPlainText_Xtended.cpp
 *  @author  Alessandro Polo
 *  @version $Id: EncodersPlainText_Xtended.cpp 2697 2010-06-22 17:59:00Z alex $
 *  @brief
 * The header for this class can be found in EncodersPlainText_Xtended.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
 #include <framework/persistence/plaintext/EncodersPlainText_Xtended.h>

 #include <core/ObjectAllocator.h>


namespace wosh {
 namespace persistence {
  namespace plaintext {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 WOSH_Encoder_PlainText_BIND(wosh::persistence::plaintext::EncoderPlainText_WeatherInfo, "wosh::xtended::WeatherInfo", 0.0, _static_EncoderPlainText_WeatherInfo )

WRESULT EncoderPlainText_WeatherInfo::serializeTo( const wosh::xtended::WeatherInfo* object, std::ostream& output_stream, int encoder_options ) {
	if ( object == NULL ) return WRET_ERR_PARAM;
	if ( isCompact(encoder_options) ) {
		output_stream << "SourceID: " << object->getSourceID() << ", TimeStamp: " << object->getTimeStamp() << ", TTL: " << object->getTTL() << std::endl;
		output_stream << "Period: " << object->getPeriodName() << " [" << Utilities::getTimeStampUTF(object->getPeriodStart()) << " to " << Utilities::getTimeStampUTF(object->getPeriodEnd()) << "]" << std::endl;
		output_stream << "Condition: " << object->getConditionKey() << " [" << object->getCondition() << "," << object->getConditionExtra() << "]" << std::endl;
		output_stream << "Temperature: " << object->getTemperature() << " [min:" << object->getTemperatureLowest() << ", max:" << object->getTemperatureHighest() << "]" << std::endl;
		output_stream << "SunRise: " << Utilities::getTimeSpan(object->getSunRise()) << ", SunSet: " << Utilities::getTimeSpan(object->getSunSet()) << std::endl;
		output_stream << "Atmosphere Humidity: " << object->getAtmosphereHumidity() << 
						", Visibility: " << object->getAtmosphereVisibility() << 
						", Pressure: " << object->getAtmospherePressure() << 
						", Rising: " << object->getAtmosphereRising() << std::endl;

	 }
	else {
		output_stream << "SourceID: " << object->getSourceID() << std::endl;
		output_stream << "TimeStamp: " << object->getTimeStamp() << ", TTL: " << object->getTTL() << std::endl;
		output_stream << "Period: " << object->getPeriodName() << " [" << Utilities::getTimeStampUTF(object->getPeriodStart()) << " to " << Utilities::getTimeStampUTF(object->getPeriodEnd()) << "]" << std::endl;
		output_stream << "Condition: " << object->getConditionKey() << " [" << object->getCondition() << "," << object->getConditionExtra() << "]" << std::endl;
		output_stream << "Temperature: " << object->getTemperature() << " [min:" << object->getTemperatureLowest() << ", max:" << object->getTemperatureHighest() << "]" << std::endl;
		output_stream << "SunRise: " << Utilities::getTimeSpan(object->getSunRise()) << std::endl;
		output_stream << "SunSet: " << Utilities::getTimeSpan(object->getSunSet()) << std::endl;
		output_stream << "Wind: " << "-" << std::endl;
		output_stream << "AtmosphereHumidity: " << object->getAtmosphereHumidity() << std::endl;
		output_stream << "AtmosphereVisibility: " << object->getAtmosphereVisibility() << std::endl;
		output_stream << "AtmospherePressure: " << object->getAtmospherePressure() << std::endl;
		output_stream << "AtmosphereRising: " << object->getAtmosphereRising() << std::endl;

	 }
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

  }; // namespace plaintext
 }; // namespace persistence
}; // namespace wosh
