/**
 * @class   wosh::plugins::PluginLoader
 * @brief   
 * 
 *
 * @version $Id: PluginLoader.h 2488 2010-05-21 01:48:27Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR BusS;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/


#ifndef __WOSH_Plugins_PluginLoader_H__
 #define __WOSH_Plugins_PluginLoader_H__

 #include <core/ObjectAllocator.h>
 #include <core/LogContext.h>
 #include <core/FileSystem.h>

 #include <framework/plugins/PluginModule.h>


namespace wosh {
 namespace plugins {


class PluginLoader : public ObjectAllocator
 {
	WOSH_SERIALIZABLE(wosh::plugins::PluginLoader)

	public:
		PluginLoader();

		virtual ~PluginLoader();
	private:
		PluginLoader( const PluginLoader& ) : ObjectAllocator() { }

	public:

		WRESULT setLibraryFolder( const std::string& library_folder );

		WRESULT setPluginFileExtension( const std::string& plugin_file_extension );


		WRESULT preload( bool deep );

		const ListT<PluginModule*>& getModules() const { return this->modules; }
		ListT<PluginModule*>& getModules()				{ return this->modules; }

	public:

		virtual IReflection* createObject( const std::string& obj_type );

		virtual WRESULT destroyObject( const std::string& obj_uuid );


		virtual bool supportObject( const std::string& obj_type );

		virtual int getSupportedObjects( std::vector<std::string>& obj_list );


	protected:
		ListT<PluginModule*> modules;

		FilePath libFolder;
		std::string pluginExtension;

		wosh::LogContext Log;


}; // class def

 }; // namespace plugins
}; // namespace wosh

#endif //__WOSH_Plugins_PluginLoader_H__
