/**
 * @class   wosh::security::SecurityAuthenticationShell
 * @brief   

 *
 ****************************************************************************
 * @version $Id: SecurityAuthenticationShell.h 2732 2010-07-06 03:18:11Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Security_SecurityAuthenticationShell_H__
 #define __WOSH_Security_SecurityAuthenticationShell_H__

 #include <woshDefs.h>
 #include <core/SecurityToken.h>


namespace wosh {
 namespace persistence {
  namespace qt {
	class EncoderXml_Qt_Security_SecurityTokenShell;
  };
 };
 namespace security {


class SecurityTokenProviderShell;
class SecurityTokenAuthenticatorShell;

#define _SecurityTokenShell_VERSION		0.8399

class SecurityTokenShell : public SecurityToken
 {
	WOSH_SERIALIZABLE(wosh::security::SecurityTokenShell)

	public:
		SecurityTokenShell()
			: SecurityToken() { }

		SecurityTokenShell( const SecurityTokenShell& m )
			: SecurityToken(m) { }

		virtual ~SecurityTokenShell() { }

	public:
		inline double getVersion() const				{ return _SecurityTokenShell_VERSION; }
		inline SOURCE_ACCESS getSourceAccess() const	{ return SecurityToken::ACCESS_LOCALHOST; }

	protected:

		friend class wosh::security::SecurityTokenProviderShell;
		friend class wosh::security::SecurityTokenAuthenticatorShell;
		friend class wosh::persistence::qt::EncoderXml_Qt_Security_SecurityTokenShell;

}; // class def


class SecurityTokenProviderShell : public SecurityTokenProvider
 {
	WOSH_CLASSNAME(wosh::security::SecurityTokenProviderShell)
	WOSH_CLASS_ABSTRACT(false)
	WOSH_CLONE_CONSTRUCTOR(wosh::security::SecurityTokenProviderShell)


	public:
		SecurityTokenProviderShell();
		SecurityTokenProviderShell( const SecurityTokenProviderShell& m );
		virtual ~SecurityTokenProviderShell() { }

	public:
		inline double getVersion() const		{ return _SecurityTokenShell_VERSION; }

	protected:
		virtual SecurityToken* createToken() const;

	public:
		static WRESULT validateShell( std::string& wosh_user, std::string* os_user );

	
}; // class def

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class SecurityTokenAuthenticatorShell : public SecurityTokenAuthenticator
 {
	WOSH_CLASSNAME(wosh::security::SecurityTokenAuthenticatorShell)
	WOSH_CLASS_ABSTRACT(false)
//	WOSH_CLONE_CONSTRUCTOR(wosh::security::SecurityTokenAuthenticatorShell)

	public:
		SecurityTokenAuthenticatorShell();
//		SecurityTokenAuthenticatorShell( const SecurityTokenAuthenticatorShell& m );
		virtual ~SecurityTokenAuthenticatorShell() { }

	public:
		bool isSupported( const char* provider_classname, double version = 0.0 ) const;

		inline double getVersion() const		{ return _SecurityTokenShell_VERSION; }

	public:
		WRESULT validateToken( const SecurityToken* token ) const;

}; // class def


 }; // namespace security
}; // namespace wosh

#endif //__WOSH_Security_SecurityAuthenticationShell_H__
