/** @file    Metronome.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Metronome.cpp 2621 2010-06-04 12:23:48Z alex $
 *  @brief
 * File containing methods for the wosh::xtended::Metronome class.
 * The header for this class can be found in Metronome.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "Metronome.h"

 #include <core/ObjectAllocator.h>


using namespace std;
namespace wosh {
 namespace xtended {

 WOSH_REGISTER(wosh::xtended::Metronome, "wosh::Thread", _Metronome_VERSION, _static_Metronome )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

Metronome::Metronome()
	: ThreadImpl(NULL, "Metronome.thread") {
	this->period = 0;
	this->listener = NULL;
	this->disabled = false;
 }

Metronome::~Metronome() {

 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD

bool Metronome::initializingThread() {
	if ( this->listener == NULL ) {
		return false;
	 }
	return true;
 }

void Metronome::runThread() {
//	Log(LOG_VERBOSE, ":runThread(): STARTING" );
	this->configMux.lockForRead();
	unsigned long tPeriod = this->period;
	this->configMux.unLock();

	while(this->running) {
		this->configMux.lockForRead();
		tPeriod = this->period;
		if ( this->disabled || tPeriod == 0 ) {
			this->configMux.unLock();
			ThreadImpl::sleepForMSec(300);
			continue;
		 }
		this->configMux.unLock();
		ThreadImpl::sleepForMSec(tPeriod);
		this->listener->tickEvent( 0, this );
	 }
//	Log(LOG_VERBOSE, ":runThread(): EXITING" );
 }

////////////////////////////////////////////////////////////////////////////////////////////////// RUN THREAD
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Metronome::startMetronome( unsigned long synch_timeout ) {
	return ThreadImpl::startThread(synch_timeout);
 }

WRESULT Metronome::stopMetronome( unsigned long synch_timeout ) {
	return ThreadImpl::quitThread(synch_timeout);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// SETTERS

WRESULT Metronome::setListener( IMetronomeListener* ptr ) {
	if ( isThreadRunning() ) return WRET_ERR_ILLEGAL_USE;
	this->listener = ptr;
	return WRET_OK;
 }

void Metronome::setDisabled( bool value ) {
	MutexLockerWrite mL(&this->configMux);
	this->disabled = value;
 }

void Metronome::setPeriod( unsigned long millisec ) {
	MutexLockerWrite mL(&this->configMux);
	this->period = millisec;
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// SETTERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// GETTERS

unsigned long Metronome::getPeriod() const {
	MutexLockerRead mL(&this->configMux);
	return this->period;
}

bool Metronome::isDisabled() const {
	MutexLockerRead mL(&this->configMux);
	return this->disabled;
}

///////////////////////////////////////////////////////////////////////////////////////////////////// GETTERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace xtended
}; // namespace wosh
