/**
 * @class   wosh::xtended::WeatherInfo
 * @brief   
 *
 *
 *
 ****************************************************************************
 * @version 0.6.030 $Id: WeatherInfo.h 2814 2010-07-30 06:58:57Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Xtended_WeatherInfo_H__
 #define __WOSH_Xtended_WeatherInfo_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>
 #include <core/Fact.h>
 

namespace wosh {
 template <class T> class DataFieldCollector;
 template <class T> class IDataFieldWrapper;
 namespace xtended {

 #define _WeatherInfo_VERSION						0.12

 #define _WeatherInfo_TemperatureUndefined			-10000

 #define _WeatherInfo_EVENT_WeatherConditionUpdated	"WeatherConditionUpdated"


class WeatherInfo : public virtual ISerializable
 {
	WOSH_SERIALIZABLE(wosh::xtended::WeatherInfo)
	WOSH_COMPARE_RTTI(wosh::xtended::WeatherInfo)

	public:
		enum WEATHER_CONDITION {
			CONDITION_UNKNOWN			= 0x00,
			CONDITION_SUNNY				= 0x01,
			CONDITION_CLOUDY			= 0x02,
			CONDITION_RAIN				= 0x04,
			CONDITION_FOGGY				= 0x08,
			CONDITION_SNOW				= 0x10,
			CONDITION_THUNDERSTORMS		= 0x20
		 };

		enum WEATHER_CONDITION_XTRA {
			CONDITION_XTRA_NONE			= 0x00,
			CONDITION_XTRA_PARTLY		= 0x01,
			CONDITION_XTRA_SCATTERED	= 0x02,
			CONDITION_XTRA_SEVERE		= 0x04,
			CONDITION_XTRA_HEAVY		= 0x08
		 };

	public:
		/**
		 * @brief  Empty constructor.
		 */
		WeatherInfo() {
			clear();
		 }
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		WeatherInfo( const WeatherInfo& m ) {
			*this = m;
		 }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~WeatherInfo() { }

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		inline void setSourceID( int64 value )							{ this->source_id = value; }
		void setTimestamp( int64 ts = -1 );
		inline void setTTL( int64 seconds )								{ this->ttl = seconds; }
		inline void setReliability( double value )						{ this->reliability = value; }

		inline void setPeriod( int64 start, int64 stop )				{ this->period_start = start; this->period_end = stop; }
		inline void setPeriodStart( int64 value )						{ this->period_start = value; }
		inline void setPeriodEnd( int64 value )							{ this->period_end = value; }
		inline void setPeriodName( const std::string& value )			{ this->period_name = value; }
		void setPeriodDay( int64 start_ts );

		inline void setCondition( int condition, int condition_xtra )	{ this->condition = condition; this->condition_x = condition_xtra; }
		inline void setConditionKey( const std::string& value )			{ this->condition_key = value; }
		inline void setConditionSummary( const std::string& value )		{ this->condition_summary = value; }

		inline void setTemperature( int value )							{ this->temperature = value; }
		inline void setTemperatureHighest( int value )					{ this->temperature_highest = value; }
		inline void setTemperatureLowest( int value )					{ this->temperature_lowest = value; }

		inline void setWindChill( int value )							{ this->wind_chill = value; }
		inline void setWindDirection( int value )						{ this->wind_direction = value; }
		inline void setWindSpeed( double value )						{ this->wind_speed = value; }

		inline void setSunRise( long value )							{ this->sunrise = value; }
		inline void setSunSet( long value )								{ this->sunset = value; }

		inline void setAtmosphereHumidity( int value )					{ this->atmosphere_humidity = value; }
		inline void setAtmosphereVisibility( double value )				{ this->atmosphere_visibility = value; }
		inline void setAtmospherePressure( double value )				{ this->atmosphere_pressure = value; }
		inline void setAtmosphereRising( int value )					{ this->atmosphere_rising = value; }

		virtual void clear();
		virtual double merge( const WeatherInfo& other );

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		inline int64 getSourceID() const						{ return this->source_id; }
		inline int64 getTimeStamp() const						{ return this->timestamp; }
		inline int64 getTTL() const								{ return this->ttl; }
		inline double getReliability() const					{ return this->reliability; }

		inline const std::string& getPeriodName() const			{ return this->period_name; }
		inline int64 getPeriodStart() const						{ return this->period_start; }
		inline int64 getPeriodEnd() const						{ return this->period_end; }

		inline int getCondition() const							{ return this->condition; }
		inline int getConditionExtra() const					{ return this->condition_x; }
		inline const std::string& getConditionKey() const		{ return this->condition_key; }
		inline const std::string& getConditionSummary() const	{ return this->condition_summary; }

		inline int getTemperature() const						{ return this->temperature; }
		inline int getTemperatureHighest() const				{ return this->temperature_highest; }
		inline int getTemperatureLowest() const					{ return this->temperature_lowest; }

		inline int getWindChill() const							{ return this->wind_chill; }
		inline int getWindDirection() const						{ return this->wind_direction; }
		inline double getWindSpeed() const						{ return this->wind_speed; }

		inline long getSunRise() const							{ return this->sunrise; }
		inline long getSunSet() const							{ return this->sunset; }

		inline int getAtmosphereHumidity() const				{ return this->atmosphere_humidity; }
		inline double getAtmosphereVisibility() const			{ return this->atmosphere_visibility; }
		inline double getAtmospherePressure() const				{ return this->atmosphere_pressure; }
		inline int getAtmosphereRising() const					{ return this->atmosphere_rising; }

		virtual bool isEmpty() const;
		virtual bool isPeriodValid() const;


//		const string& getConditionIcon() const		{ return this->condition_icon; }
		virtual double evalSimilarity( const WeatherInfo& other ) const;

		static DataFieldCollector<WeatherInfo>* getDataFields();
		static IDataFieldWrapper<WeatherInfo>* getPrimaryKeyWrapper();

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual WeatherInfo& operator=(const WeatherInfo& m);
		virtual bool operator==(const WeatherInfo &other) const;
		bool operator!=(const WeatherInfo &other) const	{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const		{ return "WeatherInfo"; }
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}

	protected:
		int64 source_id;				///< 
		int64 timestamp;				///< (epoch) timestamp of the information
		int64 ttl;						///<
		double reliability;				///< 

		std::string period_name;		///< human_friendly name of the period (such as 'thus')
		int64 period_start;				///< (epoch) beginning of period
		int64 period_end;				///< (epoch) end of period 

	protected:
		int condition;					///< 
		int condition_x;				///< 
		std::string condition_key;		///< 
		std::string condition_summary;	///< textual summary of the condition

	protected:
		int temperature;
		int temperature_highest;		///< highest temperature in degrees celsius
		int temperature_lowest;			///< lowest temperature in degrees celsius

	protected:
		int wind_chill;
		int wind_direction;
		double wind_speed;

	protected:
		long sunrise;					///< seconds after midnight
		long sunset;					///< seconds after midnight

	protected:
		int atmosphere_humidity;
		double atmosphere_visibility;
		double atmosphere_pressure;
		int atmosphere_rising;

}; // class def






// EVENTS:
//  - current weather updated
//  - forecast updated







/**
 * @class   wosh::xtended::WeatherCondition_Updated
 * @brief   Raised when an user is added or updated.
 *
 * Internally, data is stored as wosh::List. Supports also wosh::Variant type.
 *
 * Fields are:
 *  - \c username [Variant::STRING] (always present, first item of the list)
 *  - \c action type (optional, default: new user)
 *
 * Some getters are provided for convenience:
 * \code

 * \endcode
 *
 * @see     wosh::Fact
 * @see     wosh::UserManager
 ****************************************************************************
 * @version 0.8.399 $Id: WeatherInfo.h 2814 2010-07-30 06:58:57Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
class WeatherCondition_Updated : public wosh::Fact
 {
	WOSH_SERIALIZABLE(wosh::xtended::WeatherCondition_Updated)

	public:
		/**
		 * @brief  Empty constructor.
		 */
		WeatherCondition_Updated()
			: wosh::Fact(_WeatherInfo_EVENT_WeatherConditionUpdated) { }
		/**
		 * @brief  Constructor with initializers.
		 * @param info [in] updated info
		 */
		WeatherCondition_Updated( wosh::xtended::WeatherInfo* info );

		WeatherCondition_Updated( wosh::xtended::WeatherInfo* info, double similarity );

		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		WeatherCondition_Updated( const WeatherCondition_Updated& m )
			: wosh::Fact(m) { }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~WeatherCondition_Updated() { }

	public:
		const wosh::xtended::WeatherInfo* getWeatherInfo() const;

		double getSimilarity() const;

}; // class def








 }; // namespace xtended
}; // namespace wosh

#endif //__WOSH_Xtended_WeatherInfo_H__
