/**
 * @class   wosh::interfaces::IBundle
 * @brief   Standard interface of generic Bundle.
 *
 * @see     wosh::interfaces::Service
 * @see     wosh::interfaces::Device
 * @see     wosh::Bundle
 * @ingroup Interfaces
 ****************************************************************************
 * @version 0.8.499 $Id: IBundle.h 2282 2010-04-07 06:20:24Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Interfaces_IBundle_H__
 #define __WOSH_Interfaces_IBundle_H__

 #include <woshDefs.h>
 #include <string.h>

namespace wosh {
 namespace interfaces {


#define WOSH_BUNDLE_INTERFACE( CLASSNAME )														\
		public: static const char* bundleType() { return CLASSNAME; }							\
		public: virtual inline const char* getBundleType() const { return bundleType(); }

#define WOSH_BUNDLE_SINGLETON( ALLOWED_ONSAME_KERNEL, ALLOWED_ONSAME_NETWORK )					\
		public: virtual int getAllowedOnSameKernel() const { return ALLOWED_ONSAME_KERNEL; }	\
		public: virtual int getAllowedOnSameNetwork() const { return ALLOWED_ONSAME_NETWORK; }

class IBundle
 {
/** @name IBundle Interface
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief  Get the classname of the interface, similar to IReflection::getClassName()
		 * @return the full class name (including namespaces) of the class
		 */
		virtual const char* getBundleType() const = 0;

		/**
		 * @brief  Get how many instances of the bundle are allowed on same host (kernel).
		 *         Evaluated by wosh::BundleManager during loading/requirements-validation.
		 * @return maximum number of instances; 0 means any number
		 */
		virtual int getAllowedOnSameKernel() const = 0;
		/**
		 * @brief  Get how many instances of the bundle are allowed on same (WOSH) network.
		 *         Evaluated by wosh::BundleManager during loading/requirements-validation.
		 * @return maximum number of instances; 0 means any number
		 */
		virtual int getAllowedOnSameNetwork() const = 0;

//@}
	public:
		virtual ~IBundle() { }

}; // class def


 }; // namespace interfaces
}; // namespace wosh

#endif //__WOSH_Interfaces_IBundle_H__
