/**
 * @file    interfaces/interfaces.h
 * @brief   Include all interfaces
 ****************************************************************************
 * @namespace wosh::interfaces
 * @brief     Contains standard interfaces for wosh::Bundle implementations (devices and services).
 *
 * WOSH Bundles may interact and refer each other only through these interfaces.
 *
 * This model allows also multiple implementation on the same service/device, 
 * by different authors or for specific underlying sw/hw.
 *
 * @see     \ref page_core_bundles, wosh::interfaces::services
 * @see     \ref page_core_devices, wosh::interfaces::devices
 ****************************************************************************
 * \defgroup Interfaces WOSH Interfaces
 *
 * This page groups all interfaces of WOSH system, they implement
 * a sub-class of wosh::interfaces::IBundle.
 * 
 * Refer to wosh::interfaces::services and wosh::interfaces::devices.
 *
 * @see     wosh::interfaces
 * @see     \ref page_core_devices, \ref page_core_bundles
 ****************************************************************************
 * @version 0.8.499 $Id: interfaces.h 2289 2010-04-09 09:14:03Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Interfaces_H__
 #define __WOSH_Interfaces_H__

 #include <woshDefs.h>

 namespace wosh {
  namespace interfaces { };
 };

 #include <interfaces/devices.h>
 #include <interfaces/services.h>


#endif //__WOSH_Interfaces_H__
