/**
 * @file    services_registration.cpp
 * @brief   Register interfaces of Services on dynamic-initialization
 *          (see ObjectAllocator and MACROS for details).
 *
 ****************************************************************************
 * @version 0.8.499 $Id: services_registration.cpp 2804 2010-07-27 04:39:07Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <interfaces/Service.h>
 #include <core/ObjectAllocator.h>

 #include <interfaces/services/Alarm.h>
 #include <interfaces/services/BuildingManager.h>
 #include <interfaces/services/Butler.h>
 #include <interfaces/services/CommunicatorService.h>
 #include <interfaces/services/CommunicationManager.h>
 #include <interfaces/services/DesktopNotifier.h>
 #include <interfaces/services/Discovery.h>
 #include <interfaces/services/DiscoveryUdp.h>
 #include <interfaces/services/Gardener.h>
 #include <interfaces/services/Heyu.h>
 #include <interfaces/services/Man.h>
 #include <interfaces/services/MediaDirector.h>
 #include <interfaces/services/SystemServices.h>
 #include <interfaces/services/TaskManager.h>
 #include <interfaces/services/Weather.h>


namespace wosh {
 namespace interfaces {
  namespace services {

  WOSH_REGISTER_INTERFACE(wosh::interfaces::Service, "wosh::interfaces::IBundle", 1.00, _static_Service )

  // Alarm Manager
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::Alarm, "wosh::interfaces::Service", 1.00, _static_Alarm )
  // Building Manager
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::BuildingManager, "wosh::interfaces::Service", 1.00, _static_BuildingManager )
  // Butler
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::Butler, "wosh::interfaces::Service", 1.00, _static_Butler )
  // Communication Manager
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::CommunicationManager, "wosh::interfaces::Service", 1.00, _static_CommunicationManager )
  // CommunicatorService
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::CommunicatorService, "wosh::interfaces::Service", 1.00, _static_CommunicatorService )
  // Desktop Notifier
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::DesktopNotifier, "wosh::interfaces::Service", 1.00, _static_DesktopNotifier )
  // Discovery and DiscvoveryUDP
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::Discovery, "wosh::interfaces::Service", 1.00, _static_Discovery )
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::DiscoveryUdp, "wosh::interfaces::services::Discovery", 1.00, _static_DiscoveryUdp )
  // Gardener
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::Gardener, "wosh::interfaces::Service", 1.00, _static_Gardener )
  // Heyu
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::Heyu, "wosh::interfaces::Service", 1.00, _static_Heyu )
  // InterpreterService
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::InterpreterService, "wosh::interfaces::Service", 1.00, _static_InterpreterService )
  // Man
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::Man, "wosh::interfaces::Service", 1.00, _static_Man )
  // Media Director
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::MediaDirector, "wosh::interfaces::Service", 1.00, _static_MediaDirector )
  // System Services
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::SystemServices, "wosh::interfaces::Service", 1.00, _static_SystemServices )
  // Task Manager
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::TaskManager, "wosh::interfaces::Service", 1.00, _static_TaskManager )
  // Weather
  WOSH_REGISTER_INTERFACE(wosh::interfaces::services::Weather, "wosh::interfaces::Service", 1.00, _static_Weather )


  }; // namespace services
 }; // namespace interfaces
}; // namespace wosh
